/*
 * Copyright (c) 2018 Mellanox Technologies LTD. ALL RIGHTS RESERVED.
 * See file LICENSE for terms.
 */

#ifndef THREAD_POOL_H
#define THREAD_POOL_H

#include <pthread.h>
#include <queue>
#include <list>

#include "ar_mgr_types.h"
#include "thread_pool_task.h"

#define DEFAULT_THREADPOOL_THREADS_NUMBER 10

typedef std::queue< class ThreadPoolTask * > TaskQueue;
typedef std::list< pthread_t > ListThreads;

class ThreadPool {

    osm_log_t          *m_osm_log_;           /** Pointer to osm log */
    TaskQueue           m_task_queue_;
    ListThreads         m_threads_;
    bool                m_stop_;
    bool                m_init_;

    pthread_mutex_t     m_queue_lock_;
    pthread_cond_t      m_queue_cond_;


public:

    ThreadPool(osm_log_t *p_osm_log) :
        m_osm_log_(p_osm_log), m_stop_(false), m_init_(false) {}

    int Init(uint16_t num_threads);

    void Stop() { m_stop_ = true; }
    void AddTask(ThreadPoolTask *p_task);

    //the method executed by each thread
    void ThreadRun();

    ~ThreadPool();

};

class ThreadPoolTasksCollection {

private:

    uint16_t                m_num_tasks_in_progress_;
    pthread_mutex_t         m_tasks_lock_;
    pthread_cond_t          m_tasks_cond_;
protected:
    bool                    m_is_init_;
    osm_log_t               *m_osm_log_;           /** Pointer to osm log */

public:

    ThreadPoolTasksCollection(osm_log_t *p_osm_log) :
        m_num_tasks_in_progress_(0),
        m_is_init_(false),
        m_osm_log_(p_osm_log){}

    ~ThreadPoolTasksCollection();

    void Init();

    void AddTaskToThreadPool(ThreadPool *p_thread_pool,
                             ThreadPoolTask *p_task);

    void WaitForTasks();

    //this should be called before each tasks ends
    void OnTaskEnd();

    osm_log_t *GetOsmLog() { return m_osm_log_; }

};

#endif //THREAD_POOL_H
