/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2019, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */

#ifndef _AR_ALGORITHM_H_
#define _AR_ALGORITHM_H_

/* osm_config.h needs to be before all other includes */
#include <opensm/osm_config.h>

#include "ar_mgr_types.h"

#define MAX_SL_NUMBER 16
#define MAX_OP_VL_CODE 5

struct AdaptiveRoutingClbck;
class AdaptiveRoutingManager;

struct LidMapping {
    u_int16_t m_lid_to_sw_lid_mapping[IB_LID_UCAST_END_HO + 1];
    u_int16_t m_lid_to_base_lid_mapping[IB_LID_UCAST_END_HO + 1];

    LidMapping() {
        memset(m_lid_to_sw_lid_mapping, 0, sizeof(m_lid_to_sw_lid_mapping));
        memset(m_lid_to_base_lid_mapping, 0, sizeof(m_lid_to_base_lid_mapping));
    }
};

struct PlftDefInfo {
	 uint8_t lid_offset;
	 uint8_t lid_space;
	 uint8_t table_idx;
};

class AdaptiveRoutingAlgorithm {
protected:
    osm_log_t              *m_p_osm_log; // Pointer to osm log
    GuidToSWDataBaseEntry  &m_sw_map_;
    AdaptiveRoutingManager &m_ar_mgr_;

    supported_feature_t m_algorithm_feature_;

public:
    AdaptiveRoutingAlgorithm(osm_log_t *p_osm_log,
                             GuidToSWDataBaseEntry &sw_map,
                             AdaptiveRoutingManager &ar_mgr,
                             supported_feature_t algorithm_feature) :
        m_p_osm_log(p_osm_log), m_sw_map_(sw_map),
        m_ar_mgr_(ar_mgr), m_algorithm_feature_(algorithm_feature) {}

    virtual ~AdaptiveRoutingAlgorithm() {}

    virtual ar_algorithm_t GetAlgorithm() = 0;
    virtual int Preprocess() = 0;
    virtual int RunCycle() = 0;

    virtual int Init() = 0;
    virtual void Clear() = 0;

    osm_log_t *GetLog() { return m_p_osm_log; }

    int BuildLidMapping(LidMapping &lid_mapping);

    virtual ThreadPoolTask *GetCalculatePortGroupsTask(
        ARSWDataBaseEntry &db_entry) = 0;

    virtual void ClearAlgorithmConfiguration() = 0;

    virtual void CalculateSwitchPortGroups(ARSWDataBaseEntry &sw_db_entry,
                                           LidMapping *p_lid_mapping) = 0;

    //Remove on SM implementation
    virtual void UpdateVlidsLfts() = 0;

protected:

    bool IsFeatureActive(ARSWDataBaseEntry &sw_db_entry,
                         supported_feature_t m_feature)
    {
        return ((sw_db_entry.m_support[SUPPORT_AR] == SUPPORTED) &&
                (sw_db_entry.m_support[m_feature] == SUPPORTED) &&
                (sw_db_entry.m_option_on));
    }

    void UpdateCalculatedArGroupTable(
        SMP_ARGroupTable calculated_ar_group_table[AR_GROUP_TABLE_NUM_BLOCKS],
        u_int16_t group_number,
        PortsBitset &group_bitmask,
        unsigned &calculated_max_pg_id);

    int GroupTableProcess();

private:
    int SetHcaLidMapping(osm_physp_t *p_hca_physp,
                         osm_node_t *p_remote_sw_node,
                         LidMapping &lid_mapping);
};

#endif //_AR_ALGORITHM_H_
