/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2011, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */


#ifndef AR_CLBCK_H_
#define AR_CLBCK_H_

/* osm_config.h needs to be before all other includes */
#include <opensm/osm_config.h>
#include <typeinfo>
#include <string>
#include <stack>
using namespace std;

#include <opensm/osm_opensm.h>
#include <opensm/osm_helper.h>
#include <opensm/osm_log.h>
#include <opensm/osm_event_plugin.h>
#include <vendor/osm_vendor_api.h>

#include "ar_mgr_types.h"

void GetARInfoClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void GetRequiredARInfoCapClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetARInfoClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetARGroupTableClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void ARCopyGroupTableClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetARLFTTableClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetRNGenStringClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetRNGenBySubGroupPriorityClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetRNRcvStringClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetRNXmitPortMaskClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void GetPrivateLFTInfoClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetPrivateLFTInfoClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetPrivateLFTDefClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetPrivateLFTMapClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetPortSlToPLFTMapClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetVL2VLMapClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetSL2VLMapOnHostsClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void GetExtendedSwitchInfoClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

void SetExtendedSwitchInfoClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

//!!! TAMIR add to OSM
void GetPrivateLFTInfoKdorClbckDlg(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

enum calback_errcnt_t {
    AR_CLBCK_GET_PLFT_INFO,
    AR_CLBCK_SET_PLFT_INFO,
    AR_CLBCK_SET_PLFT_DEF,
    AR_CLBCK_SET_PLFT_MAP,
    AR_CLBCK_SET_PORT_SL_TO_PLFT_MAP,
    AR_CLBCK_SET_VL_TO_VL_MAP,
    AR_CLBCK_GET_EXT_SW_INFO,
    AR_CLBCK_SET_EXT_SW_INFO,
    AR_CLBCK_GET_AR_INFO,
    AR_CLBCK_GET_AR_INFO_CAP,
    AR_CLBCK_SET_AR_INFO,
    AR_CLBCK_SET_AR_GROUP_TABLE,
    AR_CLBCK_SET_AR_LFT_TABLE,
    AR_CLBCK_SET_RN_GEN_STRING,
    AR_CLBCK_SET_RN_GEN_BY_SG_PRIORITY,
    AR_CLBCK_SET_RN_RCV_STRING,
    AR_CLBCK_SET_RN_GEN_PORT_MASK,
    AR_CLBCK_LAST
};


struct AdaptiveRoutingClbck {

    ////////////////////
    //members
    /////////////////////
    osm_log_t           *m_p_osm_log;               /** Pointer to osm log */
    u_int16_t            *m_p_max_timeout_num; /** Pointer to max_timeout_num */

    int                 m_errcnt[AR_CLBCK_LAST];
    bool                m_is_temporary_error;        /** Indicates temporary error occurred
                                                       * (will be solved after heavy sweep) */

    supported_feature_t m_algorithm_feature;

    ////////////////////
    //methods
    ////////////////////
    AdaptiveRoutingClbck(osm_log_t *p_osm_log):
        m_p_osm_log(p_osm_log),
        m_is_temporary_error(false) {
        memset (m_errcnt, 0, sizeof(m_errcnt));
    }

    void ResetErrorCount() {
        memset (m_errcnt, 0, sizeof(m_errcnt));
    }

    void SetMaxTimeoutNum(u_int16_t *p_max_timeout_num) {
        m_p_max_timeout_num = p_max_timeout_num;
    }

    void HandleError(
        int rec_status,
        calback_errcnt_t err_type,
        supported_feature_t feature,
        ARSWDataBaseEntry *p_sw_entry);

    void GetARInfoClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void GetRequiredARInfoCapClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void SetARInfoClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void SetARGroupTableClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void ARCopyGroupTableClbck(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

    void SetARLFTTableClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void SetRNGenStringClbck(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

    void SetRNGenBySubGroupPriorityClbck(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

    void SetRNRcvStringClbck(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

    void SetRNXmitPortMaskClbck(
        const clbck_data_t &clbck_data,
        int rec_status,
        void *p_attribute_data);

    void GetPrivateLFTInfoClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void SetPrivateLFTInfoClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void SetPrivateLFTDefClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void SetPrivateLFTMapClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void SetPortSlToPLFTMapClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void SetVL2VLMapClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void SetSL2VLMapOnHostsClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void GetExtendedSwitchInfoClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    void SetExtendedSwitchInfoClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

    //!!! TAMIR add to OSM
    void GetPrivateLFTInfoKdorClbck(
            const clbck_data_t &clbck_data,
            int rec_status,
            void *p_attribute_data);

};



#endif          /* AR_CLBCK_H_ */

