/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2011, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */

%{
    /*
     * header section
     */
    #include <stdio.h>
    #include <stdlib.h>
    #include <string.h>

    #ifndef alloca
        #define alloca malloc
    #endif
    #define YYERROR_VERBOSE

    extern int yyerror(const char *msg);
    extern int ar_conf_lex(void);
    extern int ar_conf_lineno;
    extern char *ar_conf_text;
    extern void ar_conf_restart(FILE *input_file);
    extern int ar_conf_lex_destroy(void);

    #include "ar_mgr_types.h"
    extern struct MasterDataBase ar_conf_general_opt_db;
    extern map < u_int64_t, map < string, string > > ar_conf_sw_opt_db;
%}


%union {
    u_int64_t       ival;
    bool            bval;
    char            sval[1024];
}

%token          tk_opt_ar_mode
%token          tk_opt_ar_algorithm
%token          tk_opt_en_sl_mask
%token          tk_opt_dfp_en_vl_mask
%token          tk_opt_disable_tr_mask
%token          tk_opt_op_mode
%token          tk_opt_max_cas_on_spine
%token          tk_opt_max_timeout
%token          tk_opt_enable
%token          tk_opt_ar_enable
%token          tk_opt_arn_enable
%token          tk_opt_flfr_enable
%token          tk_opt_flfr_remote_disable
%token          tk_opt_error_window
%token          tk_opt_max_errors
%token          tk_opt_ageing
%token          tk_opt_log_size
%token          tk_opt_log_file
%token          tk_opcode_switch
%token          tk_ar_mode
%token          tk_ar_algorithm
%token          tk_left_bracket
%token          tk_right_bracket
%token          tk_colon
%token          tk_semicolon
%token          tk_comma
%token          tk_integer
%token          tk_bool
%token          tk_name
%token          tk_error

%type<sval>     tk_ar_mode
%type<sval>     tk_ar_algorithm
%type<ival>     tk_integer
%type<bval>     tk_bool
%type<sval>     tk_name
%left           tk_semicolon


%{
static int arConfErr = 0;
map < string, string > curr_map_sw_opt;

#ifdef DEBUG
    #define AR_CONF_DEBUG(fmt, ...) printf(fmt, ## __VA_ARGS__);
#else
    #define AR_CONF_DEBUG(fmt, ...)
#endif
%}


%%
S : SECTIONS_LIST {
    AR_CONF_DEBUG("Parsed all the file\n");
};
SECTIONS_LIST : SECTIONS_LIST SECTION {
    AR_CONF_DEBUG("Parsed SECTIONS_LIST\n");
};
SECTIONS_LIST : {
    AR_CONF_DEBUG("Parsed SECTIONS_LIST epselon\n");
};
SECTION : GENERAL_OPTION | SWITCH_SECTION {
    AR_CONF_DEBUG("Parsed SECTION epselon\n");
}

GENERAL_OPTION : GENERAL_OPTION_AR_MODE | GENERAL_OPTION_AGEING |
    GENERAL_OPTION_LOG_FILE | GENERAL_OPTION_LOG_SIZE |
    GENERAL_OPTION_MAX_ERRORS | GENERAL_OPTION_ERROR_WINDOW |
    GENERAL_OPTION_ENABLE |
    GENERAL_OPTION_AR_ENABLE | GENERAL_OPTION_ARN_ENABLE |
    GENERAL_OPTION_FLFR_ENABLE | GENERAL_OPTION_FLFR_REMOTE_DISABLE |
    GENERAL_OPTION_ALGORITHM |
    GENERAL_OPTION_EN_SL_MASK | GENERAL_OPTION_DFP_EN_VL_MASK |
    GENERAL_OPTION_DISABLE_TR_MASK |
    GENERAL_OPTION_OP_MODE | GENERAL_OPTION_MAX_CAS_ON_SPINE |
    GENERAL_OPTION_MAX_TIMEOUT {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION epselon\n");
}
GENERAL_OPTION_AR_MODE : tk_opt_ar_mode tk_colon tk_ar_mode tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_AR_MODE %s\n", $3);
    if (strcmp($3,  "FREE") == 0) {
        ar_conf_general_opt_db.m_ar_mode = AR_IB_LID_STATE_FREE;
    } else if (strcmp($3,  "BOUNDED") == 0) {
        ar_conf_general_opt_db.m_ar_mode = AR_IB_LID_STATE_BOUNDED;
    }
    //else we will not chagne nothing and we will take default
}
GENERAL_OPTION_AGEING : tk_opt_ageing tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_AGEING " U64D_FMT "\n", $3);
    ar_conf_general_opt_db.m_ageing_time = $3;
}
GENERAL_OPTION_LOG_FILE : tk_opt_log_file tk_colon tk_name tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_LOG_FILE %s\n", $3);
    ar_conf_general_opt_db.m_ar_log_file_name = $3;
}
GENERAL_OPTION_LOG_SIZE : tk_opt_log_size tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_LOG_SIZE " U64D_FMT "\n", $3);
    ar_conf_general_opt_db.m_ar_log_file_size = $3;
}
GENERAL_OPTION_MAX_ERRORS : tk_opt_max_errors tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_MAX_ERRORS " U64D_FMT "\n", $3);
    ar_conf_general_opt_db.m_max_errors = $3;
}
GENERAL_OPTION_ERROR_WINDOW : tk_opt_error_window tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_ERROR_WINDOW " U64D_FMT "\n", $3);
    ar_conf_general_opt_db.m_error_window = $3;
}
GENERAL_OPTION_ENABLE : tk_opt_enable tk_colon tk_bool tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_ENABLE %u\n", $3);
    ar_conf_general_opt_db.m_enable = $3;
}
GENERAL_OPTION_AR_ENABLE : tk_opt_ar_enable tk_colon tk_bool tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_AR_ENABLE %u\n", $3);
    ar_conf_general_opt_db.m_ar_enable = $3;
}
GENERAL_OPTION_ARN_ENABLE : tk_opt_arn_enable tk_colon tk_bool tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_ARN_ENABLE %u\n", $3);
    ar_conf_general_opt_db.m_arn_enable = $3;
}
GENERAL_OPTION_FLFR_ENABLE : tk_opt_flfr_enable tk_colon tk_bool tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_FLFR_ENABLE %u\n", $3);
    ar_conf_general_opt_db.m_flfr_enable = $3;
}
GENERAL_OPTION_FLFR_REMOTE_DISABLE : tk_opt_flfr_remote_disable tk_colon tk_bool tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_FLFR_REMOTE_DISABLE %u\n", $3);
    ar_conf_general_opt_db.m_flfr_remote_disable = $3;
}
GENERAL_OPTION_ALGORITHM : tk_opt_ar_algorithm tk_colon tk_ar_algorithm tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_ALGORITHM %u\n", $3);
    ar_conf_general_opt_db.m_ar_algorithm_str = $3;
}
GENERAL_OPTION_EN_SL_MASK : tk_opt_en_sl_mask tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_EN_SL_MASK %u\n", $3);
    ar_conf_general_opt_db.m_en_sl_mask = $3;
}
GENERAL_OPTION_DFP_EN_VL_MASK : tk_opt_dfp_en_vl_mask tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_DFP_EN_VL_MASK %u\n", $3);
    ar_conf_general_opt_db.m_dfp_en_vl_mask = $3;
}
GENERAL_OPTION_DISABLE_TR_MASK : tk_opt_disable_tr_mask tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_DISABLE_TR_MASK %u\n", $3);
    ar_conf_general_opt_db.m_disable_tr_mask = $3;
}
GENERAL_OPTION_OP_MODE : tk_opt_op_mode tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_OP_MODE " U64H_FMT "\n", $3);
    ar_conf_general_opt_db.m_op_mode = $3;
}
GENERAL_OPTION_MAX_CAS_ON_SPINE : tk_opt_max_cas_on_spine tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_MAX_CAS_ON_SPINE %u \n", $3);
    ar_conf_general_opt_db.m_max_cas_on_spine = $3;
}
GENERAL_OPTION_MAX_TIMEOUT : tk_opt_max_timeout tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed GENERAL_OPTION_MAX_TIMEOUT " U64H_FMT "\n", $3);
    ar_conf_general_opt_db.m_max_timeout_num = $3;
}

SWITCH_SECTION :  tk_opcode_switch tk_integer tk_left_bracket SWITCH_OPTIONS_LIST tk_right_bracket {
    AR_CONF_DEBUG("Parsed SWITCH_SECTION, guid = " U64H_FMT "\n", $2);
    ar_conf_sw_opt_db[$2] = curr_map_sw_opt;
    curr_map_sw_opt.clear();
}
SWITCH_OPTIONS_LIST : SWITCH_OPTIONS_LIST SWITCH_OPTION {
    AR_CONF_DEBUG("Parsed SWITCH_OPTIONS_LIST\n");
}
SWITCH_OPTIONS_LIST : {
    AR_CONF_DEBUG("Parsed SWITCH_OPTIONS_LIST epselon\n");
}
SWITCH_OPTION : SWITCH_OPTION_ENABLE | SWITCH_OPTION_AGEING {
    AR_CONF_DEBUG("Parsed SWITCH_OPTION\n");
}
SWITCH_OPTION_ENABLE : tk_opt_enable tk_colon tk_bool tk_semicolon {
    AR_CONF_DEBUG("Parsed SWITCH_OPTION_ENABLE %u\n", $3);
    curr_map_sw_opt[AR_CFG_SW_OPT_STR_ENABLE] = $3 ? "TRUE" : "FALSE";
}
SWITCH_OPTION_AGEING : tk_opt_ageing tk_colon tk_integer tk_semicolon {
    AR_CONF_DEBUG("Parsed SWITCH_AGEING " U64D_FMT "\n", $3);
    char buff[1024];
    sprintf(buff, U64D_FMT, $3);
    curr_map_sw_opt[AR_CFG_SW_OPT_STR_AGEING] = buff;
}


%%


/*
 * yyerror function
 */
#define AR_CONF_MAX(a, b)	((a) > (b) ? (a) : (b))

int yyerror(const char *msg)
{
    printf("AR Manager - conf file syntax error in line: %d (or between lines: %d-%d) " \
            "- unexpected or illegal token: <%s>\n, (Error description = %s)\n",
        ar_conf_lineno, AR_CONF_MAX(0, ar_conf_lineno - 1),
        ar_conf_lineno, ar_conf_text,
        msg);
    arConfErr = 1;
    return 1;
}


/*
 * parse configuration
 */
int arParseConfFile(const char *fileName)
{
    extern FILE * ar_conf_in;

    /*
     * open the file
     */
    ar_conf_in = fopen(fileName,"r");
    if (!ar_conf_in) {
        printf("-E- AR Manager - Failed to Open File:%s\n", fileName);
        return(1);
    }

    AR_CONF_DEBUG("-I- AR Manager - Parsing:%s\n", fileName);
    arConfErr = 0;
    /*
     * parse it
     */
    yyparse();

    ar_conf_restart(ar_conf_in);
    fclose(ar_conf_in);
    ar_conf_lex_destroy();
    return(arConfErr);
}
