/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2011, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */


%{
    #include <stdio.h>
    #include <stdlib.h>
    #include <string.h>
    #include <ctype.h>   
    
    #include "generated_code/ar_conf_parser.hh"

    /*
     * inorder to compile in windows
     */
    #define YY_NEVER_INTERACTIVE 1
%}


OPCODE_SWITCH               [sS][wW][iI][tT][cC][hH]
OPT_AR_MODE                 [aA][rR][_][mM][oO][dD][eE]
OPT_AR_ALGORITHM            [aA][rR][_][aA][lL][gG][oO][rR][iI][tT][hH][mM]
OPT_EN_SL_MASK              [eE][nN]_[sS][lL]_[mM][aA][sS][kK]
OPT_DFP_EN_VL_MASK          [dD][fF][pP]_[eE][nN]_[vV][lL]_[mM][aA][sS][kK]
OPT_DISABLE_TR_MASK         [dD][iI][sS][aA][bB][lL][eE]_[tT][rR]_[mM][aA][sS][kK]
OPT_OP_MODE                 [oO][pP][_][mM][oO][dD][eE]
OPT_MAX_CAS_ON_SPINE        [mM][aA][xX][_][cC][aA][sS][_][oO][nN][_][sS][pP][iI][nN][eE]
OPT_MAX_TIMEOUT             [mM][aA][xX][_][tT][iI][mM][eE][oO][uU][tT]
OPT_ENABLE                  [eE][nN][aA][bB][lL][eE]
OPT_AR_ENABLE               [aA][rR]_[eE][nN][aA][bB][lL][eE]
OPT_ARN_ENABLE              [aA][rR][nN]_[eE][nN][aA][bB][lL][eE]
OPT_FLFR_ENABLE             [fF][lL][fF][rR]_[eE][nN][aA][bB][lL][eE]
OPT_FLFR_REMOTE_DISABLE     [fF][lL][fF][rR]_[rR][eE][mM][oO][tT][eE]_[dD][iI][sS][aA][bB][lL][eE]
OPT_ERROR_WINDOW            [eE][rR][rR][oO][rR][_][wW][iI][nN][dD][oO][wW]
OPT_MAX_ERRORS              [mM][aA][xX][_][eE][rR][rR][oO][rR][sS]
OPT_AGEING                  [aA][gG][eE][iI][nN][gG][_][tT][iI][mM][eE]
OPT_LOG_SIZE                [lL][oO][gG][_][sS][iI][zZ][eE]
OPT_LOG_FILE                [lL][oO][gG][_][fF][iI][lL][eE]
LEFT_BRACKET                \{
RIGHT_BRACKET               \}
MODE_BOUNDED                [bB][oO][uU][nN][dD][eE][dD]
MODE_FREE                   [fF][rR][eE][eE]
AR_MODE                     ({MODE_BOUNDED}|{MODE_FREE})
LAG_ALGORITHM               [lL][aA][gG]
TREE_ALGORITHM              [tT][rR][eE][eE]
DF_PLUS_ALGORITHM           [dD][fF]_[pP][lL][uU][sS]
KDOR_HC_ALGORITHM           [hH][cC]
AR_ALGORITHM                ({LAG_ALGORITHM}|{TREE_ALGORITHM}|{DF_PLUS_ALGORITHM}|{KDOR_HC_ALGORITHM})
COMMENT                     #(.)*
COLON                       :
SEMICOLON                   ;
DIGIT                       [0-9]
HEX_DIGIT                   [A-Fa-f]
ANY_DIGIT                   ({DIGIT}|{HEX_DIGIT})
INTEGER                     (0x|0X)({ANY_DIGIT}+)|{DIGIT}+
BOOL_TRUE                   [tT][rR][uU][eE]
BOOL_FALSE                  [fF][aA][lL][sS][eE]
BOOL                        ({BOOL_TRUE}|{BOOL_FALSE})
NAME                        [a-zA-Z0-9_/.]+

%option never-interactive
%option yylineno
%option noyywrap
%option nounput
%option noinput


%{
#ifdef DEBUG
    #define AR_CONF_DEBUG(fmt, ...) printf(fmt, ## __VA_ARGS__);
#else
    #define AR_CONF_DEBUG(fmt, ...)
#endif
    

void toUpperCase(char str[])
{
    for (unsigned i = 0; i < strlen(str); ++i)
        str[i] = toupper(str[i]);
    return;
}
%}


%%
{OPCODE_SWITCH} {
    AR_CONF_DEBUG("token = OPCODE_SWITCH (%s)\n", yytext);
    return tk_opcode_switch;
}

{OPT_AR_MODE} {
    AR_CONF_DEBUG("token = OPT_AR_MODE (%s)\n", yytext);
    return tk_opt_ar_mode;
}

{OPT_AR_ALGORITHM} {
    AR_CONF_DEBUG("token = OPT_AR_ALGORITHM (%s)\n", yytext);
    return tk_opt_ar_algorithm;
}

{OPT_EN_SL_MASK} {
    AR_CONF_DEBUG("token = OPT_EN_SL_MASK (%s)\n", yytext);
    return tk_opt_en_sl_mask;
}

{OPT_DFP_EN_VL_MASK} {
    AR_CONF_DEBUG("token = OPT_DFP_EN_VL_MASK (%s)\n", yytext);
    return tk_opt_dfp_en_vl_mask;
}

{OPT_DISABLE_TR_MASK} {
    AR_CONF_DEBUG("token = OPT_DISABLE_TR_MASK (%s)\n", yytext);
    return tk_opt_disable_tr_mask;
}

{OPT_OP_MODE} {
    AR_CONF_DEBUG("token = OPT_OP_MODE (%s)\n", yytext);
    return tk_opt_op_mode;
}

{OPT_MAX_CAS_ON_SPINE} {
    AR_CONF_DEBUG("token = OPT_MAX_CAS_ON_SPINE (%s)\n", yytext);
    return tk_opt_max_cas_on_spine;
}

{OPT_MAX_TIMEOUT} {
    AR_CONF_DEBUG("token = OPT_MAX_TIMEOUT (%s)\n", yytext);
    return tk_opt_max_timeout;
}

{OPT_ENABLE} {
    AR_CONF_DEBUG("token = OPT_ENABLE (%s)\n", yytext);
    return tk_opt_enable;
}

{OPT_AR_ENABLE} {
    AR_CONF_DEBUG("token = OPT_AR_ENABLE (%s)\n", yytext);
    return tk_opt_ar_enable;
}

{OPT_ARN_ENABLE} {
    AR_CONF_DEBUG("token = OPT_ARN_ENABLE (%s)\n", yytext);
    return tk_opt_arn_enable;
}

{OPT_FLFR_ENABLE} {
    AR_CONF_DEBUG("token = OPT_FLFR_ENABLE (%s)\n", yytext);
    return tk_opt_flfr_enable;
}

{OPT_FLFR_REMOTE_DISABLE} {
    AR_CONF_DEBUG("token = OPT_FLFR_REMOTE_DISABLE (%s)\n", yytext);
    return tk_opt_flfr_remote_disable;
}

{OPT_ERROR_WINDOW} {
    AR_CONF_DEBUG("token = OPT_ERROR_WINDOW (%s)\n", yytext);
    return tk_opt_error_window;
}

{OPT_MAX_ERRORS} {
    AR_CONF_DEBUG("token = OPT_MAX_ERRORS (%s)\n", yytext);
    return tk_opt_max_errors;
}

{OPT_AGEING} {
    AR_CONF_DEBUG("token = OPT_AGEING (%s)\n", yytext);
    return tk_opt_ageing;
}

{OPT_LOG_SIZE} {
    AR_CONF_DEBUG("token = OPT_LOG_SIZE (%s)\n", yytext);
    return tk_opt_log_size;
}

{OPT_LOG_FILE} {
    AR_CONF_DEBUG("token = OPT_LOG_FILE (%s)\n", yytext);
    return tk_opt_log_file;
}

{LEFT_BRACKET} {
    AR_CONF_DEBUG("token = LEFT_BRACKET (%s)\n", yytext);
    return tk_left_bracket;
}

{RIGHT_BRACKET} {
    AR_CONF_DEBUG("token = RIGHT_BRACKET (%s)\n", yytext);
    return tk_right_bracket;
}

{AR_MODE} {
    AR_CONF_DEBUG("token = AR_MODE (%s)\n", yytext);
    strcpy(ar_conf_lval.sval, yytext);
    toUpperCase(ar_conf_lval.sval);
    return tk_ar_mode;
}

{AR_ALGORITHM} {
    AR_CONF_DEBUG("token = AR_ALGORITHM (%s)\n", yytext);
    strcpy(ar_conf_lval.sval, yytext);
    toUpperCase(ar_conf_lval.sval);
    return tk_ar_algorithm;
}

{COLON} {
    AR_CONF_DEBUG("token = COLON (%s)\n", yytext);
    return tk_colon;
}

{SEMICOLON} {
    AR_CONF_DEBUG("token = SEMICOLON (%s)\n", yytext);
    return tk_semicolon;
}

{COMMENT} {
    AR_CONF_DEBUG("token = COMMENT (%s)\n", yytext);
}

{INTEGER} {
    AR_CONF_DEBUG("token = INTEGER (%s)\n", yytext);
    ar_conf_lval.ival = (u_int64_t)strtoull(yytext, NULL, 0);
    return tk_integer;
}

{BOOL} {
    AR_CONF_DEBUG("token = BOOL (%s)\n", yytext);
    char buff[1024];
    strcpy(buff, yytext);
    toUpperCase(buff);
    if (strcmp(buff, "FALSE") == 0) {
        ar_conf_lval.bval = false;
    } else {    //TRUE
        ar_conf_lval.bval = true;
    }    
    return tk_bool;
}

{NAME} {
    AR_CONF_DEBUG("token = FILE_NAME (%s)\n", yytext);
    strcpy(ar_conf_lval.sval, yytext);
    return tk_name;
}

[ |\t|\n]+ ;

. {
    AR_CONF_DEBUG("Error - tk = %s\n", yytext);
    return(tk_error);
}

<<EOF>> {
    AR_CONF_DEBUG("token = EOF (%s)\n", yytext);
    YY_NEW_FILE;
    yyterminate();
}
%%
