/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2013, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */

/* osm_config.h needs to be before all other includes */
#include <opensm/osm_config.h>
#include <errno.h>
#include <ctype.h>
#include <stdlib.h>
#include <sys/time.h>

#include <opensm/osm_sm.h>
#include <opensm/osm_opensm.h>
#include <opensm/osm_base.h>
#include <complib/cl_qmap.h>

#include <algorithm>

#include <iostream>
#include <sstream>

#include "ar_mgr.h"
#include "memory_pool.h"
#include "ar_algorithm.h"
#include "ar_kdor_algorithm.h"

MemoryPool<CopyFromToGroups> copy_from_to_groups_pool;

#define IS_Eagle(device_id) (device_id == 0xcf08 ? true : false)
#define IS_Raven(device_id) (device_id == 0xd2f0 ? true : false)

#define GROUP_BLOCKS_NUMBER(calculated_groups_number , ar_info)  \
        (((calculated_groups_number*(ar_info.sub_grps_active + 1)) + \
          (AR_GROUP_TABLE_BLOCK_SIZE-1)) / AR_GROUP_TABLE_BLOCK_SIZE)

#define IS_GENERATE_ARN(master_db, sw_db_entry) \
        (master_db.m_arn_enable && sw_db_entry.m_ar_info.is_arn_sup && \
         sw_db_entry.m_ar_info.group_table_copy_sup)

#define IS_GENERATE_FRN(master_db, sw_db_entry) \
        (master_db.m_frn_enable && sw_db_entry.m_ar_info.is_frn_sup && \
         sw_db_entry.m_ar_info.group_table_copy_sup)

//Bit 0 = 2 sub groups (1,3,5,7)
//Bit 1 = 3 sub groups (2,3,6,7)
//Bit 2 = 4 sub groups (4,5,6,7)
//Bits 3 to 7 are reserved
                               //0,1,2,3,4,5,6,7
//u_int8_t DFSubGrpsRequired[7] = {1,2,3,1,3,1,2,1};



/****************************************************/
/**************** private methods *******************/
/****************************************************/
//============================================================================//

void AdaptiveRoutingManager::SetRequiredARInfo(ARSWDataBaseEntry &db_entry)
{
    CLEAR_STRUCT(db_entry.m_required_ar_info);

    //db_entry.m_required_ar_info.group_cap = 0;
    //db_entry.m_required_ar_info.by_sl_en = 0;
    //db_entry.m_required_ar_info.enable_by_sl_mask = 0;

    if(m_master_db.m_ar_algorithm == AR_ALGORITHM_DF_PLUS) {
        u_int16_t dfp_en_vl_mask = m_master_db.m_dfp_en_vl_mask;
        if (m_master_db.m_en_sl_mask == ALL_SL_VL_AR_ENABLED) {
            //all sl/vl are AR enabled
            dfp_en_vl_mask = ALL_SL_VL_AR_ENABLED;
        }

        if (db_entry.m_p_df_data->m_df_sw_setup[NEW_DATA].
            m_sw_type == SW_TYPE_SPINE) {
            db_entry.m_required_ar_info.by_sl_en = 1;
            db_entry.m_required_ar_info.enable_by_sl_mask =
                (u_int16_t)(dfp_en_vl_mask & AR_ENABLE_BY_VL_MASK);
        } else if (dfp_en_vl_mask != ALL_SL_VL_AR_ENABLED) {
            db_entry.m_required_ar_info.by_sl_en = 1;
            db_entry.m_required_ar_info.enable_by_sl_mask =
                dfp_en_vl_mask;
        }
        //(when using DF, one can not disable AR on a specific switch )
        db_entry.m_required_ar_info.e = 0x1;
        //currently DF supports only 2 sub groups
        db_entry.m_required_ar_info.sub_grps_active = 1;
        //db_entry.m_required_ar_info.sub_grps_active =
        //    DFSubGrpsRequired[db_entry.m_ar_info.sub_grps_supported & 0x7];
    } else {
        if (m_master_db.m_en_sl_mask != ALL_SL_VL_AR_ENABLED) {
            if (!db_entry.m_ar_info.by_sl_cap) {
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                        "Switch GUID " U64H_FMT ", LID %u, " \
                        "do not support by_sl_cap.\n",
                        db_entry.m_general_sw_info.m_guid,
                        db_entry.m_general_sw_info.m_lid);
                db_entry.m_option_on = false;
            } else {
                db_entry.m_required_ar_info.by_sl_en = 1;
                db_entry.m_required_ar_info.enable_by_sl_mask =
                    m_master_db.m_en_sl_mask;
            }
        }

        db_entry.m_required_ar_info.e = db_entry.m_option_on ? 0x1 : 0x0;
        db_entry.m_required_ar_info.sub_grps_active = 0;
    }

    if (m_master_db.m_disable_tr_mask != 0x0) {
        if (!db_entry.m_ar_info.by_transp_cap) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                    "Switch GUID " U64H_FMT ", LID %u, " \
                    "do not support by_transport_disable.\n",
                    db_entry.m_general_sw_info.m_guid,
                    db_entry.m_general_sw_info.m_lid);
            db_entry.m_option_on = false;
        } else {
            db_entry.m_required_ar_info.by_transp_cap = 1;
            db_entry.m_required_ar_info.by_transport_disable =
                m_master_db.m_disable_tr_mask;
        }
    }

    db_entry.m_required_ar_info.ageing_time_value =
        db_entry.m_ageing_time_value;

    //RN
    if (IS_GENERATE_ARN(m_master_db, db_entry) ||
        IS_GENERATE_FRN(m_master_db, db_entry))
        db_entry.m_required_ar_info.rn_xmit_enabled = 1;
    else if (m_master_db.m_arn_enable || m_master_db.m_frn_enable) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                "Switch GUID " U64H_FMT ", LID %u, " \
                "do not support routing notification.\n",
                db_entry.m_general_sw_info.m_guid,
                db_entry.m_general_sw_info.m_lid);
    }

    if (db_entry.m_ar_info.is_fr_sup &&
        m_master_db.m_fr_enable)
        db_entry.m_required_ar_info.fr_enabled = 1;

    // On moving from FR disabled to FR enabled, clear lft tables
    // because reserved group fieald (on static lid state) become valid
    if (!db_entry.m_ar_info.fr_enabled &&
        db_entry.m_required_ar_info.fr_enabled) {
        db_entry.ClearARLftData();
        if (db_entry.m_p_df_data != NULL) {
            db_entry.ClearDFLftData();
        }
    }
}

void AdaptiveRoutingManager::CheckRC(int& rc)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    struct timeval curr_time;
    // Return if error checking mechanism is disabled
    if (!this->m_master_db.m_error_window)
        OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);

    if (rc == IBIS_MAD_STATUS_SUCCESS)
        OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);

    //
    // The error window is supposed to prevent the AR Manager
    // from stealing the CPU time from the SM and wasting it
    // on errors/timeouts.
    // We will NOT treat errors in the response MAD as a
    // problem in the AR Manager execution cycle and will
    // not count them in the error window.
    //
    if ((rc != IBIS_MAD_STATUS_SEND_FAILED) &&
        (rc != IBIS_MAD_STATUS_RECV_FAILED) &&
        (rc != IBIS_MAD_STATUS_TIMEOUT))
        OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);

    ++this->m_num_errors;
    if (!this->m_master_db.m_max_errors)
        // Abort on first error
        goto check_rc_error;

    gettimeofday(&curr_time, NULL);
    this->m_oldest_error = (this->m_oldest_error + 1) % this->m_master_db.m_max_errors;
    if (this->m_p_error_window_arr[this->m_oldest_error].tv_sec &&  // If equals to zero,
            // no error happened, thus not enough errors happened
        (curr_time.tv_sec - this->m_p_error_window_arr[this->m_oldest_error].tv_sec) <= (long)this->m_master_db.m_error_window) {
        goto check_rc_error;
    }
    // Override oldest error with the new error
    this->m_p_error_window_arr[this->m_oldest_error] = curr_time;

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);

check_rc_error:
    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
            "%s Exited, reached %d error's in less than %u seconds\n",
            ARMGR_ERR_PREFIX1, m_num_errors, this->m_master_db.m_error_window);
    throw 1;
}


//============================================================================//
void AdaptiveRoutingManager::ParseConfFileName(char* osm_plugin_options)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    if (osm_plugin_options == NULL)
        OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);

    string osm_plugin_options_str = osm_plugin_options;
    string conf_file_flag = AR_CFG_INPUT_FILE_FLAG;
    string::size_type flag_pos = 0, start_pos = 0, last_pos = 0;

    if (osm_plugin_options_str.empty()) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,"plugin_options_str is empty.\n");
        OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
    }

    if ((flag_pos = 
         osm_plugin_options_str.find(conf_file_flag)) == string::npos) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,"%s not in %s.\n",
                   conf_file_flag.c_str(),osm_plugin_options_str.c_str());
        OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
    }

    if ((start_pos =
            osm_plugin_options_str.find_first_not_of(" ", flag_pos + conf_file_flag.length()))
            == string::npos) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,"can not find ' ' after %s in %s.\n",
                   conf_file_flag.c_str(),osm_plugin_options_str.c_str());
        OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
    }

    if ((last_pos =
            osm_plugin_options_str.find_first_of(" ", start_pos)) == string::npos) {
        last_pos = osm_plugin_options_str.length();
    }
    last_pos--;

    this->m_conf_file_name = osm_plugin_options_str.substr(start_pos, last_pos - start_pos + 1);

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,"last_pos:%d start_pos:%d str_len:%d str:%s.\n",
               (short)last_pos, (short)start_pos, (short)(last_pos - start_pos + 1),
                osm_plugin_options_str.c_str());
    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,"lm_conf_file_name:%s.\n",
               m_conf_file_name.c_str());

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


//============================================================================//
int AdaptiveRoutingManager::Init()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    static bool already_initialized = false;

    ///!!! TAMIR add to OSM
    //init m_ar_algorithm
    if ((m_ar_algorithm != NULL) &&
        (!this->m_master_db.m_enable ||
         m_ar_algorithm->GetAlgorithm() != m_master_db.m_ar_algorithm)) {

        m_ar_algorithm->ClearAlgorithmConfiguration();
        delete m_ar_algorithm;
        m_ar_algorithm = NULL;
    }

    if ((m_ar_algorithm == NULL) &&
         m_master_db.m_enable &&
         m_master_db.m_ar_algorithm == AR_ALGORITHM_KDOR_HC) {

        m_ar_algorithm = new ArHcAlgorithm(m_p_osm_log, m_sw_db.m_sw_map, *this);

        if (m_ar_algorithm == NULL) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                    "Failed to create HC algorithm\n");
            throw 1;
        }
    }


    if (already_initialized)
        return 0;

    //open log file
    if (this->m_master_db.m_ar_log_file_size) {
        //Only if the file size is greater than 0, the log file will be created.
        if (tt_log_construct_v2(0xff,                               // need_to_flush
                this->m_master_db.m_ar_log_file_name.c_str(),       // log file
                this->m_master_db.m_ar_log_file_size,               // max size (in MB)
                0)) {                                               // append to prev log
            this->m_master_db.m_ar_log_file_size = 0;
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                    "%s Error opening Adaptive Routing log file : %s\n",
                    ARMGR_ERR_PREFIX2, this->m_master_db.m_ar_log_file_name.c_str());
            throw 1;
        }
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO, "Adaptive Routing log file: %s\n",
                this->m_master_db.m_ar_log_file_name.c_str());
    }

    //init ibis
    if (this->m_ibis_obj.IsInit() == true) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "ibis already initialized\n");
    } else if (this->m_ibis_obj.Init()) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR, "%s ibis initialization failed\n", ARMGR_ERR_PREFIX3);
        throw 1;
    } else {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_VERBOSE, "ibis initialized\n");
    }

    //set port
    this->m_port_guid = cl_ntoh64(this->m_p_osm_vendor->umad_port.port_guid);
    if (this->m_ibis_obj.IsReady() == true) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "ibis SetPort ready\n");
    } else if (this->m_ibis_obj.SetPort(ADB2C_CPU_TO_BE64(this->m_port_guid))) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR, "%s ibis SetPort failed\n", ARMGR_ERR_PREFIX4);
        throw 1;
    } else {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_VERBOSE,
                "ibis running on GUID " U64H_FMT "\n", this->m_port_guid);
    }

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
               "Set Max Mads OnWire GMP:%d SMP:%d",
               (int)(u_int16_t)DEFAULT_GMPS_WINDOW,
               (int)(u_int8_t)m_p_osm_subn->opt.max_wire_smps);

    this->m_ibis_obj.SetMaxMadsOnWire((u_int16_t)DEFAULT_GMPS_WINDOW,
                                      (u_int8_t)m_p_osm_subn->opt.max_wire_smps);

    this->m_ibis_obj.SetMKeyManager(&this->m_mkey_manager);

    if (m_sw_db.Init()) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                   "ARSWDataBase init failed.\n");
        throw 1;
    }

    if (m_thread_pool.Init(m_p_osm_subn->opt.routing_threads_num)) {
        throw 1;
    }

    m_port_groups_calculator.Init();

    already_initialized = true;
    OSM_AR_LOG_RETURN(this->m_p_osm_log, 0);
}


//============================================================================//
/***************/
//TODO - what if new devices will support AR in the future
bool AdaptiveRoutingManager::IsDeviceIDSupported(const ARGeneralSWInfo& general_sw_info)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    u_int16_t device_id = cl_ntoh16(general_sw_info.m_p_osm_sw->p_node->node_info.device_id);

    bool is_supported = (general_sw_info.m_sx_dev ||
                         this->m_ibis_obj.IsDevShaldag(device_id));

    if (!is_supported)
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_VERBOSE,
                "Switch GUID " U64H_FMT ", LID %u is not supported, device id = %u\n",
                general_sw_info.m_guid, general_sw_info.m_lid, device_id);

    OSM_AR_LOG_RETURN(this->m_p_osm_log, is_supported);
}

/***************/
void AdaptiveRoutingManager::UpdateSW(const ARGeneralSWInfo& general_sw_info)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    GuidToSWDataBaseEntryIter sw_node_it;
    // Case new switch was added
    if ((sw_node_it = this->m_sw_db.m_sw_map.find(general_sw_info.m_guid)) ==
            this->m_sw_db.m_sw_map.end()) {

        ARSWDataBaseEntry new_sw_node(general_sw_info, &m_port_groups_calculator);
        GuidToSWDataBaseEntryInsertRes res = this->m_sw_db.m_sw_map.insert(
            pair<u_int64_t, ARSWDataBaseEntry>(general_sw_info.m_guid, new_sw_node));
        sw_node_it = res.first;
        sw_node_it->second.Init();
#ifdef _DEBUG_AR_UPDATE_DB
        printf("Switch GUID " U64H_FMT " was added to db\n",
                sw_node_it->second.m_general_sw_info.m_guid);
#endif
    }

    // Save general info even if we know this switch already because
    // this general info might have been changed by osm
    sw_node_it->second.SetARGeneralSWInfo(general_sw_info);
    sw_node_it->second.in_temporary_error = false;

    // Mark that update was done
    sw_node_it->second.m_app_data = ARMGR_SW_UPDATED;

    // Update osm_update_needed field
    if (this->m_osm_event_id == OSM_EVENT_ID_HEAVY_SWEEP_DONE ||
            this->m_osm_event_id == OSM_EVENT_ID_START_LFT_UPDATE) {
        sw_node_it->second.m_osm_update_needed = general_sw_info.m_p_osm_sw->need_update
                || m_p_osm_subn->need_update;
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

/***************/
void AdaptiveRoutingManager::AddNewAndUpdateExistSwitches()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    osm_switch_t *p_osm_sw;
    // Go over only on switches in osm database to find new ones or changed ones
    for (p_osm_sw = (osm_switch_t *) cl_qmap_head(&this->m_p_osm_subn->sw_guid_tbl);
            p_osm_sw != (osm_switch_t *) cl_qmap_end(&this->m_p_osm_subn->sw_guid_tbl);
            p_osm_sw = (osm_switch_t *) cl_qmap_next(&p_osm_sw->map_item)) {

        u_int16_t device_id = cl_ntoh16(p_osm_sw->p_node->node_info.device_id);
        bool is_sx_dev = this->m_ibis_obj.IsDevSwitchXIB(device_id) ||
                         this->m_ibis_obj.IsDevPelican(device_id) ||
                         IS_Eagle(device_id) ||
                         IS_Raven(device_id);

        ARGeneralSWInfo general_sw_info(ADB2C_BE64_TO_CPU(osm_node_get_node_guid(p_osm_sw->p_node)),  // GUID
                ADB2C_BE16_TO_CPU(osm_node_get_base_lid(p_osm_sw->p_node, 0)),                        // Lid
                osm_node_get_num_physp(p_osm_sw->p_node) - 1,                                   // Num ports
                is_sx_dev,                                                                      //Is switchX
                p_osm_sw,                                                                       // Switch ptr
                osm_physp_get_dr_path_ptr(osm_node_get_physp_ptr(p_osm_sw->p_node, 0)));        // DR ptr

        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "AddNewAndUpdateExistSwitches GUID " U64H_FMT " Lid: %u\n",
                   general_sw_info.m_guid, general_sw_info.m_lid);

        this->UpdateSW(general_sw_info);

    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

/***************/
void AdaptiveRoutingManager::RemoveAbsentSwitches()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    map<u_int64_t, ARSWDataBaseEntry>::iterator aux_sw_node_it;
    while (sw_node_it !=  this->m_sw_db.m_sw_map.end()) {
        if (sw_node_it->second.m_app_data == ARMGR_SW_UPDATED) {
            ++sw_node_it;
            continue;
        }

#ifdef _DEBUG_AR_UPDATE_DB
        printf("Switch GUID " U64H_FMT " was removed from db\n",
                sw_node_it->second.m_general_sw_info.m_guid);
#endif

        aux_sw_node_it = sw_node_it;
        ++sw_node_it;

        m_sw_db.UnMapLidToGroup(aux_sw_node_it->second.m_general_sw_info.m_lid);
        this->m_sw_db.m_sw_map.erase(aux_sw_node_it);
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


//============================================================================//
/***************/

/*
void AdaptiveRoutingManager::MarkSWNotSupport(ARSWDataBaseEntry &sw_db_entry,
                                              supported_feature_t feature,
                                              support_errs_t type)
{
    sw_db_entry.m_support[feature] = NOT_SUPPORTED;
    sw_db_entry.m_error[feature] = type;
}

bool AdaptiveRoutingManager::IsNotSupported(ARSWDataBaseEntry &sw_db_entry,
                                            supported_feature_t feature) {
    return (sw_db_entry.m_support[feature] == NOT_SUPPORTED);
}
*/


void AdaptiveRoutingManager::MarkSWNotSupportAR(ARSWDataBaseEntry &sw_db_entry,
        support_errs_t type)
{
    sw_db_entry.m_support[SUPPORT_AR] = NOT_SUPPORTED;
    sw_db_entry.m_error[SUPPORT_AR] = type;
}

bool AdaptiveRoutingManager::IsARNotSupported(ARSWDataBaseEntry &sw_db_entry) {
    return (sw_db_entry.m_support[SUPPORT_AR] == NOT_SUPPORTED);
}

void AdaptiveRoutingManager::MarkSWSupportAR(ARSWDataBaseEntry &sw_db_entry) {
    sw_db_entry.m_support[SUPPORT_AR] = SUPPORTED;
}

bool AdaptiveRoutingManager::IsARActive(ARSWDataBaseEntry &sw_db_entry) {
    return (sw_db_entry.m_support[SUPPORT_AR] == SUPPORTED && sw_db_entry.m_option_on);
}

bool AdaptiveRoutingManager::IsDFActive(ARSWDataBaseEntry &sw_db_entry) {
    return (IsARActive(sw_db_entry) &&
            sw_db_entry.m_support[SUPPORT_DF] == SUPPORTED);
}

int  AdaptiveRoutingManager::ARInfoGetProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    int unsupported_num = 0;
    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &this->m_ar_clbck;

    adaptive_routing_info smp_ar_info;
    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    for ( ; sw_node_it != this->m_sw_db.m_sw_map.end() ; ++sw_node_it) {

        if (sw_node_it->second.m_osm_update_needed) {
            sw_node_it->second.m_support[SUPPORT_AR] = SUPPORT_UNKNOWN;
            sw_node_it->second.m_max_lid = 0;
            //sw_node_it->second.m_total_groups_number = 0;
            sw_node_it->second.m_group_top = 0;
            sw_node_it->second.m_no_groups = true;
        }

        /*
         * Step 1.1 -
         *  Skip if we know already that this switch not support AR
         */
        if (this->IsARNotSupported(sw_node_it->second)) {
            /*
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Switch GUID " U64H_FMT ", LID: %u " \
                    "AR not supported, ar information skipped.\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid);
            //we already suppose to have this switch in not supported list
            */
            unsupported_num++;
            continue;
        }

        /* Step 1.2 -
         *  Make sure this device id supports AR
         */
        if (!this->IsDeviceIDSupported(sw_node_it->second.m_general_sw_info)) {
            this->MarkSWNotSupportAR(sw_node_it->second, AR_DEVICE_ID_NOT_SUPPORTED);
            unsupported_num++;
            continue;
        }

        /*
         * Step 1.3 -
         *  We support AR until AR_MAX_PORTS_SUPPORTED only
         */
        if (sw_node_it->second.m_general_sw_info.m_num_ports > MAX_SUPPORTED_PORTS) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                    "%s Switch GUID " U64H_FMT ", LID %u " \
                    "has %u external ports, no AR support\n",
                    ARMGR_ERR_PREFIX5,
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid,
                    sw_node_it->second.m_general_sw_info.m_num_ports);
            this->MarkSWNotSupportAR(sw_node_it->second, AR_MAX_PORTS_EXCEEDED);
            unsupported_num++;
            continue;
        }

        if (!sw_node_it->second.m_osm_update_needed &&
            sw_node_it->second.m_ar_info.group_cap != 0)
            continue; //we already read this info

        /*
         * Step 2 -
         *  Sending AR information
         */

        // Sending GET()
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                "Getting AR Info from "    \
                "Switch GUID " U64H_FMT ", LID %u\n",
                sw_node_it->second.m_general_sw_info.m_guid,
                sw_node_it->second.m_general_sw_info.m_lid);

        clbck_data.m_data1 = &sw_node_it->second;
        clbck_data.m_handle_data_func = GetARInfoClbckDlg;

        m_ibis_obj.SMPARInfoGetSetByDirect(
                &sw_node_it->second.m_general_sw_info.m_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                false, //not get cap
                &smp_ar_info, &clbck_data);

    }

    //Receive all Get Info reply
    m_ibis_obj.MadRecAll();
    unsupported_num += m_ar_clbck.m_errcnt[AR_CLBCK_GET_AR_INFO];

    OSM_AR_LOG_RETURN(this->m_p_osm_log, unsupported_num);

}

int  AdaptiveRoutingManager::ARInfoGetGroupCapProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    int unsupported_num = 0;
    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &this->m_ar_clbck;

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    for ( ; sw_node_it != this->m_sw_db.m_sw_map.end() ; ++sw_node_it) {

        /*
         * Step 1.1 -
         *  Skip if we know already that this switch not support AR
         */
        if (this->IsARNotSupported(sw_node_it->second))
            continue;

        if (sw_node_it->second.in_temporary_error)
            continue;

        SetRequiredARInfo(sw_node_it->second);

        //OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
        //           "IsEqualSMPARInfo: sub_grps_active:%d required sub_grps_active:%d \n",
        //           sw_node_it->second.m_ar_info.sub_grps_active,
        //           sw_node_it->second.m_required_ar_info.sub_grps_active);

        if (this->IsEqualSMPARInfo(&sw_node_it->second.m_ar_info,
                                   &sw_node_it->second.m_required_ar_info,
                                   true, true)) {

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "Switch GUID " U64H_FMT ", LID %u use group_cap:%u\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid,
                    sw_node_it->second.m_ar_info.group_cap   );

            sw_node_it->second.m_required_ar_info.group_cap =
                    sw_node_it->second.m_ar_info.group_cap;
            continue; //we already read this info
        }

        if (!sw_node_it->second.m_ar_info.dyn_cap_calc_sup) {

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "dynamic_cap_calc not supported for "    \
                    "Switch GUID " U64H_FMT ", LID %u use group_cap:%u\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid,
                    sw_node_it->second.m_ar_info.group_cap   );

            sw_node_it->second.m_required_ar_info.group_cap =
                    sw_node_it->second.m_ar_info.group_cap;
            continue;
        }

        /*
         * Step 2 -
         *  Sending AR information
         */

        // Sending GET()
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                "Getting AR Info Cap from "    \
                "Switch GUID " U64H_FMT ", LID %u \n",
                sw_node_it->second.m_general_sw_info.m_guid,
                sw_node_it->second.m_general_sw_info.m_lid);

        clbck_data.m_data1 = &sw_node_it->second;
        clbck_data.m_handle_data_func = GetRequiredARInfoCapClbckDlg;

        m_ibis_obj.SMPARInfoGetSetByDirect(
                &sw_node_it->second.m_general_sw_info.m_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                true, //get cap
                &sw_node_it->second.m_required_ar_info,
                &clbck_data);

    }

    //Receive all Get Info reply
    m_ibis_obj.MadRecAll();
    unsupported_num += m_ar_clbck.m_errcnt[AR_CLBCK_GET_AR_INFO_CAP];

    OSM_AR_LOG_RETURN(this->m_p_osm_log, unsupported_num);

}

int AdaptiveRoutingManager::ARInfoSetProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    int unsupported_num = 0;
    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &this->m_ar_clbck;

    adaptive_routing_info smp_ar_info;
    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    for ( sw_node_it = this->m_sw_db.m_sw_map.begin() ;
          sw_node_it != this->m_sw_db.m_sw_map.end() ; ++sw_node_it) {

        if (this->IsARNotSupported(sw_node_it->second)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Switch GUID " U64H_FMT ", LID %u: " \
                       "AR not supported, ar information skipped.\n",
                       sw_node_it->second.m_general_sw_info.m_guid,
                       sw_node_it->second.m_general_sw_info.m_lid);
            //we already suppose to have this switch in not supported list
            continue;
        }

        if (sw_node_it->second.in_temporary_error)
            continue;

        clbck_data.m_data2 = (void *)(intptr_t)ALL_SL_VL_AR_ENABLED;
        smp_ar_info = sw_node_it->second.m_required_ar_info;

        /*
         *  Set Group Top
         */
        if (sw_node_it->second.m_ar_info.ar_version_cap >=
            AR_VERSION_CAP_GROUP_TOP)
            smp_ar_info.group_top = sw_node_it->second.m_group_top;

        /*
         *  If not configure as we want than configure again
         */

        // Sending SET()
        if (!this->IsEqualSMPARInfo(&sw_node_it->second.m_ar_info,
                                    &smp_ar_info, false, false)) {

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "Setting AR Info to "   \
                    "Switch GUID " U64H_FMT ", LID %u, " \
                    "Setting AR mode to %s, sub grps in grp %d.\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid,
                    (smp_ar_info.e) ? "enable" : "disable",
                    smp_ar_info.sub_grps_active + 1);

            //TODO random ageing
            clbck_data.m_data1 = &sw_node_it->second;
            clbck_data.m_handle_data_func = SetARInfoClbckDlg;

            m_ibis_obj.SMPARInfoGetSetByDirect(
                    &sw_node_it->second.m_general_sw_info.m_direct_route,
                    IBIS_IB_MAD_METHOD_SET,
                    false, //not get cap
                    &smp_ar_info, /*sw_node_it->second.m_ar_info*/
                    &clbck_data);

        }
    }

    //Receive all Set Info reply
    m_ibis_obj.MadRecAll();
    unsupported_num += m_ar_clbck.m_errcnt[AR_CLBCK_SET_AR_INFO];

    OSM_AR_LOG_RETURN(this->m_p_osm_log, unsupported_num);
}

void AdaptiveRoutingManager::TreeRoutingNotificationProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &this->m_ar_clbck;

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    for ( sw_node_it = this->m_sw_db.m_sw_map.begin() ;
          sw_node_it != this->m_sw_db.m_sw_map.end() ; ++sw_node_it) {

        if (this->IsARNotSupported(sw_node_it->second)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Switch GUID " U64H_FMT ", LID %u: " \
                       "AR not supported, ar information skipped.\n",
                       sw_node_it->second.m_general_sw_info.m_guid,
                       sw_node_it->second.m_general_sw_info.m_lid);
            //we already suppose to have this switch in not supported list
            continue;
        }

        if (sw_node_it->second.in_temporary_error ||
            !sw_node_it->second.m_config_rn)
            continue;

        clbck_data.m_data1 = &sw_node_it->second;

        /*
         * Step 1 -
         *  RNGenStringTable
         */

        rn_gen_string_tbl gen_string;
        CLEAR_STRUCT(gen_string);

        u_int16_t max_string = (u_int16_t)
            ((0x1 << sw_node_it->second.m_ar_info.string_width_cap) - 1);

        if (sw_node_it->second.m_general_sw_info.m_p_osm_sw->rank >
            max_string)

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                       "Switch GUID " U64H_FMT ", LID %u, will not genarate RN"
                       " because sw rank:%u is greater than max_string:%u.\n",
                       sw_node_it->second.m_general_sw_info.m_guid,
                       sw_node_it->second.m_general_sw_info.m_lid,
                       sw_node_it->second.m_general_sw_info.m_p_osm_sw->rank,
                       max_string)

        else
            gen_string.element[0].rn_gen_string =
                sw_node_it->second.m_general_sw_info.m_p_osm_sw->rank;

        // Sending
        if (gen_string.element[0].rn_gen_string !=
            sw_node_it->second.m_rn_gen_string) {

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "Setting RNGenStringTable "   \
                    "Switch GUID " U64H_FMT ", LID %u, " \
                    "rn_gen_string:%u.\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid,
                    gen_string.element[0].rn_gen_string);

            clbck_data.m_handle_data_func = SetRNGenStringClbckDlg;

            m_ibis_obj.SMPRNGenStringTableGetSetByDirect(
                    &sw_node_it->second.m_general_sw_info.m_direct_route,
                    IBIS_IB_MAD_METHOD_SET,
                    0, //group_block
                    0, //pLFTID
                    &gen_string,
                    &clbck_data);

        }

        /*
         * Step 2 -
         *  RNGenBySubGroupPriority
         */

        rn_gen_by_sub_group_prio gen_by_sg_priority;
        CLEAR_STRUCT(gen_by_sg_priority);

        gen_by_sg_priority.element[0].gen_arn =
            IS_GENERATE_ARN(m_master_db, sw_node_it->second);
        gen_by_sg_priority.element[0].gen_frn =
            IS_GENERATE_FRN(m_master_db, sw_node_it->second);

        if (memcmp(&gen_by_sg_priority.element[0],
                   &sw_node_it->second.m_rn_gen_by_sub_group_priority.
                   element[0],
                   sizeof(rn_gen_by_sub_group_prio))) {

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "Setting RNGenBySubGroupPriority "   \
                    "Switch GUID " U64H_FMT ", LID %u, " \
                    "gen_arn:%u, gen_frn:%u \n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid,
                    gen_by_sg_priority.element[0].gen_arn,
                    gen_by_sg_priority.element[0].gen_frn);

            clbck_data.m_handle_data_func = SetRNGenBySubGroupPriorityClbckDlg;

            m_ibis_obj.SMPRNGenBySubGroupPriorityMadGetSetByDirect(
                    &sw_node_it->second.m_general_sw_info.m_direct_route,
                    IBIS_IB_MAD_METHOD_SET,
                    &gen_by_sg_priority,
                    &clbck_data);
        }

        /*
         * Step 3 -
         *  RNRcvString
         */
        if (sw_node_it->second.m_to_set_rn_rcv_string) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "Setting RNRcvString "   \
                    "Switch GUID " U64H_FMT ", LID %u, \n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid);

            clbck_data.m_handle_data_func = SetRNRcvStringClbckDlg;

            m_ibis_obj.SMPRNRcvStringGetSetByDirect(
                    &sw_node_it->second.m_general_sw_info.m_direct_route,
                    IBIS_IB_MAD_METHOD_SET,
                    0, //string_block
                    &sw_node_it->second.m_rn_rcv_string,
                    &clbck_data);
        }


        /*
         * Step 4 -
         *  RNGenPortMask
         */
        if (sw_node_it->second.m_to_set_rn_xmit_port_mask) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "Setting RNXmitPortMask "   \
                    "Switch GUID " U64H_FMT ", LID %u, \n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid);

            for (int i = 0 ; i < RN_XMIT_PORT_MASK_BLOCK_SIZE ; ++i) {
                u_int8_t mask = sw_node_it->second.m_rn_xmit_port_mask.element[i];
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                        "Setting RNXmitPortMask "   \
                        "Switch GUID " U64H_FMT ", LID %u, "
                        " element %u gen_arn %u gen_frn %u pass_on %u\n",
                        sw_node_it->second.m_general_sw_info.m_guid,
                        sw_node_it->second.m_general_sw_info.m_lid, i,
                        mask & RN_XMIT_PORT_MASK_GENERATE_ARN,
                        mask & RN_XMIT_PORT_MASK_GENERATE_FRN,
                        mask & RN_XMIT_PORT_MASK_PASS_ON_RN);

            }

            clbck_data.m_handle_data_func = SetRNXmitPortMaskClbckDlg;

            m_ibis_obj.SMPRNXmitPortMaskGetSetByDirect(
                    &sw_node_it->second.m_general_sw_info.m_direct_route,
                    IBIS_IB_MAD_METHOD_SET,
                    0, //block
                    &sw_node_it->second.m_rn_xmit_port_mask,
                    &clbck_data);
        }
    }

    //Receive all Set Info reply
    m_ibis_obj.MadRecAll();

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}
/***************/
void AdaptiveRoutingManager::ARDumpSWSettings(ARSWDataBaseEntry &sw_db_entry)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    if (!osm_log_is_active(this->m_p_osm_log,OSM_LOG_ROUTING))
        return;

    char buff[1024];
    string str = "---------------\n";
    ConvertARInfoToStr(sw_db_entry.m_general_sw_info,
                       sw_db_entry.m_ar_info, buff);

    str += buff;
    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ROUTING, "%s", str.c_str());

    //u_int16_t max_group_id =
    //    (sw_db_entry.m_total_groups_number > 0 ?
    //     sw_db_entry.m_total_groups_number -1 : 0);

    unsigned supported_group_blocks =
            sw_db_entry.m_group_top / AR_GROUP_TABLE_BLOCK_SIZE;

    for (unsigned group_block = 0; group_block <= supported_group_blocks; ++group_block) {
        sprintf(buff, "Group Table Settings : Block " U32D_FMT "/" U32D_FMT "\n",
                group_block,
                supported_group_blocks);
        str = buff;
        str += this->ConvertARGroupTableBlockToStr(
            &sw_db_entry.m_ar_group_table[group_block],
            sw_db_entry.m_ar_info, group_block);
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ROUTING, "%s", str.c_str());
    }

    if (sw_db_entry.m_general_sw_info.m_sx_dev) {

        unsigned supported_lft_blocks =
                sw_db_entry.m_max_lid / AR_LFT_TABLE_BLOCK_SIZE_SX;
        for (unsigned lft_block = 0; lft_block <= supported_lft_blocks; ++lft_block) {
            sprintf(buff, "SX LFT Table Settings : Block " U32D_FMT "/" U32D_FMT "\n",
                    lft_block,
                    AR_LFT_TABLE_NUM_BLOCKS_SX);
            str = buff;
            str += this->ConvertARLFTTableBlockToStr(&sw_db_entry.m_ar_lft.m_ar_lft_table_sx[lft_block],
                    lft_block);
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ROUTING, "%s", str.c_str());
        }
    } else {
        unsigned supported_lft_blocks =
                sw_db_entry.m_max_lid / AR_LFT_TABLE_BLOCK_SIZE;
        for (unsigned lft_block = 0; lft_block <= supported_lft_blocks; ++lft_block) {
            sprintf(buff, "LFT Table Settings : Block " U32D_FMT "/" U32D_FMT "\n",
                    lft_block,
                    AR_LFT_TABLE_NUM_BLOCKS);
            str = buff;
            str += this->ConvertARLFTTableBlockToStr(&sw_db_entry.m_ar_lft.m_ar_lft_table[lft_block],
                    lft_block);
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ROUTING, "%s", str.c_str());
        }
    }

    str = "---------------\n";
    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ROUTING, "%s", str.c_str());

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

/***************/
//Currently implemented for one sub group per group
void AdaptiveRoutingManager::ARUpdateSWGroupToCopy(
        ARSWDataBaseEntry &sw_db_entry,
        GroupsList &assign_groups)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    set<uint16_t> copy_to_groups;
    uint16_t copy_from_group = 0;
    uint16_t tmp_copy_from_group = 0;
    uint16_t copy_from_block = 0;
    uint16_t modified_block = 0;
    uint16_t prev_modified_block = 0;
    pair<set<uint16_t>::iterator,bool> insert_res;
    set<uint16_t>::iterator modified_group_iter_1;
    set<uint16_t>::iterator modified_group_iter_2;

    uint16_t group_number = 0;
    uint16_t group_top = 0;
    PortGroup_Block_Element calculated_element;
    PortGroup_Block_Element *p_current_element;
    int element_cmp;
    uint16_t group_block_idx;

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
            "ARUpdateSWGroupToCopy "   \
            "Switch GUID " U64H_FMT ", LID %u, \n",
            sw_db_entry.m_general_sw_info.m_guid,
            sw_db_entry.m_general_sw_info.m_lid);

    /*For each group_bitmask we try to find group to copy from
      and a list of copy to groups.
      we search for copy from group in the folowing priority:
      1) Two consecutive unmodified groups on the same block
      2) One unmodified group - should be optimized
      3) Two consecutive modified groups on the same block
      4) One modified group.
      */

    GroupsListIter list_iter = assign_groups.begin();
    for ( ;list_iter != assign_groups.end(); ++list_iter) {

        copy_to_groups.clear();

        copy_from_group = 0;
        copy_from_block = 0;
        tmp_copy_from_group = 0;
        modified_block = 0;
        prev_modified_block = 0;

        (*list_iter)->m_group_bitmask.get_data(calculated_element.SubGroup_0,
                       calculated_element.SubGroup_1,
                       calculated_element.SubGroup_2,
                       calculated_element.SubGroup_3);

        if(this->m_p_osm_log->level & OSM_LOG_DEBUG) {

            stringstream sstr;
            sstr << "Group bitset:" << (*list_iter)->m_group_bitmask;

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "ARUpdateSWGroupToCopy %s\n",
                       sstr.str().c_str());
        }

        LidToGroupNumberMapConstIter leaf_switch_grp_iter =
            (*list_iter)->m_sw_lid_to_group_number.begin();

        for (;
             leaf_switch_grp_iter !=
                (*list_iter)->m_sw_lid_to_group_number.end();
             ++leaf_switch_grp_iter) {

            group_number = leaf_switch_grp_iter->second;
            group_top = max(group_top, group_number);

            group_block_idx = group_number / AR_GROUP_TABLE_BLOCK_SIZE;

            p_current_element = &sw_db_entry.
                m_ar_group_table[group_block_idx].
                Group[group_number % AR_GROUP_TABLE_BLOCK_SIZE];

            element_cmp = 0;

            //If the block was already set to be updated we will update
            //this group before copy from it.
            //Thes occur if previous bitamsk selected a group from this
            //block to be the copy from group.
            if(sw_db_entry.m_to_set_group_table[group_block_idx] == true)
                ARUpdateSWGroupElement(sw_db_entry,calculated_element,
                                       group_number);
            else {
                element_cmp =
                    (sw_db_entry.m_osm_update_needed ||
                     memcmp(p_current_element, &calculated_element,
                                     sizeof(PortGroup_Block_Element)));
            }

            if(element_cmp){
                insert_res = copy_to_groups.insert(group_number);

                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                           "ARUpdateSWGroupToCopy Copy to group %u\n",
                           group_number);

                if (modified_block == 0) {
                    if (prev_modified_block == group_block_idx){
                        modified_block = group_block_idx;
                        modified_group_iter_2 = insert_res.first;
                    } else {
                        prev_modified_block = group_block_idx;
                        modified_group_iter_1 = insert_res.first;
                    }
                }

            } else if (copy_from_group == 0) {
                if (copy_from_block == group_block_idx) {
                    //complete block we can copy
                    copy_from_group = tmp_copy_from_group;
                }
                tmp_copy_from_group = group_number;
                copy_from_block = group_block_idx;
            }
        }

        if (copy_from_group == 0){
            if(tmp_copy_from_group !=0) {
                //2) One unmodified group

                copy_from_group = tmp_copy_from_group;
            } else if (modified_block != 0){
                //3) Two consecutive modified groups on the same block

                copy_from_group = (uint16_t)(modified_block *
                                             AR_GROUP_TABLE_BLOCK_SIZE);

                ARUpdateSWGroupElement(sw_db_entry,calculated_element,
                                       copy_from_group);
                ARUpdateSWGroupElement(sw_db_entry,calculated_element,
                                       copy_from_group+1);

                copy_to_groups.erase(modified_group_iter_1);
                copy_to_groups.erase(modified_group_iter_2);

                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                           "ARUpdateSWGroupToCopy UpdateSWGroups %u and %u\n",
                           copy_from_group, copy_from_group+1);

                //optimize - remove copy_from_group from copy_to_groups
            } else {
                //4) One modified group.

                if (copy_to_groups.empty())
                    continue; //empty group

                copy_from_group = *copy_to_groups.begin();
                copy_to_groups.erase(copy_to_groups.begin());

                ARUpdateSWGroupElement(sw_db_entry,calculated_element,
                                       copy_from_group);

                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                           "ARUpdateSWGroupToCopy remove group %u "
                           "from copy to groups. (set copy from)\n",
                           group_number);
            }
        }

        if (copy_to_groups.empty())
            continue;

        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                   "ARUpdateSWGroupToCopy copy from group %u\n",
                   copy_from_group);

        //Now we convert the copy_to_groups to pair of consecutive groups

        CopyFromToGroups *p_copy_from_to_groups = NULL;
        adaptive_routing_group_table_copy_element *p_copy_element = NULL;
        set<uint16_t>::iterator modified_group_iter = copy_to_groups.begin();

        u_int16_t copy_to_element = AR_COPY_GROUP_BLOCK_SIZE;
        u_int16_t next_copy_to_element = AR_COPY_GROUP_BLOCK_SIZE;
        bool copy_direction = false;


        u_int16_t copy_cap = 1000;
        u_int16_t total_curr_copy = 0;

        while (true) {

            if (!copy_direction &&
                *sw_db_entry.GetSubGroupDirectionPtr(*modified_group_iter,0))
                copy_direction = true;

            if ( next_copy_to_element == AR_COPY_GROUP_BLOCK_SIZE) {
                //set copy_direction of prev copy_from_to_groups
                if (p_copy_from_to_groups)
                    p_copy_from_to_groups->m_copy_direction = copy_direction;
                copy_direction = false;

                p_copy_from_to_groups = copy_from_to_groups_pool.alloc();
                p_copy_from_to_groups->m_copy_from_group = copy_from_group;
                next_copy_to_element = 0;
                 //make sure copy_to_element != next_copy_to_element
                copy_to_element = AR_COPY_GROUP_BLOCK_SIZE;
                total_curr_copy = 0;
                sw_db_entry.m_copy_from_to_group_list.push_back(p_copy_from_to_groups);
            }
            if (copy_to_element != next_copy_to_element) {

                copy_to_element = next_copy_to_element;
                p_copy_element = &p_copy_from_to_groups->m_ar_group_table_copy.
                    element[copy_to_element];

                p_copy_element->copy_from_index = *modified_group_iter;
                p_copy_element->copy_to_index = *modified_group_iter;
                total_curr_copy++;
            }

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "ARUpdateSWGroupToCopy modified_group %u "
                       "from index:%u to index:%u element:%u\n",
                       *modified_group_iter,
                       p_copy_element->copy_from_index,
                       p_copy_element->copy_to_index,
                       copy_to_element);

            ++modified_group_iter;
            if (modified_group_iter == copy_to_groups.end())
                break;

            if ((*modified_group_iter == (p_copy_element->copy_to_index + 1)) &&
                (total_curr_copy < copy_cap)){
                p_copy_element->copy_to_index++;
                total_curr_copy++;
            } else {
                next_copy_to_element++;
                //if copy_cap exceeded - create new copy_from_to_group
                if((total_curr_copy >= copy_cap) &&
                   (next_copy_to_element != AR_COPY_GROUP_BLOCK_SIZE)){
                    p_copy_from_to_groups->m_ar_group_table_copy.
                        element[next_copy_to_element].copy_from_index = 0;
                    p_copy_from_to_groups->m_ar_group_table_copy.
                        element[next_copy_to_element].copy_to_index = 0;
                    next_copy_to_element = AR_COPY_GROUP_BLOCK_SIZE;
                }
            }
        }

        //finalize
        p_copy_from_to_groups->m_copy_direction = copy_direction;
        copy_to_element++;
        if (copy_to_element < AR_COPY_GROUP_BLOCK_SIZE) {
            p_copy_from_to_groups->m_ar_group_table_copy.
                element[copy_to_element].copy_from_index = 0;
            p_copy_from_to_groups->m_ar_group_table_copy.
                element[copy_to_element].copy_to_index = 0;
        }

    }

    sw_db_entry.m_group_top = group_top;
    sw_db_entry.m_no_groups = false;

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void AdaptiveRoutingManager::ARUpdateSWGroupElement(
        ARSWDataBaseEntry &sw_db_entry,
        PortGroup_Block_Element &calculated_element,
        u_int16_t group_number)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
               "ARUpdateSWGroupElement Switch GUID " U64H_FMT ", LID %u " \
               "group_number:%u\n",
               sw_db_entry.m_general_sw_info.m_guid,
               sw_db_entry.m_general_sw_info.m_lid,
               group_number);

    u_int16_t block_idx = group_number / AR_GROUP_TABLE_BLOCK_SIZE;
    sw_db_entry.m_ar_group_table[block_idx].
        Group[group_number % AR_GROUP_TABLE_BLOCK_SIZE] =
        calculated_element;
    sw_db_entry.m_to_set_group_table[block_idx] = true;

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
               "set_group_element group:%u " \
               U64H_FMT "\n",
               group_number,
               sw_db_entry.m_ar_group_table[block_idx].
               Group[group_number % AR_GROUP_TABLE_BLOCK_SIZE].SubGroup_0);

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void AdaptiveRoutingManager::ARUpdateSWGroupTable(ARSWDataBaseEntry &sw_db_entry,
        struct SMP_ARGroupTable p_ar_calculated_group_table[],
        u_int16_t calculated_groups_number)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    if (calculated_groups_number == 0) {
        sw_db_entry.m_no_groups = true;
        sw_db_entry.m_group_top = 0;
        return;
    }

    sw_db_entry.m_group_top = calculated_groups_number - 1;
    sw_db_entry.m_no_groups = false;

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
               "ARUpdateSWGroupTable Switch GUID " U64H_FMT ", LID %u " \
               "calculated_groups_number:%u\n",
               sw_db_entry.m_general_sw_info.m_guid,
               sw_db_entry.m_general_sw_info.m_lid,
               calculated_groups_number);

    // Update blocks
    // In fact we go over only on blocks we need - until max group id
    unsigned total_blocks_number =
        GROUP_BLOCKS_NUMBER(calculated_groups_number,
                            sw_db_entry.m_required_ar_info);

    for (unsigned group_block_idx = 0; group_block_idx < total_blocks_number;
         ++group_block_idx) {
        if (sw_db_entry.m_osm_update_needed ||
            !sw_db_entry.m_is_group_table_valid[group_block_idx] ||
            !this->IsEqualSMPARGroupTableBlock(
                        &p_ar_calculated_group_table[group_block_idx],
                        &sw_db_entry.m_ar_group_table[group_block_idx])) {

            sw_db_entry.m_ar_group_table[group_block_idx] =
                p_ar_calculated_group_table[group_block_idx];
            sw_db_entry.m_to_set_group_table[group_block_idx] = true;

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "set_group_table block:%u " \
                       U64H_FMT " " U64H_FMT "\n",
                       group_block_idx,
                       sw_db_entry.m_ar_group_table[group_block_idx].Group[0].SubGroup_0,
                       sw_db_entry.m_ar_group_table[group_block_idx].Group[1].SubGroup_0);
        }
    }

    //sw_db_entry.m_total_groups_number = calculated_groups_number;
    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void AdaptiveRoutingManager::ARUpdateSWLFTTable(ARSWDataBaseEntry &sw_db_entry,
        struct SMP_ARLinearForwardingTable p_ar_calculated_lft_table[],
        u_int16_t calculated_max_lid)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    // Update blocks
    // In fact we go over only on blocks we need  - until max lid
    unsigned supported_lft_blocks =
            calculated_max_lid / AR_LFT_TABLE_BLOCK_SIZE;
    for (unsigned lft_block = 0; lft_block <= supported_lft_blocks; ++lft_block) {
        if (sw_db_entry.m_osm_update_needed ||
                !this->IsEqualSMPARLftTableBlock(
                        &p_ar_calculated_lft_table[lft_block],
                        &sw_db_entry.m_ar_lft.m_ar_lft_table[lft_block])) {
                sw_db_entry.m_ar_lft.m_ar_lft_table[lft_block] = p_ar_calculated_lft_table[lft_block];
                sw_db_entry.m_to_set_lft_table[lft_block] = true;
        }
    }

    sw_db_entry.m_max_lid = calculated_max_lid;
    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void AdaptiveRoutingManager::ARUpdateSWLFTTable(ARSWDataBaseEntry &sw_db_entry,
        struct SMP_ARLinearForwardingTable_SX p_ar_calculated_lft_table[],
        u_int16_t calculated_max_lid)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    // Update blocks
    // In fact we go over only on blocks we need  - until max lid
    unsigned supported_lft_blocks =
            calculated_max_lid / AR_LFT_TABLE_BLOCK_SIZE_SX;
    for (unsigned lft_block = 0; lft_block <= supported_lft_blocks; ++lft_block) {
        if (sw_db_entry.m_osm_update_needed ||
                !this->IsEqualSMPARLftTableBlock(
                        &p_ar_calculated_lft_table[lft_block],
                        &sw_db_entry.m_ar_lft.m_ar_lft_table_sx[lft_block])) {
                sw_db_entry.m_ar_lft.m_ar_lft_table_sx[lft_block] = p_ar_calculated_lft_table[lft_block];
                sw_db_entry.m_to_set_lft_table[lft_block] = true;
        }
    }

    sw_db_entry.m_max_lid = calculated_max_lid;
    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


//will consume strings
// string == sw_rank - 1
// sw_rank < string < max_consume_rank
//if fabric uses parallel links max_consume_rank == sw_rank + 1
// will consume only string == sw_rank - 1 (like pure fat tree)
void AdaptiveRoutingManager::UpdateRNRcvString(ARSWDataBaseEntry &sw_db_entry,
                                               uint8_t max_rank,
                                               uint8_t sw_rank,
                                               uint8_t max_consume_rank)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    rn_rcv_string rn_rcv_str;
    CLEAR_STRUCT(rn_rcv_str);

    uint8_t consume_decision =
        (IS_GENERATE_FRN(m_master_db, sw_db_entry) ?
         RN_RCV_STRING_CONSUME_ALL : RN_RCV_STRING_CONSUME_ARN);

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
               "UpdateRNRcvString Switch GUID " U64H_FMT ", LID %u "
               " max_rank:%u sw_rank:%u"
               " max_consume_rank:%u consume_decision:%u\n",
               sw_db_entry.m_general_sw_info.m_guid,
               sw_db_entry.m_general_sw_info.m_lid,
               max_rank, sw_rank, max_consume_rank, consume_decision);

    //memset to 0
    if (sw_rank != 0) {
        //consume sw_rank-1;
        rn_rcv_str.element[sw_rank-1].decision =
            consume_decision;

        //pass on sw_rank
        rn_rcv_str.element[sw_rank].decision = RN_RCV_STRING_PASS_ON;
        rn_rcv_str.element[sw_rank].plft_id = 0;
        rn_rcv_str.element[sw_rank].string2string = sw_rank;
    }

    uint8_t rank = sw_rank+1;

    for (;rank < max_consume_rank;++rank)
        rn_rcv_str.element[rank].decision = consume_decision;

    for (;rank < max_rank;++rank) {
        rn_rcv_str.element[rank].decision = RN_RCV_STRING_PASS_ON;
        rn_rcv_str.element[rank].plft_id = 0;
        rn_rcv_str.element[rank].string2string = rank;
    }

    if (memcmp(&rn_rcv_str, &sw_db_entry.m_rn_rcv_string,
               sizeof(rn_rcv_string))) {
        sw_db_entry.m_to_set_rn_rcv_string = true;
        sw_db_entry.m_rn_rcv_string = rn_rcv_str;

        if ( this->m_p_osm_log->level & OSM_LOG_DEBUG) {
            for (uint8_t i = 0 ; i < max_rank ; ++i)
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "UpdateRNRcvString rec_string:%u decision:%u"
                    " pLFT:%u string_to_string:%u\n",
                    i, rn_rcv_str.element[i].decision,
                    rn_rcv_str.element[i].plft_id,
                    rn_rcv_str.element[i].string2string);
        }
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void AdaptiveRoutingManager::UpdateRNXmitPortMask(
    ARSWDataBaseEntry &sw_db_entry,
    PortsBitset &ca_ports,
    PortsBitset &sw_ports,
    bool is_down_sw)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    if (is_down_sw) {

        u_int8_t *p_sw_mask_element;
        u_int8_t calc_mask = 0;

        if(IS_GENERATE_ARN(m_master_db, sw_db_entry))
            calc_mask |= RN_XMIT_PORT_MASK_GENERATE_ARN;

        if(IS_GENERATE_FRN(m_master_db, sw_db_entry))
            calc_mask |= RN_XMIT_PORT_MASK_GENERATE_FRN;

        calc_mask |= RN_XMIT_PORT_MASK_PASS_ON_RN;

        for (u_int8_t port_num = 1;
             port_num <= sw_db_entry.m_general_sw_info.m_num_ports;
             ++port_num) {

            p_sw_mask_element = &sw_db_entry.m_rn_xmit_port_mask.
                element[port_num];

            if (ca_ports.test(port_num) && *p_sw_mask_element != 0) {

                *p_sw_mask_element = 0;
                sw_db_entry.m_to_set_rn_xmit_port_mask = true;

                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                           "UpdateRNXmitPortMask Switch GUID " U64H_FMT
                           " , LID %u Port %u (to CA)"
                           " gen_arn = gen_frn = pass_on = 0\n",
                           sw_db_entry.m_general_sw_info.m_guid,
                           sw_db_entry.m_general_sw_info.m_lid,
                           port_num);


            } else if (sw_ports.test(port_num) && *p_sw_mask_element != calc_mask) {

                *p_sw_mask_element = calc_mask;
                sw_db_entry.m_to_set_rn_xmit_port_mask = true;

                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                           "UpdateRNXmitPortMask Switch GUID " U64H_FMT
                           " , LID %u Port %u (to SW)"
                           " gen_arn %u gen_frn %u pass_on %u\n",
                           sw_db_entry.m_general_sw_info.m_guid,
                           sw_db_entry.m_general_sw_info.m_lid,
                           port_num,
                           calc_mask & RN_XMIT_PORT_MASK_GENERATE_ARN,
                           calc_mask & RN_XMIT_PORT_MASK_GENERATE_FRN,
                           calc_mask & RN_XMIT_PORT_MASK_PASS_ON_RN);
            }

        }

        //reset doun ports
        if (sw_db_entry.m_to_set_rn_xmit_port_mask) {
            PortsBitset disconnected_ports = ca_ports | sw_ports;
            disconnected_ports.flip();

            for (u_int8_t port_num = 1;
                 port_num <= sw_db_entry.m_general_sw_info.m_num_ports;
                 ++port_num) {

                p_sw_mask_element = &sw_db_entry.m_rn_xmit_port_mask.
                    element[port_num];

                if (disconnected_ports.test(port_num) && *p_sw_mask_element != 0) {

                    *p_sw_mask_element = 0;
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                               "UpdateRNXmitPortMask Switch GUID " U64H_FMT
                               " , LID %u Port %u (disconnected)"
                               " gen_arn = gen_frn = pass_on = 0\n",
                               sw_db_entry.m_general_sw_info.m_guid,
                               sw_db_entry.m_general_sw_info.m_lid,
                               port_num);
                }
            }

        }

    } else {
        //clear all xmit_port_mask
        rn_xmit_port_mask clear_mask;
        CLEAR_STRUCT(clear_mask);

        if (memcmp(&sw_db_entry.m_rn_xmit_port_mask, &clear_mask,
                   sizeof(clear_mask))) {
            sw_db_entry.m_rn_xmit_port_mask = clear_mask;
            sw_db_entry.m_to_set_rn_xmit_port_mask = true;

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "UpdateRNXmitPortMask Switch GUID " U64H_FMT
                       " , LID %u clear mask (no down SW)\n",
                       sw_db_entry.m_general_sw_info.m_guid,
                       sw_db_entry.m_general_sw_info.m_lid);
        }

    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

/***************/
void AdaptiveRoutingManager::ARCalculatePortGroupsParallelLinks()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    // some variables for calculation
    PortsBitset ports_per_remote_node[IB_LID_UCAST_END_HO + 1];
    u_int8_t port_2_group[MAX_SUPPORTED_PORTS];
    SMP_ARGroupTable calculated_ar_group_table[AR_GROUP_TABLE_NUM_BLOCKS];
    SMP_AR_LFT calculated_ar_lft;
    //SMP_ARLinearForwardingTable calculated_ar_lft_table[AR_LFT_TABLE_NUM_BLOCKS];
    unsigned calculated_max_pg_id;
    unsigned calculated_max_lid;

    PortGroup_Block_Element *p_element = NULL;
    PortsBitset group_bitset;

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Calculate Port Groups Parallel Links. (switches number:%u)\n",
               (int)this->m_sw_db.m_sw_map.size());

    /*
     * Go through all the switches and create their port groups.
     * In this algorithm ports groups are created out of "parallel"
     * links - links that are connecting the same pair of nodes.
     * So here's what we're doing here:
     *   - for each switch
     *       - for each port of the switch (port_num)
     *           - get the remote node
     *           - if the remote node is CA or RTR
     *               - no need to create a group, as there can be
     *                 only one link to its port - skip this port
     *           - if remote node is switch
     *               - get remote switch's LID (remote_lid)
     *               - set bit num. port_num in the bitmask that holds
     *                 bit for every port that has this LID as a remote
     *                 side of the link (ports_per_remote_node[remote_lid])
     *       - at this point we have all the ports organized in groups
     *         by the remote node (ports_per_remote_node[])
     *       - set the groups table according to ports_per_remote_node[]
     *       - create port_2_group map
     *       - set the lft table according to port_2_group and ucast_lft_port used before
     */
    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    while (sw_node_it != this->m_sw_db.m_sw_map.end()) {
        /*
         * Step 1.1 -
         *  Ensure that this switch supports AR, and that AR is enabled.
         */
        if (!this->IsARActive(sw_node_it->second)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Switch GUID " U64H_FMT ", LID %u: " \
                    "AR/FR not supported or not enabled, calculate of port groups skipped.\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid);
            //we already suppose to have this switch in not supported list
            goto next_switch;
        }

        /*
         * Step 1.2 -
         *  Set default values for calculations
         */
        memset(ports_per_remote_node, 0, sizeof(ports_per_remote_node));
        memset(port_2_group, OSM_NO_PATH, sizeof(port_2_group));    // default is no group for this port
        this->ARGroupTableSetDefault(calculated_ar_group_table);
        if (sw_node_it->second.m_general_sw_info.m_sx_dev) 
            this->ARLftTableSetDefault(calculated_ar_lft.m_ar_lft_table_sx);
        else 
            this->ARLftTableSetDefault(calculated_ar_lft.m_ar_lft_table);

        calculated_max_pg_id = (m_master_db.m_fr_enable ? 1 : 0);
        calculated_max_lid = 0;

        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Calculate LAG groups for Switch GUID " U64H_FMT ", LID %u \n" ,
            sw_node_it->second.m_general_sw_info.m_guid,
            sw_node_it->second.m_general_sw_info.m_lid);

        /*
         * Step 2 -
         *  Iterate through all the switch ports and fill ports_per_remote_node
         *  We create bitmask for each remote node that is connected to this switch.
         *  Each bit that is set represent a port number that connected to the remote node that
         *  this bit mask belongs to.
         */
        for (u_int8_t port_num = 1; port_num <= sw_node_it->second.m_general_sw_info.m_num_ports; ++port_num) {
            osm_node_t *p_node = sw_node_it->second.m_general_sw_info.m_p_osm_sw->p_node;
            osm_node_t *p_remote_node = osm_node_get_remote_node(p_node, port_num, NULL);

            osm_physp_t *p_physp = osm_node_get_physp_ptr(p_node, port_num);
            if (!p_physp || !osm_link_is_healthy(p_physp))
                continue;
            osm_physp_t *p_remote_physp = osm_physp_get_remote(p_physp);
            if (!p_remote_physp)
                continue;

            // Ignore any loopback connection on switch
            if (p_node == p_remote_node) {
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Ignoring loopback on "  \
                        "Switch GUID " U64H_FMT ", LID %u, during calculate of port groups.\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid);
                continue;
            }

            u_int64_t remote_node_guid = ADB2C_BE64_TO_CPU(osm_node_get_node_guid(p_remote_node));
            u_int8_t remote_node_type = osm_node_get_type(p_remote_node);
            u_int16_t remote_node_lid;
            if (remote_node_type == IB_NODE_TYPE_ROUTER || remote_node_type == IB_NODE_TYPE_CA) {
                remote_node_lid = ADB2C_BE16_TO_CPU(osm_physp_get_base_lid(p_remote_physp));
            } else if (remote_node_type == IB_NODE_TYPE_SWITCH) {
                remote_node_lid = ADB2C_BE16_TO_CPU(osm_node_get_base_lid(p_remote_node, 0));
            } else {
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                        "%s  Switch GUID " U64H_FMT " - Unknown node type"  \
                        "type: %u, calculate of port groups skipped.\n",
                        ARMGR_ERR_PREFIX9, remote_node_guid, remote_node_type);
                throw 1;
            }

            // Mark that port number port_num routes to lid remote_node_lid.
            // Note that bit number represent port number.
            ports_per_remote_node[remote_node_lid].set(port_num);

            if (this->m_p_osm_log->level & OSM_LOG_DEBUG) {
    			OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Update ports_per_remote_node on "  \
    					"Switch GUID " U64H_FMT ", LID: %u : port_num %u, ports_per_remote_node[%u]=%s.\n",
    					   sw_node_it->second.m_general_sw_info.m_guid,
    					   sw_node_it->second.m_general_sw_info.m_lid,
    					   port_num,
    					   remote_node_lid,
    					   ports_per_remote_node[remote_node_lid].to_string().c_str());
            }

        }   /* End of loop switch ports */

        /*
         * Step 3 -
         *  Create all port groups in the following way:
         *  At this point we have all the ports marked their bits in the ports_per_remote_node array
         *  Bits that are marked in the same array entry effectively form port group.
         *
         *  Number of port groups <= number of switch ports.
         *
         *  Check all the entries of the ports_per_remote_node array.
         *  If an entry contain more than one port and the entry is not all 0, create port group.
         */
        for (unsigned lid_num = 1; lid_num <= m_p_osm_subn->max_lid; ++lid_num) {
            if (!ports_per_remote_node[lid_num].any())
                continue;

            // count number of ports that are connected directly to the same LID (remote node) */
            u_int8_t ports_counter = 0;
            for (unsigned i = 1; i <= sw_node_it->second.m_general_sw_info.m_num_ports; ++i)
                if (ports_per_remote_node[lid_num].test(i))
                    ++ports_counter;

            // If there is only one port in such group, then the AR mode would be Static, and there is no
            // need to create port group for it - the GroupNumber in the LFT entry for Static LID
            // State is actually a port number. So, if we see a port that is mapped to group
            // id OSM_NO_PATH, it means that the port is alone - no port group was created for it.
            if (ports_counter == 1)
                continue;

            //calculated_ar_group_table[calculated_max_pg_id / AR_GROUP_TABLE_BLOCK_SIZE].Group[calculated_max_pg_id % AR_GROUP_TABLE_BLOCK_SIZE].SubGroup_0 =
            //        ports_per_remote_node[lid_num];
            p_element = &calculated_ar_group_table[calculated_max_pg_id / AR_GROUP_TABLE_BLOCK_SIZE].Group[calculated_max_pg_id % AR_GROUP_TABLE_BLOCK_SIZE];
            ports_per_remote_node[lid_num].get_data(p_element->SubGroup_0, p_element->SubGroup_1, p_element->SubGroup_2, p_element->SubGroup_3);

            ++calculated_max_pg_id;
        }      /* End of loop all lids */
        if (calculated_max_pg_id)
            --calculated_max_pg_id;

        /*
         * Step 4.1 -
         *  Init port-2-group mapping.
         *  Value OSM_NO_PATH (255) in port-2-group mapping means that the port is mapped to itself,
         *  which is valid in case of Static LID State.
         */
        for (unsigned pg_id = 0; pg_id <= calculated_max_pg_id; ++pg_id) {
            //u_int64_t bitamsk = calculated_ar_group_table[pg_id / AR_GROUP_TABLE_BLOCK_SIZE].Group[pg_id % AR_GROUP_TABLE_BLOCK_SIZE].SubGroup_0;
            p_element = &calculated_ar_group_table[pg_id / AR_GROUP_TABLE_BLOCK_SIZE].Group[pg_id % AR_GROUP_TABLE_BLOCK_SIZE];
            group_bitset.set(p_element->SubGroup_0, p_element->SubGroup_1, p_element->SubGroup_2, p_element->SubGroup_3);

            if (!group_bitset.any())
                // no ports in this group - skip it
                continue;

            for (unsigned i = 0; i <= sw_node_it->second.m_general_sw_info.m_num_ports; ++i)
                if (group_bitset.test(i))
                    port_2_group[i] = pg_id;
        }      /* End of loop valid port groups */
        //TODO enter assert regarding port zero

        /*
         * Step 4.2 -
         *  Now let's configure AR LFTs.
         */
        for (unsigned dest_lid = 1; dest_lid <= m_p_osm_subn->max_lid; ++dest_lid) {
            // Find out which port was used to route to lid_num in the usual unicast routing
            /*u_int8_t ucast_lft_port =
                    osm_switch_get_port_by_lid(sw_node_it->second.m_general_sw_info.m_p_osm_sw,
                            dest_lid);*/
            u_int8_t ucast_lft_port =
                    OsmSwitchGetPortByLidUseNewLft(sw_node_it->second.m_general_sw_info.m_p_osm_sw,
                            dest_lid);
            if (ucast_lft_port == OSM_NO_PATH)  //no route to this lid_num from this switch
                continue;
            calculated_max_lid = dest_lid;

            u_int8_t lft_entry_num;
            if (sw_node_it->second.m_general_sw_info.m_sx_dev) //SX Device
                lft_entry_num = dest_lid % AR_LFT_TABLE_BLOCK_SIZE_SX;
            else
                lft_entry_num = FixLFTEntryNum(dest_lid % AR_LFT_TABLE_BLOCK_SIZE);

            // Check port_2_group mapping of this port.
            // If there is no port group for this port (the mapped port group ID is OSM_NO_PATH),
            // then it means that the port is alone. In that case, LID state would be Static,
            // which means that the GroupNumber in the LFT entry is actually a port number.
            //
            // Note: if (lid_num == my LID), then (ucast_lft_port == 0), and its port_2_group
            // mapping will be OSM_NO_PATH. This is loopback scenario.
            unsigned pg_id = port_2_group[ucast_lft_port];
            if (sw_node_it->second.m_general_sw_info.m_sx_dev) { //SX Device

                unsigned lft_block_num = dest_lid / AR_LFT_TABLE_BLOCK_SIZE_SX;
                LidPortMapping_Block_Element_SX &lid_entry = calculated_ar_lft.
                    m_ar_lft_table_sx[lft_block_num].LidEntry[lft_entry_num];

                lid_entry.DefaultPort = ucast_lft_port;
                if (pg_id != OSM_NO_PATH) {
                    lid_entry.GroupNumber = pg_id;
                    lid_entry.LidState = (this->m_master_db.m_ar_enable ?
                                          this->m_master_db.m_ar_mode :
                                          AR_IB_LID_STATE_STATIC);
                } else {
                    lid_entry.GroupNumber = 0;
                    lid_entry.LidState = AR_IB_LID_STATE_STATIC;
                }
            } else { //NOT SX Device

                if (pg_id == OSM_NO_PATH) {
                    calculated_ar_lft.m_ar_lft_table[dest_lid / AR_LFT_TABLE_BLOCK_SIZE].LidEntry[lft_entry_num].GroupNumber =
                            ucast_lft_port;
                    // We already mark static in lid state by default
                } else {
                    calculated_ar_lft.m_ar_lft_table[dest_lid / AR_LFT_TABLE_BLOCK_SIZE].LidEntry[lft_entry_num].GroupNumber =
                            pg_id;
                    calculated_ar_lft.m_ar_lft_table[dest_lid / AR_LFT_TABLE_BLOCK_SIZE].LidEntry[lft_entry_num].LidState =
                            this->m_master_db.m_ar_mode;
                }

            }
        }      /* End of loop destination lids */

        /*
         * Step 5 -
         *  Now we will decide if the calculated settings are different from old,
         *  or if we need to ignore old settings and use the new ones
         */
        this->ARUpdateSWGroupTable(sw_node_it->second,
                calculated_ar_group_table,
                calculated_max_pg_id+1);

        if (sw_node_it->second.m_general_sw_info.m_sx_dev)  //SX Device
            this->ARUpdateSWLFTTable(sw_node_it->second,
                    calculated_ar_lft.m_ar_lft_table_sx,
                    calculated_max_lid);
        else
            this->ARUpdateSWLFTTable(sw_node_it->second,
                    calculated_ar_lft.m_ar_lft_table,
                    calculated_max_lid);

        /*
         * Step 6 -
         * Now dump debug information to osm log
         */
        this->ARDumpSWSettings(sw_node_it->second);

next_switch:
        ++sw_node_it;
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


u_int16_t AdaptiveRoutingManager::AllocateSwArGroup(uint16_t sw_lid,
                                                    uint16_t group_cap)
{
    //After we configure the fabric once, all leaf switches should be assigned
    //with AR groups so we optimize by not locking m_sw_db.m_db_lock on read
    uint16_t ar_group_number = m_sw_db.m_sw_lid_to_group_number[sw_lid];

    if (ar_group_number != AR_GROUP_UNASSIGNED && ar_group_number < group_cap) {
        return ar_group_number;
    }

    ar_group_number = AR_GROUP_UNASSIGNED;
    pthread_mutex_lock(&m_sw_db.m_db_lock);

    if (m_sw_db.m_sw_lid_to_group_number[sw_lid] == AR_GROUP_UNASSIGNED ||
        m_sw_db.m_sw_lid_to_group_number[sw_lid] >= group_cap) {

        if (!m_sw_db.m_free_ar_group_numbers.empty()) {
            set <uint16_t>::iterator iter =
                m_sw_db.m_free_ar_group_numbers.begin();

            if (*iter < group_cap) {
                ar_group_number = *iter;
                m_sw_db.m_free_ar_group_numbers.erase(iter);
            }
        } else if (m_sw_db.m_next_ar_group_number < group_cap){
            ar_group_number = m_sw_db.m_next_ar_group_number;
            m_sw_db.m_next_ar_group_number++;
        }

        if (ar_group_number != AR_GROUP_UNASSIGNED) {
            if (m_sw_db.m_sw_lid_to_group_number[sw_lid] ==
                AR_GROUP_UNASSIGNED) {
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                           "AllocateSwArGroup Sw_Lid:%u Group:%u.\n",
                           sw_lid, ar_group_number);
            } else {
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                           "Replace SwArGroup Sw_Lid:%u Group:%u with:%u.\n",
                           sw_lid,
                           m_sw_db.m_sw_lid_to_group_number[sw_lid],
                           ar_group_number);
                m_sw_db.m_free_ar_group_numbers.insert(
                    m_sw_db.m_sw_lid_to_group_number[sw_lid]);
            }
            m_sw_db.m_sw_lid_to_group_number[sw_lid] = ar_group_number;

        } else
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Failed to AllocateSwArGroup Sw_Lid:%u.\n",
                       sw_lid);
    } else
        ar_group_number = m_sw_db.m_sw_lid_to_group_number[sw_lid];

    pthread_mutex_unlock(&m_sw_db.m_db_lock);

    return ar_group_number;
}

int AdaptiveRoutingManager::SetHcaToSwLidMapping(osm_physp_t *p_hca_physp,
                                                 osm_node_t *p_remote_sw_node,
                                                 u_int16_t *hca_to_sw_lid_mapping )
{

    u_int16_t node_lid = ADB2C_BE16_TO_CPU(osm_physp_get_base_lid(p_hca_physp));
    if (p_remote_sw_node->node_info.node_type != IB_NODE_TYPE_SWITCH ||
        node_lid == 0 ) {
        return -1;
    }

    uint8_t lmc = osm_physp_get_lmc(p_hca_physp);

    u_int16_t remote_node_lid = ADB2C_BE16_TO_CPU(osm_node_get_base_lid(p_remote_sw_node, 0));

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Map (HCA/Router) LID %u lmc:%u to SW LID %u.\n",
            node_lid, lmc, remote_node_lid);

    u_int16_t numLidsPerPort = (u_int16_t)((u_int16_t)1 << lmc);
    u_int16_t lid_limit = node_lid+numLidsPerPort;

    for (u_int16_t l = node_lid; l < lid_limit; l++)
        hca_to_sw_lid_mapping[l] = remote_node_lid;

    //Handle vports
    LidsList vlids_list;
    GetVlidsList(p_hca_physp, vlids_list);
    if (!vlids_list.empty()) {
        for (LidsList::const_iterator iter = vlids_list.begin();
             iter != vlids_list.end(); ++iter) {

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Map VLID %u to SW LID %u.\n",
                       *iter, remote_node_lid);
            hca_to_sw_lid_mapping[*iter] = remote_node_lid;
        }
    }

    return 0;
}

void AdaptiveRoutingManager::GetVlidsList(osm_physp_t *p_hca_physp,
                                          LidsList &vlids_list)
{
    OSM_AR_LOG_ENTER(m_p_osm_log);

    osm_port *p_osm_port = osm_get_port_by_guid(m_p_osm_subn,
                                                p_hca_physp->port_guid);

    if (p_osm_port == NULL) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                   "Failed to find osm_port for port GUID: " U64H_FMT ".\n",
                   ADB2C_BE64_TO_CPU(p_hca_physp->port_guid));
        return;
    }

    uint16_t vport_top_index = osm_port_get_vport_top_index(p_osm_port);
    osm_vport * p_osm_vport;
    u_int16_t vlid;
    for (uint16_t vport_index = 1; vport_index <= vport_top_index;
         ++vport_index) {
        p_osm_vport = osm_port_get_vport(p_osm_port, vport_index);
        if (p_osm_vport == NULL) {
            continue;
        }

        vlid = ADB2C_BE16_TO_CPU(osm_vport_calc_lid(p_osm_vport));

        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                   "Found VLID %u on CA GUID: " U64H_FMT ".\n",
                   vlid, ADB2C_BE64_TO_CPU(p_hca_physp->port_guid));

        vlids_list.push_back(vlid);
    }
    OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
}

/***************/
inline void UpdateCalculatedArGroupTable(
    SMP_ARGroupTable calculated_ar_group_table[AR_GROUP_TABLE_NUM_BLOCKS],
    u_int16_t group_number,
    PortsBitset &group_bitmask,
    unsigned &calculated_max_pg_id)
{
    PortGroup_Block_Element *p_element =
        &calculated_ar_group_table[group_number / AR_GROUP_TABLE_BLOCK_SIZE].
            Group[group_number % AR_GROUP_TABLE_BLOCK_SIZE];

    group_bitmask.get_data(p_element->SubGroup_0,
                           p_element->SubGroup_1,
                           p_element->SubGroup_2,
                           p_element->SubGroup_3);

    if (group_number > calculated_max_pg_id)
        calculated_max_pg_id = group_number;
}

#if 0
void AdaptiveRoutingManager::ARCalculatePortGroupsTree()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "ARCalculatePortGroupsTree.\n");

    // some variables for calculation
    SMP_ARGroupTable calculated_ar_group_table[AR_GROUP_TABLE_NUM_BLOCKS];
    SMP_AR_LFT calculated_ar_lft;
    //SMP_ARLinearForwardingTable calculated_ar_lft_table[AR_LFT_TABLE_NUM_BLOCKS];
    unsigned calculated_max_pg_id;
    unsigned calculated_max_lid;
    int rc = 0;

    TreeAlgorithmData algorithm_data;
    uint16_t max_lid = 0;
    uint8_t **hops = NULL;
    GroupData group_data;
    LidPortMappingPtrBase  *p_lft_entry_ptr = NULL;
    LidPortMappingPtr       lft_entry_ptr;
    LidPortMappingPtrSX     lft_entry_ptr_sx;

    //for ARN/FRN consume string calculation
    PortsBitset down_sw_ports;
    PortsBitset ca_ports;
    PortsBitset sw_ports;

    PortGroup_Block_Element empty_element;
    memset(&empty_element, 0, sizeof(empty_element));

    uint8_t sw_rank = OSM_SW_NO_RANK;
    uint8_t max_rank = 0;
    uint8_t max_consume_rank = 0;

    //u_int8_t last_port_num;

    //Minimal hops array contains information for switches only
    //find for each hca the connected switch and fill in hca_to_sw_lid_mapping
    u_int16_t hca_to_sw_lid_mapping[IB_LID_UCAST_END_HO + 1] = {};
    uint8_t sw_lid_to_rank_mapping[IB_LID_UCAST_END_HO + 1] = {};
    memset(sw_lid_to_rank_mapping, 0xFF, sizeof(sw_lid_to_rank_mapping));

    osm_node_t *p_osm_node;
    // Go over all nodes in osm database
    for (p_osm_node =  (osm_node_t *) cl_qmap_head(&m_p_osm_subn->node_guid_tbl);
         p_osm_node != (osm_node_t *) cl_qmap_end(&m_p_osm_subn->node_guid_tbl);
         p_osm_node =  (osm_node_t *) cl_qmap_next(&p_osm_node->map_item)) {

        if (p_osm_node->node_info.node_type == IB_NODE_TYPE_SWITCH) {
            //set sw_lid_to_rank_mapping
            if (m_master_db.m_frn_enable || m_master_db.m_arn_enable) {

                u_int16_t lid = ADB2C_BE16_TO_CPU(
                    osm_node_get_base_lid(p_osm_node, 0));

                if (p_osm_node->sw->rank == OSM_SW_NO_RANK) {
                    m_master_db.m_frn_enable = false;
                    m_master_db.m_arn_enable = false;

                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO,
                            "Switch GUID " U64H_FMT ", LID %u: "
                            "has no rank - can not support ARN/FRN.\n",
                            ADB2C_BE64_TO_CPU(osm_node_get_node_guid(p_osm_node)),
                            lid);

                } else if (p_osm_node->sw->rank > MAX_SUPPORTED_RANK) {
                    m_master_db.m_frn_enable = false;
                    m_master_db.m_arn_enable = false;

                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO,
                            "Switch GUID " U64H_FMT ", LID %u: "
                            "has rank:%u which exceeds maximum.\n",
                            ADB2C_BE64_TO_CPU(osm_node_get_node_guid(p_osm_node)),
                            lid,
                            p_osm_node->sw->rank);

                }

                sw_lid_to_rank_mapping[lid] = p_osm_node->sw->rank;
                max_rank = max(max_rank,p_osm_node->sw->rank);
            }
            continue;
        }

        for( u_int8_t port_num = 1 ; port_num <= p_osm_node->node_info.num_ports ; port_num++) {
            osm_physp_t *p_physp = osm_node_get_physp_ptr(p_osm_node, port_num);
            osm_node_t *p_remote_node = osm_node_get_remote_node(p_osm_node, port_num, NULL);

            if (!p_physp || !osm_link_is_healthy(p_physp))
                continue;
            osm_physp_t *p_remote_physp = osm_physp_get_remote(p_physp);
            if (!p_remote_physp)
                continue;

            SetHcaToSwLidMapping(p_physp, p_remote_node, hca_to_sw_lid_mapping);
        }
    }

    /*
     * Go through all the switches and create their port groups.
     * In this algorithm ports groups are created out of minimal hops
     * links - each lid will recieve a group containing all the ports
     * with minimal hops to this lid.
     * So here's what we're doing here:
     *   - for each switch
     *       - for each port of the switch (port_num)
     *           - calculate group bitmask for this lid
     *           - asisign groups acording to device limitations
     */
    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    while (sw_node_it != this->m_sw_db.m_sw_map.end()) {

        bool is_group_per_leaf_sw = false;
        /*
         * Step 1.1 -
         *  Ensure that this switch supports AR, and that AR is enabled.
         */
        if (!this->IsARActive(sw_node_it->second)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "Skip Switch GUID " U64H_FMT ", LID %u: " \
                       "AR not supported or not enabled.\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid);
            //we already suppose to have this switch in not supported list
            goto next_switch;
        }

        /*
         * Step 1.2 -
         *  Set default values for calculations
         */
        this->ARGroupTableSetDefault(calculated_ar_group_table);
        if (sw_node_it->second.m_general_sw_info.m_sx_dev)
            this->ARLftTableSetDefault(calculated_ar_lft.m_ar_lft_table_sx);
        else
            this->ARLftTableSetDefault(calculated_ar_lft.m_ar_lft_table);

        calculated_max_pg_id = 0;
        calculated_max_lid = 0;
        algorithm_data.Clear();

        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                "Calculate group bitmask for GUID " U64H_FMT ", LID %u: \n",
                sw_node_it->second.m_general_sw_info.m_guid,
                sw_node_it->second.m_general_sw_info.m_lid);

        /*
         * Step 1.3 -
         *  Build down switch ports and ca ports bitset
         */
        sw_node_it->second.m_config_rn =
            (IS_GENERATE_ARN(m_master_db, sw_node_it->second) ||
             IS_GENERATE_FRN(m_master_db, sw_node_it->second));
        is_group_per_leaf_sw = sw_node_it->second.m_config_rn;

        if (sw_node_it->second.m_config_rn) {

            down_sw_ports.reset();
            ca_ports.reset();
            sw_ports.reset();

            sw_rank = sw_node_it->second.m_general_sw_info.m_p_osm_sw->rank;

            osm_node_t *p_osm_node =
                sw_node_it->second.m_general_sw_info.m_p_osm_sw->p_node;

            for( u_int8_t port_num = 1 ;
                 port_num <= p_osm_node->node_info.num_ports ; port_num++) {

                osm_physp_t *p_physp = osm_node_get_physp_ptr(p_osm_node, port_num);
                if (!p_physp || !osm_link_is_healthy(p_physp))
                    continue;

                osm_physp_t *p_remote_physp = osm_physp_get_remote(p_physp);
                if (!p_remote_physp || !p_remote_physp->p_node)
                    continue;


                if(p_remote_physp->p_node->sw){
                    sw_ports.set(port_num);
                    if (p_remote_physp->p_node->sw->rank > sw_rank)
                        down_sw_ports.set(port_num);
                } else
                    ca_ports.set(port_num);

                /*
                if(!p_remote_physp->p_node->sw)
                    ca_ports.set(port_num);
                else if (p_remote_physp->p_node->sw->rank > sw_rank)
                    down_sw_ports.set(port_num);
                    */
            }

        }


        /*
         * Step 2.1 -
         *  Iterate through all lids and calculate group bitmask for each lid.
         */
        max_lid = sw_node_it->second.m_general_sw_info.m_p_osm_sw->max_lid_ho;
        hops = sw_node_it->second.m_general_sw_info.m_p_osm_sw->hops;
        //u_int8_t last_port_num;

        uint16_t sw_lid_num;
        for (uint16_t lid_num = 1; lid_num <= max_lid; ++lid_num) {

            if (!hops[lid_num]) {
                //this is not sw lid. get the sw lid connected to this lid
                sw_lid_num = hca_to_sw_lid_mapping[lid_num];
                if (sw_lid_num == 0) {
                    continue;
                }
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "hops[lid:%u] = hops[sw_lid:%u]+1 \n",lid_num , sw_lid_num);
            } else
                sw_lid_num = lid_num;

            if (!hops[sw_lid_num] || hops[sw_lid_num][0] < 1 || hops[sw_lid_num][0] == OSM_NO_PATH) {
                if(!hops[sw_lid_num]) {
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "hops[lid:%u] == NULL \n", sw_lid_num);
                } else {
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "hops[lid:%u][0] == %u \n", sw_lid_num, hops[sw_lid_num][0]);
                }
                continue;
            }

            LidToGroupDataMapIter lid_to_group_iter = algorithm_data.m_lid_to_group_map.find(sw_lid_num);
            if (lid_to_group_iter != algorithm_data.m_lid_to_group_map.end()) {

                AddLidToARGroup(lid_num, sw_lid_num, lid_to_group_iter->second,
                                algorithm_data, false,
                                is_group_per_leaf_sw);
                continue;
            }

            group_data.m_group_bitmask.reset();
            group_data.m_group_size = 0;


            /*
             *  Calculate group bitmask.
             */
            for (u_int8_t port_num = 1; port_num <= sw_node_it->second.m_general_sw_info.m_num_ports; ++port_num) {            
                if(hops[sw_lid_num][0] == hops[sw_lid_num][port_num]) {                
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "hops[%u][%u]:%u \n", sw_lid_num, port_num , hops[sw_lid_num][port_num]);
                    group_data.m_group_bitmask.set(port_num);
                    group_data.m_group_size++;
                    //OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "bitset:%s size:%u \n", 
                    //           group_data.m_group_bitmask.to_string().c_str(),group_data.m_group_size);
                    //last_port_num = port_num;
                }
            }

            if (group_data.m_group_size > 1 || sw_node_it->second.m_config_rn){

                GroupBitmaskToDataMapInsertRes res =
                    algorithm_data.m_groups_map.insert(
                        make_pair(group_data.m_group_bitmask,group_data));

                AddLidToARGroup(lid_num, sw_lid_num, &res.first->second,
                                algorithm_data, true,
                                is_group_per_leaf_sw);
            }
        }

        /*
         * Step 2.2 -
         *  try to assign group number and group table number(for IS4) for all groups.
         */
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Step 2.2 AssignPortGroups \n");

        if (!algorithm_data.m_lid_to_group_map.empty()) {       
            rc = AssignPortGroups(sw_node_it->second, algorithm_data);
            if (rc) 
                goto next_switch;
        }

        /*
         * Step 2.3 -
         *  Calculate min string to consume.
         */

        max_consume_rank = sw_rank;
        if (sw_node_it->second.m_config_rn &&
            !algorithm_data.m_assign_groups.empty()) {

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Step 2.3 Calculate min string to consume."
                       " sw_rank:%u\n",
                       sw_rank);

            GroupsListIter list_iter = algorithm_data.m_assign_groups.begin();
            uint8_t grp_min_rank = sw_rank;

            for ( ;
                  list_iter != algorithm_data.m_assign_groups.end();
                  ++list_iter) {

                PrintGroupData("Calculate consume_rank for group: ",
                               *(*list_iter));

                if ((*list_iter)->m_group_size > 1 &&
                    ((*list_iter)->m_group_bitmask & down_sw_ports).any() ) {

                        grp_min_rank = max_rank;

                        for (list<uint16_t>::iterator lid_iter =
                                (*list_iter)->m_lids_list.begin() ;
                             lid_iter != (*list_iter)->m_lids_list.end();
                             ++lid_iter)

                            grp_min_rank =
                                min(grp_min_rank,
                                    sw_lid_to_rank_mapping[*lid_iter]);

                        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                                   "grp_min_rank:%u \n", grp_min_rank);

                        //ignore parallel links (grp_min_rank == sw_rank+1)
                        if (grp_min_rank < sw_rank+2 ||
                            max_consume_rank == grp_min_rank)
                            continue;

                        if (max_consume_rank == sw_rank)
                            max_consume_rank = grp_min_rank;
                        else {
                            OSM_AR_LOG(
                                this->m_p_osm_log, OSM_LOG_INFO,
                                "Group minimal rank (%u) "
                                "is different from current rank (%u) "
                                "for GUID " U64H_FMT ", LID %u: bitmask:%s\n",
                                grp_min_rank, max_consume_rank,
                                sw_node_it->second.m_general_sw_info.m_guid,
                                sw_node_it->second.m_general_sw_info.m_lid,
                                (*list_iter)->m_group_bitmask.
                                    to_string().c_str());

                            //set maximal rank
                            if (max_consume_rank > grp_min_rank)
                                max_consume_rank = grp_min_rank;
                        }

                        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                                   "max_consume_rank:%u \n", max_consume_rank);


                }


            }
       }


        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Step 3\n");

        /*
         * Step 3 -
         *  Update calculated_ar_group_table:
         *  At this point we have all sggigned groups in algorithm_data.m_assign_groups list
         */
        if (!algorithm_data.m_assign_groups.empty()) {
        
            GroupsListIter list_iter = algorithm_data.m_assign_groups.begin();
            PortGroup_Block_Element *p_element = NULL;
            if (sw_node_it->second.m_general_sw_info.m_sx_dev) {

                //In case we use group_per_leaf_sw we optimize ARGroup MAD sending by using copy
                //otherwize we use UpdateCalculatedArGroupTable
                if (!is_group_per_leaf_sw) {

                    for ( ;
                          list_iter != algorithm_data.m_assign_groups.end();
                          ++list_iter) {
                          UpdateCalculatedArGroupTable(calculated_ar_group_table,
                                                       (*list_iter)->m_group_number,
                                                       (*list_iter)->m_group_bitmask,
                                                       calculated_max_pg_id);
                    }
                }
            } else {
    
                uint16_t group_block_number;
                uint16_t group_block_idx;
                uint16_t total_group_idx;
    
                for ( ;
                      list_iter != algorithm_data.m_assign_groups.end();
                      ++list_iter) {
                    group_block_number = 
                        GROUP_BLOCKS_NUMBER(sw_node_it->second.m_ar_info.group_cap,
                                            sw_node_it->second.m_required_ar_info)*
                        (*list_iter)->m_group_table_number +
                        GROUP_BLOCKS_NUMBER((*list_iter)->m_group_number+1,
                                            sw_node_it->second.m_required_ar_info);

                    group_block_idx = group_block_number-1;

                    p_element = &calculated_ar_group_table[group_block_idx].Group[(*list_iter)->m_group_number%AR_GROUP_TABLE_BLOCK_SIZE];
                    (*list_iter)->m_group_bitmask.get_data(p_element->SubGroup_0, p_element->SubGroup_1, p_element->SubGroup_2, p_element->SubGroup_3);
    
                    total_group_idx = (group_block_number*AR_GROUP_TABLE_BLOCK_SIZE)-1;
                    if (total_group_idx > calculated_max_pg_id)
                        calculated_max_pg_id = total_group_idx;
                }
            }
        }

        /*
         * Step 4 -
         *  Now let's configure AR LFTs.
         */
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Step 4 configure AR LFTs \n");
        for (unsigned dest_lid = 1; dest_lid <= m_p_osm_subn->max_lid; ++dest_lid) {
            // Find out which port was used to route to lid_num in the usual unicast routing
            /*u_int8_t ucast_lft_port =
                    osm_switch_get_port_by_lid(sw_node_it->second.m_general_sw_info.m_p_osm_sw,
                            dest_lid);*/
            u_int8_t ucast_lft_port =
                    OsmSwitchGetPortByLidUseNewLft(sw_node_it->second.m_general_sw_info.m_p_osm_sw,
                            dest_lid);
            if (ucast_lft_port == OSM_NO_PATH)  //no route to this lid_num from this switch
                continue;
            calculated_max_lid = dest_lid;

            u_int8_t lft_entry_num = 0;
            u_int16_t lft_block_num = 0;
            if (sw_node_it->second.m_general_sw_info.m_sx_dev) { //SX Device
                lft_entry_num = dest_lid % AR_LFT_TABLE_BLOCK_SIZE_SX;
                lft_block_num = dest_lid / AR_LFT_TABLE_BLOCK_SIZE_SX;
                lft_entry_ptr_sx.SetPtr(&calculated_ar_lft.m_ar_lft_table_sx[lft_block_num].LidEntry[lft_entry_num]);
                p_lft_entry_ptr = &lft_entry_ptr_sx;
            } else {            
                lft_entry_num = FixLFTEntryNum(dest_lid % AR_LFT_TABLE_BLOCK_SIZE);
                lft_block_num = dest_lid / AR_LFT_TABLE_BLOCK_SIZE;
                lft_entry_ptr.SetPtr(&calculated_ar_lft.m_ar_lft_table[lft_block_num].LidEntry[lft_entry_num]);
                p_lft_entry_ptr = &lft_entry_ptr;
            }

            LidToGroupDataMapConstIter iter = algorithm_data.m_lid_to_group_map.find(dest_lid);

            if (iter == algorithm_data.m_lid_to_group_map.end()) {
            
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "LID %u dest_lid:%u - Set lid sate Static Group:%u\n",
                           sw_node_it->second.m_general_sw_info.m_lid, 
                           dest_lid, 
                           ucast_lft_port);

                p_lft_entry_ptr->SetData(AR_IB_LID_STATE_STATIC, 0, 0, ucast_lft_port);

                // We already mark static in lid state by default
            } else if (iter->second->m_group_table_number == ARMGR_MAX_GROUP_TABLES) {

                p_lft_entry_ptr->SetData(AR_IB_LID_STATE_STATIC, 0, 0, ucast_lft_port);

                if ((this->m_p_osm_log->level & OSM_LOG_DEBUG) &&
                    iter != algorithm_data.m_lid_to_group_map.end()) 
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                               "LID %u dest_lid:%u - was not mapped to group because group bitmask:%s was not mapped.\n",
                               sw_node_it->second.m_general_sw_info.m_lid, 
                               dest_lid, 
                               iter->second->m_group_bitmask.to_string().c_str());

            } else {
                uint16_t  group_number = iter->second->m_group_number;
                if (is_group_per_leaf_sw) {

                    if (!hops[dest_lid])
                        //this is not sw lid. get the sw lid connected to this lid
                        sw_lid_num = hca_to_sw_lid_mapping[dest_lid];
                    else
                        sw_lid_num = dest_lid;

                    LidToGroupNumberMapConstIter sw_grp_iter =
                        iter->second->m_sw_lid_to_group_number.find(sw_lid_num);

                    if (sw_grp_iter ==
                        iter->second->m_sw_lid_to_group_number.end()) {
                        group_number = iter->second->m_group_number;
                        /*
                        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO,
                                   "LID %u dest_lid:%u failed to find sw_lid_num:%u in sw_lid_to_group_number\n",
                                   sw_node_it->second.m_general_sw_info.m_lid,
                                   dest_lid, sw_lid_num);
                                   */
                    } else
                        group_number = sw_grp_iter->second;
                }

                enum SMP_AR_LID_STATE ar_mode = this->m_master_db.m_ar_mode;
                if ((is_group_per_leaf_sw && group_number == AR_GROUP_UNASSIGNED) ||
                    !this->m_master_db.m_ar_enable) //only FR/FRN enabled
                    ar_mode = AR_IB_LID_STATE_STATIC;

                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "LID %u dest_lid:%u - Set lid state %u Group:%u Table:%u (p_group:%p)\n",
                           sw_node_it->second.m_general_sw_info.m_lid, dest_lid, 
                           ar_mode,
                           group_number,
                           iter->second->m_group_table_number,
                           iter->second);

                p_lft_entry_ptr->SetData(ar_mode,
                                         iter->second->m_group_table_number,
                                         group_number,
                                         ucast_lft_port);
            } 

            if (this->m_p_osm_log->level & OSM_LOG_DEBUG) {
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "LID %u dest_lid:%u - calculated_ar_lft_table[%u].LidEntry[%u] GroupNumber:%u TableNumber:%u LidState:%u\n",
                           sw_node_it->second.m_general_sw_info.m_lid, 
                           dest_lid,
                           lft_block_num, lft_entry_num,                       
                           p_lft_entry_ptr->GetGroupNumber(),
                           p_lft_entry_ptr->GetTableNumber(),
                           p_lft_entry_ptr->GetLidState());
            }
            
        }      /* End of loop destination lids */

        /*
         * Step 5 -
         *  Now we will decide if the calculated settings are different from old,
         *  or if we need to ignore old settings and use the new ones
         */

        //In case we use group_per_leaf_sw we optimize ARGroup MAD sending by using copy
        if (is_group_per_leaf_sw) {

            //verify group 0 is set to empty group
            if(!sw_node_it->second.m_is_group_table_valid[0]||
               memcmp(&sw_node_it->second.m_ar_group_table[0].Group[0],
                      &empty_element, sizeof(PortGroup_Block_Element))){

                ARUpdateSWGroupElement(sw_node_it->second, empty_element, 0);
            }

            this->ARUpdateSWGroupToCopy(sw_node_it->second,
                    algorithm_data.m_assign_groups);
        } else
            this->ARUpdateSWGroupTable(sw_node_it->second,
                    calculated_ar_group_table,
                    calculated_max_pg_id+1);

        if (sw_node_it->second.m_general_sw_info.m_sx_dev)  //SX Device
            this->ARUpdateSWLFTTable(sw_node_it->second,
                    calculated_ar_lft.m_ar_lft_table_sx,
                    calculated_max_lid);
        else
            this->ARUpdateSWLFTTable(sw_node_it->second,
                    calculated_ar_lft.m_ar_lft_table,
                    calculated_max_lid);

        if (sw_node_it->second.m_config_rn){
            this->UpdateRNRcvString(sw_node_it->second, max_rank, sw_rank,
                                    max_consume_rank);
            this->UpdateRNXmitPortMask(sw_node_it->second, ca_ports, sw_ports,
                                       down_sw_ports.any());
        }

        /*
         * Step 6 -
         * Now dump debug information to osm log
         */
        this->ARDumpSWSettings(sw_node_it->second);

next_switch:
        ++sw_node_it;
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}
#endif

void AdaptiveRoutingManager::ARCalculatePortGroupsTree()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    uint8_t max_rank = 0;

    //Minimal hops array contains information for switches only
    //find for each hca the connected switch and fill in hca_to_sw_lid_mapping
    u_int16_t hca_to_sw_lid_mapping[IB_LID_UCAST_END_HO + 1] = {};
    uint8_t sw_lid_to_rank_mapping[IB_LID_UCAST_END_HO + 1] = {};
    memset(sw_lid_to_rank_mapping, 0xFF, sizeof(sw_lid_to_rank_mapping));

    osm_node_t *p_osm_node;
    // Go over all nodes in osm database
    for (p_osm_node =  (osm_node_t *) cl_qmap_head(&m_p_osm_subn->node_guid_tbl);
         p_osm_node != (osm_node_t *) cl_qmap_end(&m_p_osm_subn->node_guid_tbl);
         p_osm_node =  (osm_node_t *) cl_qmap_next(&p_osm_node->map_item)) {

        if (p_osm_node->node_info.node_type == IB_NODE_TYPE_SWITCH) {
            //set sw_lid_to_rank_mapping
            if (m_master_db.m_frn_enable || m_master_db.m_arn_enable) {

                u_int16_t lid = ADB2C_BE16_TO_CPU(
                    osm_node_get_base_lid(p_osm_node, 0));

                if (p_osm_node->sw->rank == OSM_SW_NO_RANK) {
                    m_master_db.m_frn_enable = false;
                    m_master_db.m_arn_enable = false;

                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO,
                            "Switch GUID " U64H_FMT ", LID %u: "
                            "has no rank - can not support ARN/FRN.\n",
                            ADB2C_BE64_TO_CPU(osm_node_get_node_guid(p_osm_node)),
                            lid);

                } else if (p_osm_node->sw->rank > MAX_SUPPORTED_RANK) {
                    m_master_db.m_frn_enable = false;
                    m_master_db.m_arn_enable = false;

                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO,
                            "Switch GUID " U64H_FMT ", LID %u: "
                            "has rank:%u which exceeds maximum.\n",
                            ADB2C_BE64_TO_CPU(osm_node_get_node_guid(p_osm_node)),
                            lid,
                            p_osm_node->sw->rank);

                }

                sw_lid_to_rank_mapping[lid] = p_osm_node->sw->rank;
                max_rank = max(max_rank,p_osm_node->sw->rank);
            }
            continue;
        }

        for( u_int8_t port_num = 1 ; port_num <= p_osm_node->node_info.num_ports ; port_num++) {
            osm_physp_t *p_physp = osm_node_get_physp_ptr(p_osm_node, port_num);
            osm_node_t *p_remote_node = osm_node_get_remote_node(p_osm_node, port_num, NULL);

            if (!p_physp || !osm_link_is_healthy(p_physp))
                continue;
            osm_physp_t *p_remote_physp = osm_physp_get_remote(p_physp);
            if (!p_remote_physp)
                continue;

            SetHcaToSwLidMapping(p_physp, p_remote_node, hca_to_sw_lid_mapping);
        }
    }

    /*
     * In this algorithm ports groups are created out of minimal hops
     * links - each lid will recieve a group containing all the ports
     * with minimal hops to this lid.
     * So here's what we're doing here:
     *   - for each switch
     *       - for each port of the switch (port_num)
     *           - calculate group bitmask for this lid
     *           - asisign groups acording to device limitations
     */
    m_port_groups_calculator.CalculatePortGroupsTree(
        max_rank, hca_to_sw_lid_mapping, sw_lid_to_rank_mapping);


    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void
AdaptiveRoutingManager::ARCalculateSwitchPortGroupsTree(
    ARSWDataBaseEntry &sw_db_entry,
    uint8_t max_rank,
    u_int16_t *hca_to_sw_lid_mapping,
    uint8_t *sw_lid_to_rank_mapping)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    // some variables for calculation
    SMP_ARGroupTable calculated_ar_group_table[AR_GROUP_TABLE_NUM_BLOCKS];
    SMP_AR_LFT calculated_ar_lft;
    //SMP_ARLinearForwardingTable calculated_ar_lft_table[AR_LFT_TABLE_NUM_BLOCKS];
    unsigned calculated_max_pg_id;
    unsigned calculated_max_lid;
    int rc = 0;

    TreeAlgorithmData algorithm_data;
    uint16_t max_lid = 0;
    uint8_t **hops = NULL;
    GroupData group_data;
    LidPortMappingPtrBase  *p_lft_entry_ptr = NULL;
    LidPortMappingPtr       lft_entry_ptr;
    LidPortMappingPtrSX     lft_entry_ptr_sx;

    //for ARN/FRN consume string calculation
    PortsBitset down_sw_ports;
    PortsBitset ca_ports;
    PortsBitset sw_ports;

    PortGroup_Block_Element empty_element;
    memset(&empty_element, 0, sizeof(empty_element));

    uint8_t sw_rank = OSM_SW_NO_RANK;
    uint8_t max_consume_rank = 0;

    bool is_group_per_leaf_sw = false;

    /*
     * Step 1.1 -
     *  Ensure that this switch supports AR, and that AR is enabled.
     */
    if (!this->IsARActive(sw_db_entry)) {
        OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                "Skip Switch GUID " U64H_FMT ", LID %u: " \
                   "AR not supported or not enabled.\n",
                sw_db_entry.m_general_sw_info.m_guid,
                sw_db_entry.m_general_sw_info.m_lid);
        //we already suppose to have this switch in not supported list
        OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
    }

    /*
     * Step 1.2 -
     *  Set default values for calculations
     */
    this->ARGroupTableSetDefault(calculated_ar_group_table);
    if (sw_db_entry.m_general_sw_info.m_sx_dev)
        this->ARLftTableSetDefault(calculated_ar_lft.m_ar_lft_table_sx);
    else
        this->ARLftTableSetDefault(calculated_ar_lft.m_ar_lft_table);

    calculated_max_pg_id = 0;
    calculated_max_lid = 0;
    algorithm_data.Clear();

    OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
            "Calculate group bitmask for GUID " U64H_FMT ", LID %u: \n",
            sw_db_entry.m_general_sw_info.m_guid,
            sw_db_entry.m_general_sw_info.m_lid);

    /*
     * Step 1.3 -
     *  Build down switch ports and ca ports bitset
     */
    sw_db_entry.m_config_rn =
        (IS_GENERATE_ARN(m_master_db, sw_db_entry) ||
         IS_GENERATE_FRN(m_master_db, sw_db_entry));
    is_group_per_leaf_sw = sw_db_entry.m_config_rn;

    if (sw_db_entry.m_config_rn) {

        down_sw_ports.reset();
        ca_ports.reset();
        sw_ports.reset();

        sw_rank = sw_db_entry.m_general_sw_info.m_p_osm_sw->rank;

        osm_node_t *p_osm_node =
            sw_db_entry.m_general_sw_info.m_p_osm_sw->p_node;

        for( u_int8_t port_num = 1 ;
             port_num <= p_osm_node->node_info.num_ports ; port_num++) {

            osm_physp_t *p_physp = osm_node_get_physp_ptr(p_osm_node, port_num);
            if (!p_physp || !osm_link_is_healthy(p_physp))
                continue;

            osm_physp_t *p_remote_physp = osm_physp_get_remote(p_physp);
            if (!p_remote_physp || !p_remote_physp->p_node)
                continue;


            if(p_remote_physp->p_node->sw){
                sw_ports.set(port_num);
                if (p_remote_physp->p_node->sw->rank > sw_rank)
                    down_sw_ports.set(port_num);
            } else
                ca_ports.set(port_num);

            /*
            if(!p_remote_physp->p_node->sw)
                ca_ports.set(port_num);
            else if (p_remote_physp->p_node->sw->rank > sw_rank)
                down_sw_ports.set(port_num);
                */
        }

    }


    /*
     * Step 2.1 -
     *  Iterate through all lids and calculate group bitmask for each lid.
     */
    max_lid = sw_db_entry.m_general_sw_info.m_p_osm_sw->max_lid_ho;
    hops = sw_db_entry.m_general_sw_info.m_p_osm_sw->hops;
    //u_int8_t last_port_num;

    uint16_t sw_lid_num;
    for (uint16_t lid_num = 1; lid_num <= max_lid; ++lid_num) {

        if (!hops[lid_num]) {
            //this is not sw lid. get the sw lid connected to this lid
            sw_lid_num = hca_to_sw_lid_mapping[lid_num];
            if (sw_lid_num == 0) {
                continue;
            }
            OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "hops[lid:%u] = hops[sw_lid:%u]+1 \n",lid_num , sw_lid_num);
        } else
            sw_lid_num = lid_num;

        if (!hops[sw_lid_num] || hops[sw_lid_num][0] < 1 || hops[sw_lid_num][0] == OSM_NO_PATH) {
            if(!hops[sw_lid_num]) {
                OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "hops[lid:%u] == NULL \n", sw_lid_num);
            } else {
                OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "hops[lid:%u][0] == %u \n", sw_lid_num, hops[sw_lid_num][0]);
            }
            continue;
        }

        LidToGroupDataMapIter lid_to_group_iter = algorithm_data.m_lid_to_group_map.find(sw_lid_num);
        if (lid_to_group_iter != algorithm_data.m_lid_to_group_map.end()) {

            this->AddLidToARGroup(lid_num, sw_lid_num, lid_to_group_iter->second,
                            algorithm_data, false,
                            is_group_per_leaf_sw);
            continue;
        }

        group_data.m_group_bitmask.reset();
        group_data.m_group_size = 0;


        /*
         *  Calculate group bitmask.
         */
        for (u_int8_t port_num = 1; port_num <= sw_db_entry.m_general_sw_info.m_num_ports; ++port_num) {            
            if(hops[sw_lid_num][0] == hops[sw_lid_num][port_num]) {                
                OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "hops[%u][%u]:%u \n", sw_lid_num, port_num , hops[sw_lid_num][port_num]);
                group_data.m_group_bitmask.set(port_num);
                group_data.m_group_size++;
                //OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "bitset:%s size:%u \n", 
                //           group_data.m_group_bitmask.to_string().c_str(),group_data.m_group_size);
                //last_port_num = port_num;
            }
        }

        if (group_data.m_group_size > 1 || sw_db_entry.m_config_rn){

            GroupBitmaskToDataMapInsertRes res =
                algorithm_data.m_groups_map.insert(
                    make_pair(group_data.m_group_bitmask,group_data));

            this->AddLidToARGroup(lid_num, sw_lid_num, &res.first->second,
                            algorithm_data, true,
                            is_group_per_leaf_sw);
        }
    }

    /*
     * Step 2.2 -
     *  try to assign group number and group table number(for IS4) for all groups.
     */
    OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "Step 2.2 AssignPortGroups \n");

    if (!algorithm_data.m_lid_to_group_map.empty()) {       
        rc = this->AssignPortGroups(sw_db_entry, algorithm_data);
        if (rc) 
            OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
    }

    /*
     * Step 2.3 -
     *  Calculate min string to consume.
     */

    max_consume_rank = sw_rank;
    if (sw_db_entry.m_config_rn &&
        !algorithm_data.m_assign_groups.empty()) {

        OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                   "Step 2.3 Calculate min string to consume."
                   " sw_rank:%u\n",
                   sw_rank);

        GroupsListIter list_iter = algorithm_data.m_assign_groups.begin();
        uint8_t grp_min_rank = sw_rank;

        for ( ;
              list_iter != algorithm_data.m_assign_groups.end();
              ++list_iter) {

            this->PrintGroupData("Calculate consume_rank for group: ",
                           *(*list_iter));

            if ((*list_iter)->m_group_size > 1 &&
                ((*list_iter)->m_group_bitmask & down_sw_ports).any() ) {

                    grp_min_rank = max_rank;

                    for (list<uint16_t>::iterator lid_iter =
                            (*list_iter)->m_lids_list.begin() ;
                         lid_iter != (*list_iter)->m_lids_list.end();
                         ++lid_iter)

                        grp_min_rank =
                            min(grp_min_rank,
                                sw_lid_to_rank_mapping[*lid_iter]);

                    OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                               "grp_min_rank:%u \n", grp_min_rank);

                    //ignore parallel links (grp_min_rank == sw_rank+1)
                    if (grp_min_rank < sw_rank+2 ||
                        max_consume_rank == grp_min_rank)
                        continue;

                    if (max_consume_rank == sw_rank)
                        max_consume_rank = grp_min_rank;
                    else {
                        OSM_AR_LOG(
                            m_p_osm_log, OSM_LOG_INFO,
                            "Group minimal rank (%u) "
                            "is different from current rank (%u) "
                            "for GUID " U64H_FMT ", LID %u: bitmask:%s\n",
                            grp_min_rank, max_consume_rank,
                            sw_db_entry.m_general_sw_info.m_guid,
                            sw_db_entry.m_general_sw_info.m_lid,
                            (*list_iter)->m_group_bitmask.
                                to_string().c_str());

                        //set maximal rank
                        if (max_consume_rank > grp_min_rank)
                            max_consume_rank = grp_min_rank;
                    }

                    OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                               "max_consume_rank:%u \n", max_consume_rank);


            }


        }
    }

    OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "Step 3\n");

    /*
     * Step 3 -
     *  Update calculated_ar_group_table:
     *  At this point we have all sggigned groups in algorithm_data.m_assign_groups list
     */
    if (!algorithm_data.m_assign_groups.empty()) {
    
        GroupsListIter list_iter = algorithm_data.m_assign_groups.begin();
        PortGroup_Block_Element *p_element = NULL;
        if (sw_db_entry.m_general_sw_info.m_sx_dev) {

            //In case we use group_per_leaf_sw we optimize ARGroup MAD sending by using copy
            //otherwize we use UpdateCalculatedArGroupTable
            if (!is_group_per_leaf_sw) {

                for ( ;
                      list_iter != algorithm_data.m_assign_groups.end();
                      ++list_iter) {
                      UpdateCalculatedArGroupTable(calculated_ar_group_table,
                                                   (*list_iter)->m_group_number,
                                                   (*list_iter)->m_group_bitmask,
                                                   calculated_max_pg_id);
                }
            }
        } else {

            uint16_t group_block_number;
            uint16_t group_block_idx;
            uint16_t total_group_idx;

            for ( ;
                  list_iter != algorithm_data.m_assign_groups.end();
                  ++list_iter) {
                group_block_number = 
                    GROUP_BLOCKS_NUMBER(sw_db_entry.m_ar_info.group_cap,
                                        sw_db_entry.m_required_ar_info)*
                    (*list_iter)->m_group_table_number +
                    GROUP_BLOCKS_NUMBER((*list_iter)->m_group_number+1,
                                        sw_db_entry.m_required_ar_info);

                group_block_idx = group_block_number-1;

                p_element = &calculated_ar_group_table[group_block_idx].Group[(*list_iter)->m_group_number%AR_GROUP_TABLE_BLOCK_SIZE];
                (*list_iter)->m_group_bitmask.get_data(p_element->SubGroup_0, p_element->SubGroup_1, p_element->SubGroup_2, p_element->SubGroup_3);

                total_group_idx = (group_block_number*AR_GROUP_TABLE_BLOCK_SIZE)-1;
                if (total_group_idx > calculated_max_pg_id)
                    calculated_max_pg_id = total_group_idx;
            }
        }
    }

    /*
     * Step 4 -
     *  Now let's configure AR LFTs.
     */
    OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "Step 4 configure AR LFTs \n");
    for (unsigned dest_lid = 1; dest_lid <= m_p_osm_subn->max_lid; ++dest_lid) {
        // Find out which port was used to route to lid_num in the usual unicast routing
        /*u_int8_t ucast_lft_port =
                osm_switch_get_port_by_lid(sw_db_entry.m_general_sw_info.m_p_osm_sw,
                        dest_lid);*/
        u_int8_t ucast_lft_port =
                OsmSwitchGetPortByLidUseNewLft(sw_db_entry.m_general_sw_info.m_p_osm_sw,
                        dest_lid);
        if (ucast_lft_port == OSM_NO_PATH)  //no route to this lid_num from this switch
            continue;
        calculated_max_lid = dest_lid;

        u_int8_t lft_entry_num = 0;
        u_int16_t lft_block_num = 0;
        if (sw_db_entry.m_general_sw_info.m_sx_dev) { //SX Device
            lft_entry_num = dest_lid % AR_LFT_TABLE_BLOCK_SIZE_SX;
            lft_block_num = dest_lid / AR_LFT_TABLE_BLOCK_SIZE_SX;
            lft_entry_ptr_sx.SetPtr(&calculated_ar_lft.m_ar_lft_table_sx[lft_block_num].LidEntry[lft_entry_num]);
            p_lft_entry_ptr = &lft_entry_ptr_sx;
        } else {            
            lft_entry_num = FixLFTEntryNum(dest_lid % AR_LFT_TABLE_BLOCK_SIZE);
            lft_block_num = dest_lid / AR_LFT_TABLE_BLOCK_SIZE;
            lft_entry_ptr.SetPtr(&calculated_ar_lft.m_ar_lft_table[lft_block_num].LidEntry[lft_entry_num]);
            p_lft_entry_ptr = &lft_entry_ptr;
        }

        LidToGroupDataMapConstIter iter = algorithm_data.m_lid_to_group_map.find(dest_lid);

        if (iter == algorithm_data.m_lid_to_group_map.end()) {
        
            OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "LID %u dest_lid:%u - Set lid sate Static Group:%u\n",
                       sw_db_entry.m_general_sw_info.m_lid, 
                       dest_lid, 
                       ucast_lft_port);

            p_lft_entry_ptr->SetData(AR_IB_LID_STATE_STATIC, 0, 0, ucast_lft_port);

            // We already mark static in lid state by default
        } else if (iter->second->m_group_table_number == ARMGR_MAX_GROUP_TABLES) {

            p_lft_entry_ptr->SetData(AR_IB_LID_STATE_STATIC, 0, 0, ucast_lft_port);

            if ((m_p_osm_log->level & OSM_LOG_DEBUG) &&
                iter != algorithm_data.m_lid_to_group_map.end()) 
                OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                           "LID %u dest_lid:%u - was not mapped to group because group bitmask:%s was not mapped.\n",
                           sw_db_entry.m_general_sw_info.m_lid, 
                           dest_lid, 
                           iter->second->m_group_bitmask.to_string().c_str());

        } else {
            uint16_t  group_number = iter->second->m_group_number;
            if (is_group_per_leaf_sw) {

                if (!hops[dest_lid])
                    //this is not sw lid. get the sw lid connected to this lid
                    sw_lid_num = hca_to_sw_lid_mapping[dest_lid];
                else
                    sw_lid_num = dest_lid;

                LidToGroupNumberMapConstIter sw_grp_iter =
                    iter->second->m_sw_lid_to_group_number.find(sw_lid_num);

                if (sw_grp_iter ==
                    iter->second->m_sw_lid_to_group_number.end()) {
                    group_number = iter->second->m_group_number;
                    /*
                    OSM_AR_LOG(m_p_osm_log, OSM_LOG_INFO,
                               "LID %u dest_lid:%u failed to find sw_lid_num:%u in sw_lid_to_group_number\n",
                               sw_db_entry.m_general_sw_info.m_lid,
                               dest_lid, sw_lid_num);
                               */
                } else
                    group_number = sw_grp_iter->second;
            }

            enum SMP_AR_LID_STATE ar_mode = this->m_master_db.m_ar_mode;
            if ((is_group_per_leaf_sw && group_number == AR_GROUP_UNASSIGNED) ||
                !this->m_master_db.m_ar_enable) //only FR/FRN enabled
                ar_mode = AR_IB_LID_STATE_STATIC;

            OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "LID %u dest_lid:%u - Set lid state %u Group:%u Table:%u (p_group:%p)\n",
                       sw_db_entry.m_general_sw_info.m_lid, dest_lid, 
                       ar_mode,
                       group_number,
                       iter->second->m_group_table_number,
                       iter->second);

            p_lft_entry_ptr->SetData(ar_mode,
                                     iter->second->m_group_table_number,
                                     group_number,
                                     ucast_lft_port);
        } 

        if (m_p_osm_log->level & OSM_LOG_DEBUG) {
            OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "LID %u dest_lid:%u - calculated_ar_lft_table[%u].LidEntry[%u] GroupNumber:%u TableNumber:%u LidState:%u\n",
                       sw_db_entry.m_general_sw_info.m_lid, 
                       dest_lid,
                       lft_block_num, lft_entry_num,                       
                       p_lft_entry_ptr->GetGroupNumber(),
                       p_lft_entry_ptr->GetTableNumber(),
                       p_lft_entry_ptr->GetLidState());
        }
        
    }      /* End of loop destination lids */

    /*
     * Step 5 -
     *  Now we will decide if the calculated settings are different from old,
     *  or if we need to ignore old settings and use the new ones
     */

    //In case we use group_per_leaf_sw we optimize ARGroup MAD sending by using copy
    if (is_group_per_leaf_sw) {

        //verify group 0 is set to empty group
        if(!sw_db_entry.m_is_group_table_valid[0]||
           memcmp(&sw_db_entry.m_ar_group_table[0].Group[0],
                  &empty_element, sizeof(PortGroup_Block_Element))){

            this->ARUpdateSWGroupElement(sw_db_entry, empty_element, 0);
        }

        pthread_mutex_lock(&m_sw_db.m_db_lock);
        this->ARUpdateSWGroupToCopy(sw_db_entry,
                algorithm_data.m_assign_groups);
        pthread_mutex_unlock(&m_sw_db.m_db_lock);

    } else
        this->ARUpdateSWGroupTable(sw_db_entry,
                calculated_ar_group_table,
                calculated_max_pg_id+1);

    if (sw_db_entry.m_general_sw_info.m_sx_dev)  //SX Device
        this->ARUpdateSWLFTTable(sw_db_entry,
                calculated_ar_lft.m_ar_lft_table_sx,
                calculated_max_lid);
    else
        this->ARUpdateSWLFTTable(sw_db_entry,
                calculated_ar_lft.m_ar_lft_table,
                calculated_max_lid);

    if (sw_db_entry.m_config_rn){
        this->UpdateRNRcvString(sw_db_entry, max_rank, sw_rank,
                                max_consume_rank);
        this->UpdateRNXmitPortMask(sw_db_entry, ca_ports, sw_ports,
                                   down_sw_ports.any());
    }

}

void AdaptiveRoutingManager::UpdateVlidsLfts()
{
    OSM_AR_LOG_ENTER(m_p_osm_log);
    OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "UpdateVlidsLfts.\n");

    LidsToLidsVecVec sw_lid_to_vlids;
    BuildSwToVlidsMap(sw_lid_to_vlids);

    if (sw_lid_to_vlids.empty()) {
        OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
    }

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it =
        m_sw_db.m_sw_map.begin();
    for (; sw_node_it != m_sw_db.m_sw_map.end(); ++sw_node_it) {
        CalculateVlidsLft(sw_lid_to_vlids,
                          sw_node_it->second.m_general_sw_info.m_p_osm_sw,
                          sw_node_it->second.m_ar_lft,
                          sw_node_it->second.m_to_set_lft_table);

        if (sw_node_it->second.m_max_lid < m_p_osm_subn->max_lid) {
            sw_node_it->second.m_max_lid = m_p_osm_subn->max_lid;
        }
    }

    ARLFTTableProcess();

    OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
}

void AdaptiveRoutingManager::CalculateVlidsLft(LidsToLidsVecVec &sw_lid_to_vlids,
                                               osm_switch_t  *p_osm_sw,
                                               SMP_AR_LFT &ar_lft,
                                               bool to_set_lft_table[])
{
    OSM_AR_LOG_ENTER(m_p_osm_log);

    u_int16_t sw_lid, vlid;
    u_int8_t lft_entry_num;
    u_int16_t lft_block_num;
    u_int8_t ucast_lft_port;

    for (LidsToLidsVecVec::const_iterator iter = sw_lid_to_vlids.begin();
         iter != sw_lid_to_vlids.end(); ++iter) {

        sw_lid = iter->first;
        lft_entry_num = sw_lid % AR_LFT_TABLE_BLOCK_SIZE_SX;
        lft_block_num = sw_lid / AR_LFT_TABLE_BLOCK_SIZE_SX;

        LidPortMapping_Block_Element_SX &sw_lid_entry =
            ar_lft.m_ar_lft_table_sx[lft_block_num].LidEntry[lft_entry_num];

        for (LidsVec::const_iterator vlids_iter = iter->second.begin();
             vlids_iter != iter->second.end(); ++vlids_iter) {

            vlid = *vlids_iter;
            lft_entry_num = vlid % AR_LFT_TABLE_BLOCK_SIZE_SX;
            lft_block_num = vlid / AR_LFT_TABLE_BLOCK_SIZE_SX;

            ucast_lft_port = OsmSwitchGetPortByLidUseNewLft(p_osm_sw, vlid);
            if (ucast_lft_port == OSM_NO_PATH ||
                sw_lid_entry.DefaultPort == OSM_NO_PATH) {
                //no route to this lid_num from this switch
                continue;
            }

            LidPortMapping_Block_Element_SX &vlid_entry =
                ar_lft.m_ar_lft_table_sx[lft_block_num].LidEntry[lft_entry_num];

            if (vlid_entry.DefaultPort != ucast_lft_port ||
                vlid_entry.GroupNumber != sw_lid_entry.GroupNumber ||
                vlid_entry.LidState != sw_lid_entry.LidState ||
                vlid_entry.TableNumber != sw_lid_entry.TableNumber) {

                vlid_entry.DefaultPort = ucast_lft_port;
                vlid_entry.GroupNumber = sw_lid_entry.GroupNumber;
                vlid_entry.LidState = sw_lid_entry.LidState;
                vlid_entry.TableNumber = sw_lid_entry.TableNumber;

                OSM_AR_LOG(
                    m_p_osm_log, OSM_LOG_DEBUG,
                    "Update vlid: %u on Switch GUID " U64H_FMT
                    "DefaultPort:%u GroupNumber:%u LidState:%u TableNumber:%u\n",
                    vlid,
                    ADB2C_BE64_TO_CPU(osm_node_get_node_guid(p_osm_sw->p_node)),
                    vlid_entry.DefaultPort, vlid_entry.GroupNumber,
                    vlid_entry.LidState, vlid_entry.TableNumber);

                to_set_lft_table[lft_block_num] = true;
            }
        }
    }
    OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
}

void AdaptiveRoutingManager::BuildSwToVlidsMap(LidsToLidsVecVec &sw_lid_to_vlids)
{
    OSM_AR_LOG_ENTER(m_p_osm_log);

    LidsList vlids_list;
    LidsVec empty_lids_vec;

    sw_lid_to_vlids.reserve(m_sw_db.m_sw_map.size());

    osm_node_t *p_osm_node;
    // Go over all nodes in osm database
    for (p_osm_node =  (osm_node_t *) cl_qmap_head(&m_p_osm_subn->node_guid_tbl);
         p_osm_node != (osm_node_t *) cl_qmap_end(&m_p_osm_subn->node_guid_tbl);
         p_osm_node =  (osm_node_t *) cl_qmap_next(&p_osm_node->map_item)) {

        if (p_osm_node->node_info.node_type != IB_NODE_TYPE_SWITCH) {
            continue;
        }

        vlids_list.clear();

        for(u_int8_t port_num = 1; port_num <= p_osm_node->node_info.num_ports;
            port_num++) {

            osm_physp_t *p_physp = osm_node_get_physp_ptr(p_osm_node, port_num);
            if (!p_physp || !osm_link_is_healthy(p_physp))
                continue;

            osm_node_t *p_remote_node = osm_node_get_remote_node(p_osm_node, port_num, NULL);
            if (!p_remote_node ||
                p_remote_node->node_info.node_type != IB_NODE_TYPE_CA) {
                continue;
            }

            osm_physp_t *p_remote_physp = osm_physp_get_remote(p_physp);
            if (!p_remote_physp)
                continue;

            GetVlidsList(p_remote_physp, vlids_list);
        }

        if (!vlids_list.empty()) {
            u_int16_t sw_lid =
                ADB2C_BE16_TO_CPU(osm_node_get_base_lid(p_osm_node, 0));

            sw_lid_to_vlids.push_back(make_pair(sw_lid, empty_lids_vec));
            LidsVec &lids_vec = sw_lid_to_vlids.back().second;
            lids_vec.resize(vlids_list.size());
            lids_vec.insert(lids_vec.begin(),
                            vlids_list.begin(), vlids_list.end());
        }
    }
    OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
}

void AdaptiveRoutingManager::AddLidToARGroup(uint16_t lid_num, uint16_t sw_lid_num,
                                             GroupData* p_group_data,
                                             TreeAlgorithmData &algorithm_data,
                                             bool is_new_group,
                                             bool is_group_per_leaf_sw)
{
    if (is_new_group) {
        p_group_data->m_lids_list.push_back(sw_lid_num);
        algorithm_data.m_lid_to_group_map.insert(
            make_pair(sw_lid_num, p_group_data));

        PrintGroupData("Add Group Data: ",*p_group_data);
    }

    if (lid_num != sw_lid_num) {
        p_group_data->m_lids_list.push_back(lid_num);
        algorithm_data.m_lid_to_group_map.insert(
            make_pair(lid_num, p_group_data));

        if (is_group_per_leaf_sw)
            p_group_data->m_leaf_switches.insert(sw_lid_num);

        if (!is_new_group)
            PrintGroupData("Update Group Data: ",*p_group_data);
    }
}


void AdaptiveRoutingManager::PrintGroupData(const char * str,GroupData &group_data)
{

    if(this->m_p_osm_log->level & OSM_LOG_DEBUG) {

        stringstream sstr;
        list<uint16_t>::const_iterator iter =  group_data.m_lids_list.begin();
        int lid_count = 1;
        //print first 100 lids
        for (; iter != group_data.m_lids_list.end() ; ++iter) {
            sstr << " " << *iter;
            lid_count++;
            if (lid_count > 100) {
                sstr << " ...";
                break;
            }
        }

        sstr << " bitset:" << group_data.m_group_bitmask;
             
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "%s Group:%p size:%d lids_number:%u lids:%s\n",
                   str, 
                   &group_data, 
                   group_data.m_group_size,
                   (uint16_t)group_data.m_lids_list.size(),
                   sstr.str().c_str());
    }
}

int AdaptiveRoutingManager::AssignPortGroups(ARSWDataBaseEntry &sw_db_entry, TreeAlgorithmData &algorithm_data)
{
    int rc = 0;
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    uint16_t group_cap = sw_db_entry.GetGroupCap();

    if (sw_db_entry.m_config_rn){
        rc = AssignPerLeafSwitchGroups(sw_db_entry, algorithm_data);
        OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);
    }


    GroupsList group_list;

    rc = GetOrderedGroupList(algorithm_data, group_list);
    if (rc) OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "AssignPortGroups \n");

    GroupsListIter list_iter = group_list.begin();

    if (sw_db_entry.m_general_sw_info.m_sx_dev) { //SwitchX
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "AssignPortGroups To SwitchX Device\n");

        //reserve group 0 as empry group
        uint16_t next_group_number = (m_master_db.m_fr_enable ? 1 : 0);
        for ( ;
              list_iter != group_list.end() && 
              next_group_number < group_cap;
              ++list_iter) {

            (*list_iter)->m_group_number = next_group_number;
            (*list_iter)->m_group_table_number = 0;
            next_group_number++;

            algorithm_data.m_assign_groups.push_front(*list_iter);

            //MapLidsToGroup(algorithm_data.m_lid_to_group_map, (*list_iter)->m_lids_list, *list_iter);
        }

        if (next_group_number == group_cap) {
            for ( ;  list_iter != group_list.end() ; ++list_iter) {
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "SX Switch GUID " U64H_FMT 
                           ", Can not assign group %s \n",
                        sw_db_entry.m_general_sw_info.m_guid,
                        (*list_iter)->m_group_bitmask.to_string().c_str());
            }

        }

    } else {

        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "AssignPortGroups To IS4 Device\n");

        bool assign = false;

        for ( ; list_iter != group_list.end() ; ++list_iter) {
            PrintGroupData("AssignPortGroup: ",*(*list_iter));

            assign = false;
            //for each group assign to group table and set group number and group table number
            for (int group_table = 0 ; group_table < group_cap ; group_table++) {
                //if can assign to this group
                if (this->m_p_osm_log->level & OSM_LOG_DEBUG) {
                    stringstream sstr;
                    sstr << "Check group:" << group_table << " " << "assign:" << algorithm_data.m_assign_group_table[group_table].m_assign_bitmask;
                    sstr << " group bitmask:" << (*list_iter)->m_group_bitmask;
                    sstr << " assign & group bitmask" << (algorithm_data.m_assign_group_table[group_table].m_assign_bitmask & (*list_iter)->m_group_bitmask);
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "%s \n", sstr.str().c_str());

                }
                if ((algorithm_data.m_assign_group_table[group_table].m_assign_bitmask & (*list_iter)->m_group_bitmask).none()) {
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Assign PortGroup:%p to group_table:%u group:%u \n",
                               *list_iter, group_table, algorithm_data.m_assign_group_table[group_table].m_next_group_number);

                    //assign the group
                    (*list_iter)->m_group_number = algorithm_data.m_assign_group_table[group_table].m_next_group_number;
                    (*list_iter)->m_group_table_number = group_table;
    
                    //update group_table
                    algorithm_data.m_assign_group_table[group_table].m_next_group_number++;
                    //algorithm_data.m_assign_group_table[group_table].m_groups_list.insert(*list_iter);
                    algorithm_data.m_assign_group_table[group_table].m_assign_bitmask |= (*list_iter)->m_group_bitmask;

                    algorithm_data.m_assign_groups.push_front(*list_iter); //larger group at the begining

                    //MapLidsToGroup(algorithm_data.m_lid_to_group_map, (*list_iter)->m_lids_list, *list_iter);

                    assign = true;
    
                    break;
                }
            }

            if (!assign) {

                if(this->m_p_osm_log->level & OSM_LOG_DEBUG) {
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "IS4 Switch GUID " U64H_FMT 
                               ", Can not assign group %s. Try to assign to contained group.\n",
                            sw_db_entry.m_general_sw_info.m_guid,
                            (*list_iter)->m_group_bitmask.to_string().c_str());
                }

                AssignToContainedGroup(sw_db_entry, algorithm_data, *list_iter);
            }
        }
    } //IS4


    OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);
}

int AdaptiveRoutingManager::AssignPerLeafSwitchGroups(ARSWDataBaseEntry &sw_db_entry,
                                                      TreeAlgorithmData &algorithm_data)
{
    int rc = 0;

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "AssignPerLeafSwitchGroups \n");

    uint16_t group_number = 0;
    bool is_group_cap_exceeded = false;

    uint16_t group_cap = sw_db_entry.GetGroupCap();

    GroupBitmaskToDataMapIter iter = algorithm_data.m_groups_map.begin();
    for (; iter != algorithm_data.m_groups_map.end() ; ++iter) {


        //will not perform AR for switches that are none leaf switches
        iter->second.m_group_number = AR_GROUP_UNASSIGNED;
        iter->second.m_group_table_number = 0;
        algorithm_data.m_assign_groups.push_front(&iter->second);

        //go over all LeafSwitches
        set<uint16_t>::iterator leafs_sw_iter =
            iter->second.m_leaf_switches.begin();

        for ( ;
              leafs_sw_iter != iter->second.m_leaf_switches.end();
              ++leafs_sw_iter) {

            group_number = AllocateSwArGroup( *leafs_sw_iter, group_cap);

            if (group_number == AR_GROUP_UNASSIGNED) {
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                           "SX Switch GUID " U64H_FMT
                           ", Can not assign group %s for lid:%u\n",
                        sw_db_entry.m_general_sw_info.m_guid,
                        iter->second.m_group_bitmask.to_string().c_str(),
                        *leafs_sw_iter);
                is_group_cap_exceeded = true;
                continue;
            }


            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Assign group %u for leaf switch lid:%u\n",
                    group_number, *leafs_sw_iter);

            iter->second.m_sw_lid_to_group_number.insert(
                make_pair(*leafs_sw_iter,group_number));
        }

    }

    if (is_group_cap_exceeded) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO,
            "SX Switch GUID " U64H_FMT
            ", Can not assign seperate groups number"
            " for all leaf switches.\n",
            sw_db_entry.m_general_sw_info.m_guid);
    }

    OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);
}

int AdaptiveRoutingManager::AssignToContainedGroup(ARSWDataBaseEntry &sw_db_entry, TreeAlgorithmData &algorithm_data, GroupData* group)
{
    int rc = 0;

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "AssignToContainedGroup \n");

    GroupsList contained_group_list;
    GroupData* port_to_group_map[MAX_SUPPORTED_PORTS] = {};

    PortsBitset total_bitmask;
    GetContainedGroupList(algorithm_data.m_assign_groups, group->m_group_bitmask, contained_group_list, total_bitmask);

    u_int8_t ucast_lft_port = 0;
    GroupsListIter list_iter;
    LidToGroupDataMapIter lid_to_group_iter;

    //Map group lids to the largest group containing ucast_lft_port 
    list<uint16_t>::const_iterator iter =  group->m_lids_list.begin();
    for (; iter != group->m_lids_list.end() ; ++iter) {
        ucast_lft_port = 
            OsmSwitchGetPortByLidUseNewLft(sw_db_entry.m_general_sw_info.m_p_osm_sw, *iter);

        if ( total_bitmask.test(ucast_lft_port)) {

            if (! port_to_group_map[ucast_lft_port]) {  

                // look in contained_group_list 
                list_iter = contained_group_list.begin();

                for ( ; 
                      list_iter != contained_group_list.end() ; 
                      ++list_iter) {

                    if ((*list_iter)->m_group_bitmask.test(ucast_lft_port)) {                    
                        port_to_group_map[ucast_lft_port] = (*list_iter);
                        break;
                    }
                }

            }

            if (! port_to_group_map[ucast_lft_port]) {            
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR, "AssignToContainedGroup: BUG: Can not find port_to_group_map. \n");
            } else {            
                //the lid was mapped to the original group, change the mapping to the contained grop
                lid_to_group_iter = algorithm_data.m_lid_to_group_map.find(*iter);
                if (lid_to_group_iter != algorithm_data.m_lid_to_group_map.end()) {
                    lid_to_group_iter->second = port_to_group_map[ucast_lft_port];
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "m_lid_to_group_map lid:%u group:%p \n", *iter , port_to_group_map[ucast_lft_port]); 
                } else {
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR, 
                               "AssignToContainedGroup: BUG: can't find lid:%d in m_lid_to_group_map. \n", *iter);
                    algorithm_data.m_lid_to_group_map.insert(make_pair(*iter,port_to_group_map[ucast_lft_port]));
                    //OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "m_lid_to_group_map lid:%u group:%p \n", *iter , port_to_group_map[ucast_lft_port]); 

                }

                if(this->m_p_osm_log->level & OSM_LOG_DEBUG) {
                    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Assign lid:%d route to contained group:%s.\n", 
                               *iter, (*list_iter)->m_group_bitmask.to_string().c_str()); 
                }
            }
        } else {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO, "Can not assign lid:%d route to contained group.\n", *iter); 
        }
    }

    OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);

}

int AdaptiveRoutingManager::GetContainedGroupList(GroupsList& group_list, PortsBitset &group_bitmask, GroupsList& contained_group_list, PortsBitset &total_bitmask)
{
    int rc = 0;

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "GetContainedGroupList \n"); 

    GroupsListIter list_iter = group_list.begin();

    PortsBitset not_group_bitmask = group_bitmask;
    not_group_bitmask.flip();

    for ( ; 
          list_iter != group_list.end() ; 
          ++list_iter) {

        if (!(not_group_bitmask & (*list_iter)->m_group_bitmask).any()) { //the group is contained in group_bitmask
            contained_group_list.push_back(*list_iter);
            total_bitmask |= (*list_iter)->m_group_bitmask;

            if (total_bitmask == group_bitmask) {
                break; //no need to continue
            }
  
        }

    }

    OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);
}

bool compare_groups_by_members(GroupData* p_group1, GroupData* p_group2)
{
    //compare by the following ordered by priority:
    //1) group members number (descending orded)
    //2) group ports number   (descending orded)
    //3) first lid number     (ascending orded) - for stable sorting
    if (p_group1->m_lids_list.size() != p_group2->m_lids_list.size()) 
        return p_group1->m_lids_list.size() > p_group2->m_lids_list.size();
    if (p_group1->m_group_size != p_group2->m_group_size) 
        return p_group1->m_group_size > p_group2->m_group_size;

    return *p_group1->m_lids_list.begin() < *p_group2->m_lids_list.begin();
}

int AdaptiveRoutingManager::GetOrderedGroupList(TreeAlgorithmData &algorithm_data, GroupsList& group_list)
{
    int rc = 0;
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    GroupBitmaskToDataMapIter iter = algorithm_data.m_groups_map.begin();
    for (; iter != algorithm_data.m_groups_map.end() ; ++iter) {
        group_list.push_back(&iter->second);
    }

    group_list.sort(compare_groups_by_members);

    OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);
}


/***************/
void AdaptiveRoutingManager::ARGroupTableProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    for (;sw_node_it != this->m_sw_db.m_sw_map.end();++sw_node_it) {

        if (sw_node_it->second.m_no_groups)
            continue;
        /*
         * Ensure that this switch supports AR,
         * and that AR is enabled.
         */
        if (!this->IsARActive(sw_node_it->second)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Switch GUID " U64H_FMT ", LID %u: " \
                    "AR not supported or not enabled, group table process skipped.\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid);
            //we already suppose to have this switch in not supported list
            continue;
        }

        bool reset_direction =
            (IS_GENERATE_ARN(m_master_db, sw_node_it->second) ||
             IS_GENERATE_FRN(m_master_db, sw_node_it->second));

        ARGroupTableProcess(sw_node_it->second,
                            //sw_node_it->second.m_total_groups_number,
                            sw_node_it->second.m_group_top,
                            reset_direction,
                            sw_node_it->second.m_ar_group_table);

    }

    //Receive all Set ARGroupTable reply
    m_ibis_obj.MadRecAll();

    if (m_master_db.m_frn_enable || m_master_db.m_arn_enable)
        ARCopyGroupTableProcess();

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


void AdaptiveRoutingManager::ARGroupTableProcess(ARSWDataBaseEntry &sw_entry,
                                                 u_int16_t group_top,
                                                 bool reset_direction,
                                                 SMP_ARGroupTable ar_group_table_array[AR_GROUP_TABLE_NUM_BLOCKS])
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);


    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
               "ARGroupTableProcess Switch GUID " U64H_FMT ", LID %u " \
               "group_top:%u\n",
               sw_entry.m_general_sw_info.m_guid,
               sw_entry.m_general_sw_info.m_lid,
               group_top);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &this->m_ar_clbck;
    clbck_data.m_handle_data_func = SetARGroupTableClbckDlg;
    clbck_data.m_data1 = &sw_entry;
    /*
     * Set() group table
     */
    // No need to set all table - we will set only what we need
    unsigned total_blocks_number =
        GROUP_BLOCKS_NUMBER((group_top + 1),
                            sw_entry.m_required_ar_info);

    unsigned blocks_per_table =
        GROUP_BLOCKS_NUMBER(sw_entry.m_ar_info.group_cap,
                            sw_entry.m_required_ar_info);

        //((sw_entry.m_ar_info.PSE == 0) ?
        //sw_entry.m_ar_info.GroupCap/AR_GROUP_TABLE_BLOCK_SIZE :
        //sw_entry.m_ar_info.GroupCap);

    for (unsigned group_block_idx = 0; group_block_idx < total_blocks_number;
         ++group_block_idx) {
        // If no need to set this block than continue to next one
        if (!sw_entry.m_to_set_group_table[group_block_idx])
            continue;

        u_int8_t reset_attr = 0;

        SMP_ARGroupTable ar_group_table = 
            ar_group_table_array[group_block_idx];

        if (sw_entry.m_general_sw_info.m_sx_dev) { // SX device
            if (reset_direction) {
                u_int16_t first_group_number =
                    group_block_idx * AR_GROUP_TABLE_BLOCK_SIZE;
                u_int8_t *p_direction_1 =
                    sw_entry.GetSubGroupDirectionPtr(first_group_number, 0);
                u_int8_t *p_direction_2 =
                    sw_entry.GetSubGroupDirectionPtr(first_group_number+1, 0);

                if (p_direction_1 && *p_direction_1)
                    reset_attr = 0x1;
                if (p_direction_2 && *p_direction_2)
                    reset_attr |= 0x2;
            }
        } else {
            for (unsigned i = 0; i < AR_GROUP_TABLE_BLOCK_SIZE; ++i) {
                ar_group_table.Group[i].SubGroup_0 = FixGroupBitmask(ar_group_table.Group[i].SubGroup_0);
                ar_group_table.Group[i].SubGroup_1 = FixGroupBitmask(ar_group_table.Group[i].SubGroup_1);
                ar_group_table.Group[i].SubGroup_2 = FixGroupBitmask(ar_group_table.Group[i].SubGroup_2);
                ar_group_table.Group[i].SubGroup_3 = FixGroupBitmask(ar_group_table.Group[i].SubGroup_3);
            }
        }

        unsigned block = group_block_idx % blocks_per_table;
        unsigned group_table = group_block_idx / blocks_per_table;
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                "Setting ARGroupTable (block=%u) (table=%u) to "   \
                "Switch GUID " U64H_FMT ", LID %u\n",
                block, group_table,
                sw_entry.m_general_sw_info.m_guid,
                sw_entry.m_general_sw_info.m_lid);

        clbck_data.m_data2 = (void *)(intptr_t)group_block_idx;
        clbck_data.m_data3 = (void *)(intptr_t)reset_attr;

        this->ARGroupTableMadGetSetByDirect(
                &sw_entry.m_general_sw_info.m_direct_route,
                IBIS_IB_MAD_METHOD_SET,
                block, group_table, 0, reset_attr,
                &ar_group_table,
                &clbck_data);

    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void AdaptiveRoutingManager::ARCopyGroupTableProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &this->m_ar_clbck;
    clbck_data.m_handle_data_func = ARCopyGroupTableClbckDlg;

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    for (;sw_node_it != this->m_sw_db.m_sw_map.end();++sw_node_it) {
        /*
         * Ensure that this switch supports AR,
         * and that AR is enabled.
         */
        if (!this->IsARActive(sw_node_it->second)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Switch GUID " U64H_FMT ", LID %u: " \
                    "AR not supported or not enabled, copy group table process skipped.\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid);
            continue;
        }

        clbck_data.m_data1 = &sw_node_it->second;

        list<CopyFromToGroups*>::iterator iter =
            sw_node_it->second.m_copy_from_to_group_list.begin();
        while (iter != sw_node_it->second.m_copy_from_to_group_list.end()) {

            clbck_data.m_data2 = (void *)(intptr_t)(*iter)->m_copy_from_group;
            clbck_data.m_data3 = (void *)(intptr_t)(*iter)->m_copy_direction;

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                "Setting ARGroupTableCopy "
                "(copy_group=%u) (copy_direction=%u) to "
                "Switch GUID " U64H_FMT ", LID %u\n",
                (*iter)->m_copy_from_group, (*iter)->m_copy_direction,
                sw_node_it->second.m_general_sw_info.m_guid,
                sw_node_it->second.m_general_sw_info.m_lid);

            for (int i=0 ; i < AR_COPY_GROUP_BLOCK_SIZE ; i++) {
                adaptive_routing_group_table_copy_element *p_element =
                    &(*iter)->m_ar_group_table_copy.element[i];

                if (p_element->copy_from_index == 0)
                    break;

                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                           "Setting ARGroupTableCopy "
                           "Switch GUID " U64H_FMT ", LID %u"
                           "(copy_group=%u) from:%u to:%u\n",
                           sw_node_it->second.m_general_sw_info.m_guid,
                           sw_node_it->second.m_general_sw_info.m_lid,
                           (*iter)->m_copy_from_group,
                           p_element->copy_from_index,
                           p_element->copy_to_index);
            }

            m_ibis_obj.SMPARGroupTableCopySetByDirect(
                    &sw_node_it->second.m_general_sw_info.m_direct_route,
                    (*iter)->m_copy_from_group, (*iter)->m_copy_direction,
                    &(*iter)->m_ar_group_table_copy,
                    &clbck_data);


            copy_from_to_groups_pool.free(*iter);
            sw_node_it->second.m_copy_from_to_group_list.pop_front();
            iter = sw_node_it->second.m_copy_from_to_group_list.begin();
        }

    }

    //Receive all Set ARGroupTable reply
    m_ibis_obj.MadRecAll();

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

/***************/
void AdaptiveRoutingManager::ARLFTTableProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    for (; sw_node_it != this->m_sw_db.m_sw_map.end(); ++sw_node_it) {
        /*
         * Ensure that this switch supports AR,
         * and that AR is enabled.
         */
        if (!this->IsARActive(sw_node_it->second)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "Switch GUID " U64H_FMT ", LID %u: " \
                    "AR not supported or not enabled, group table process skipped.\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid);
            //we already suppose to have this switch in not supported list
            continue;
        }

        ARLFTTableProcess(sw_node_it->second,
                          sw_node_it->second.m_max_lid,
                          0,
                          sw_node_it->second.m_to_set_lft_table,
                          sw_node_it->second.m_ar_lft);
    }

    //Receive all Set ARLFTTable reply
    m_ibis_obj.MadRecAll();

    sw_node_it = this->m_sw_db.m_sw_map.begin();
    for (; sw_node_it != this->m_sw_db.m_sw_map.end(); ++sw_node_it) {
        if (this->IsARActive(sw_node_it->second))
            // Mark that update was done for this switch in this cycle
            sw_node_it->second.m_app_data = ARMGR_SW_UPDATED;
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void AdaptiveRoutingManager::ARLFTTableProcess(ARSWDataBaseEntry &sw_entry,
                                              u_int16_t max_lid,
                                              u_int8_t pLFTID,
                                              bool to_set_lft_table[AR_LFT_TABLE_NUM_BLOCKS_SX],
                                              SMP_AR_LFT &ar_lft)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    /*
     * Set() group table
     */
    // No need to set all table - we will set only what we need,
    // If a destination lid > lid_cap in switch_info will come the hardware will drop it
    unsigned supported_lft_blocks = 0;
    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &this->m_ar_clbck;
    clbck_data.m_handle_data_func = SetARLFTTableClbckDlg;
    clbck_data.m_data1 = &sw_entry;
    clbck_data.m_data3 = to_set_lft_table;

    if (sw_entry.m_general_sw_info.m_sx_dev) // SX device
        supported_lft_blocks =
            max_lid / AR_LFT_TABLE_BLOCK_SIZE_SX;
    else
        supported_lft_blocks =
            max_lid / AR_LFT_TABLE_BLOCK_SIZE;
    for (unsigned lft_block = 0; lft_block <= supported_lft_blocks; ++lft_block) {
        // If no need to set this block than continue to next one
        if (!to_set_lft_table[lft_block])
            continue;

        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                "Setting ARLFTTable (block=%u) to "   \
                "Switch GUID " U64H_FMT ", LID %u pLFTID %u\n",
                lft_block,
                sw_entry.m_general_sw_info.m_guid,
                sw_entry.m_general_sw_info.m_lid,
                pLFTID);

        AR_MGR_LOG(TT_LOG_LEVEL_INFO, "Setting ARLFTTable (block=%u) to "   \
                "Switch GUID " U64H_FMT ", LID %u pLFTID %u\n",
                lft_block,
                sw_entry.m_general_sw_info.m_guid,
                sw_entry.m_general_sw_info.m_lid,
                pLFTID);

        clbck_data.m_data2 = (void *)(intptr_t)lft_block;

        if (sw_entry.m_general_sw_info.m_sx_dev) {
            this->ARLinearForwardingTableMadGetSetByDirect(&sw_entry.m_general_sw_info.m_direct_route,
                IBIS_IB_MAD_METHOD_SET,
                lft_block,
                pLFTID,
                &ar_lft.m_ar_lft_table_sx[lft_block],
                &clbck_data);
        } else {
            this->ARLinearForwardingTableMadGetSetByDirect(&sw_entry.m_general_sw_info.m_direct_route,
                IBIS_IB_MAD_METHOD_SET,
                lft_block,
                pLFTID,
                &ar_lft.m_ar_lft_table[lft_block],
                &clbck_data);
        }

    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);

}


/***************/
int AdaptiveRoutingManager::ARCycle()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "ARCycle.\n");
    int rc = 0;

    if (m_master_db.m_ar_algorithm == AR_ALGORITHM_DF_PLUS) {

        if (m_p_osm->subn.opt.qos == TRUE) {
            OSM_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                    "DF_PLUS Algorithm not supported together with qos.\n");
            this->m_master_db.m_enable = false;
        }

        if ((m_master_db.m_en_sl_mask != ALL_SL_VL_AR_ENABLED) &&
            (m_p_osm->subn.opt.max_op_vls <= 2)) {
            OSM_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                    "DF_PLUS Algorithm do not support disabling AR on sl "
                    " when op_vls equals %u\n", m_p_osm->subn.opt.max_op_vls);
            this->m_master_db.m_enable = false;
        }

        if (!this->m_master_db.m_enable)
            ClearAllDragonflyConfiguration();
        else
            rc = ARDragonFlyCycle();

        OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);
    }

    // not AR_ALGORITHM_DF_PLUS

    if (m_df_configured) {
        ClearAllDragonflyConfiguration();
    }

    if (!this->m_master_db.m_enable)
        OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);

    this->ARInfoGetProcess();
    this->ARInfoGetGroupCapProcess();

    switch( m_master_db.m_ar_algorithm )
    {
        case AR_ALGORITHM_TREE:
        {
            this->ARCalculatePortGroupsTree();
            break;
        }
        case AR_ALGORITHM_LAG:
        {
            this->ARCalculatePortGroupsParallelLinks();
            break;
        }
        default:
        {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                       "Unknown algorithm: %d.\n", m_master_db.m_ar_algorithm);
            OSM_AR_LOG_RETURN(this->m_p_osm_log, -1);
        }
    };

    //Set ARInfo after grouop top calculation
    this->ARInfoSetProcess();

    //currently RoutingNotification is supported only on Tree algorithm
    if ((m_master_db.m_ar_algorithm == AR_ALGORITHM_TREE) &&
        (m_master_db.m_frn_enable || m_master_db.m_arn_enable))
        this->TreeRoutingNotificationProcess();

    this->ARGroupTableProcess();

#ifdef _DEBUG_AR_OSM_INTEGRATION_BAD_FLOW
    static int cycle_num = 0;
    if (cycle_num)
        throw 1;
    ++cycle_num;
#endif

    this->ARLFTTableProcess();

    OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);
}


//============================================================================//
/***************/
void AdaptiveRoutingManager::AROSMIntegrationProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    m_is_temporary_error =
        ((!m_is_permanent_error) &&
         (m_is_temporary_error || m_ar_clbck.m_is_temporary_error));

    if (m_is_temporary_error) {
        m_p_osm_subn->subnet_initialization_error = TRUE;
        OSM_AR_LOG(m_p_osm_log, OSM_LOG_ERROR,
                   "Temporary error. set subnet_initialization_error = TRUE.\n");
    }

    /*
     * Step 1 -
     *  Decide for each switch the status
     */
    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    while (sw_node_it != this->m_sw_db.m_sw_map.end()) {
        osm_switch_t *p_osm_sw = sw_node_it->second.m_general_sw_info.m_p_osm_sw;

        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                "-----\n" \
                "Switch GUID " U64H_FMT ", LID %u \n",
                sw_node_it->second.m_general_sw_info.m_guid,
                sw_node_it->second.m_general_sw_info.m_lid);
#ifdef _DEBUG_AR_OSM_INTEGRATION
        printf("-----\n" \
                "Switch GUID " U64H_FMT ", LID %u\n",
                sw_node_it->second.m_general_sw_info.m_guid,
                sw_node_it->second.m_general_sw_info.m_lid);
#endif


        if ((sw_node_it->second.m_app_data == ARMGR_SW_UPDATED) &&
                (this->IsARActive(sw_node_it->second))) {
            /*
             * Step 1.1 -
             *  we configured ar for this switch, need to mark it
             */

            // mark configured
            p_osm_sw->ar_configured = 1;
        } else {

            //all AR configuration will be deleated from the switch
            //in case we had temp error (timeout) we will need to reconfigure.
            if(sw_node_it->second.m_ar_info.group_cap != 0)
                sw_node_it->second.ClearARData();
            /*
             * Step 1.2 -
             *  we didn't configured ar for this switch, need to mark it
             *  and also abort ar if it was enabled in previous cycle
             */
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "AR NOT configured for the above switch\n");
#ifdef _DEBUG_AR_OSM_INTEGRATION
            printf("AR NOT configured for the above switch\n");
#endif

            if (p_osm_sw->ar_configured) {
                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                        "Clear old lft for the above switch (because was configured in previous cycle)\n");
#ifdef _DEBUG_AR_OSM_INTEGRATION
                printf("Clear old lft for the above switch (because was configured in previous cycle)\n");
#endif
                // mark to SM to perform LFT set
                p_osm_sw->ar_configured = 2;
            }
        }

        ++sw_node_it;
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

/****************************************************/
/***************** public methods *******************/
/****************************************************/
//============================================================================//
AdaptiveRoutingManager::AdaptiveRoutingManager(osm_opensm_t *p_osm):
        m_p_osm(p_osm),
        m_p_osm_vendor(p_osm->p_vendor),
        m_p_osm_subn(&p_osm->subn),
        m_p_osm_log(&p_osm->log),
        m_thread_pool(&p_osm->log),
        m_port_groups_calculator(this, &m_thread_pool, &p_osm->log, &m_sw_db),
        m_ar_clbck(&p_osm->log),
        m_conf_file_name(AR_CFG_INPUT_FILE_NAME),
        m_p_error_window_arr(NULL),
        m_is_permanent_error(false),
        m_is_temporary_error(false),
        m_df_configured(false),
        m_ar_algorithm(NULL), //!!! TAMIR add to OSM
        m_sw_info_configured(false),
        m_max_df_group_number(0),
        m_group_discovered(false),
        m_options_file_crc(0),
        m_mkey_manager(p_osm)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    this->ParseConfFileName(this->m_p_osm->subn.opt.event_plugin_options);
    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO, "Configuration file is %s\n",
            this->m_conf_file_name.c_str());
    fprintf(stdout, "AR Manager - Configuration file is %s\n",
            this->m_conf_file_name.c_str());
    m_ar_clbck.SetMaxTimeoutNum(&m_master_db.m_max_timeout_num);
    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


//============================================================================//
AdaptiveRoutingManager::~AdaptiveRoutingManager()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO, "Exiting...\n");
    if (m_p_error_window_arr != NULL)
        delete[] m_p_error_window_arr;
    tt_log_destroy();
    delete m_ar_algorithm; //!!! TAMIR add to OSM
    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


//============================================================================//
void AdaptiveRoutingManager::SetOSMEventId(osm_epi_event_id_t osm_event_id)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    this->m_osm_event_id = osm_event_id;
    switch (this->m_osm_event_id) {
    case OSM_EVENT_ID_HEAVY_SWEEP_DONE: {
        break;
    }
    case OSM_EVENT_ID_START_LFT_UPDATE: {
        break;
    }
    case OSM_EVENT_ID_SUBNET_UP: {
        break;
    }
    case OSM_EVENT_ID_STATE_CHANGE: {
        break;
    }
    default: {
        break;
    }
    }
    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


//============================================================================//
void AdaptiveRoutingManager::UpdateFabricSwitches()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    // Mark all switches as not updated
    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    while (sw_node_it != this->m_sw_db.m_sw_map.end()) {
        sw_node_it->second.m_app_data = ARMGR_SW_NOT_UPDATED;
        ++sw_node_it;
    }

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG, "UpdateFabricSwitches\n");

    // Add new switches from osm database and update details on exist ones
    this->AddNewAndUpdateExistSwitches();

    // Remove non exists switches (in osm database) from our database
    this->RemoveAbsentSwitches();

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


//============================================================================//
int AdaptiveRoutingManager::Run()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    int rc = 0;

    bool prev_temporary_error = m_is_temporary_error;
    m_is_permanent_error = false;
    m_is_temporary_error = false;
    m_ar_clbck.m_is_temporary_error = false;

    static unsigned cycle_num = 0;
    ++cycle_num;

    uint32_t options_file_crc =
        osm_calc_file_crc32(this->m_conf_file_name.c_str());

    if (!prev_temporary_error &&
        options_file_crc == m_options_file_crc &&
        m_master_db.m_enable &&
        m_p_osm->sm.ucast_mgr.cache_valid) {

        if (m_p_osm->sm.ucast_mgr.vlid_cache_valid) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_VERBOSE,
                       "Skip Configuration cycle on valid ucast cache\n");
        } else {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_VERBOSE,
                       "Short vlid Configuration cycle.\n");

            //Update Vlid lfts
            if (m_ar_algorithm) {
                m_ar_algorithm->UpdateVlidsLfts();
            } else if (m_master_db.m_ar_algorithm == AR_ALGORITHM_DF_PLUS) {
                UpdateVlidsLftsDF();
            } else {
                UpdateVlidsLfts();
            }

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_VERBOSE,
                       "End short vlid Configuration cycle.\n");
        }
        OSM_AR_LOG_RETURN(this->m_p_osm_log, 0);
    }
    m_options_file_crc = options_file_crc;

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO, "Started configuration cycle\n");
    try {
        // Take the user options
        this->UpdateUserOptions();

        //!!! TAMIR add to OSM
        //LAG and TREE do not support temporary error
        //Need to check how opensm support it.
        if((m_master_db.m_ar_algorithm == AR_ALGORITHM_LAG) ||
           (m_master_db.m_ar_algorithm == AR_ALGORITHM_TREE) ||
            !this->m_master_db.m_enable) {
            // Mark all switches as not updated in this cycle
            map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it =
                this->m_sw_db.m_sw_map.begin();
            while (sw_node_it != this->m_sw_db.m_sw_map.end()) {
                sw_node_it->second.m_app_data = ARMGR_SW_NOT_UPDATED;
                ++sw_node_it;
            }
        }

        this->Init();

        if (m_ar_algorithm) {
            rc = m_ar_algorithm->RunCycle();
        } else{
            rc = ARCycle();
        }

    } catch (int error) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                "%s Caught exception = %u, exiting AR Manager...\n",
                ARMGR_ERR_PREFIX0, error);
        rc = 1;
    }

    this->AROSMIntegrationProcess();
    if (rc) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO, "Configuration cycle (number %u) failed\n", cycle_num);
        fprintf(stdout, "AR Manager - Configuration cycle (number %u) failed\n", cycle_num);
    } else {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO, "Configuration cycle (number %u) completed successfully\n", cycle_num);
        fprintf(stdout, "AR Manager - Configuration cycle (number %u) completed successfully\n", cycle_num);
    }
    OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);
}
