/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2011, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */


#include "ar_mgr.h"
#include "ar_mgr_api.h"


//============================================================================//
/* globals */
static AdaptiveRoutingManager *arMgr = NULL;


//============================================================================//
int initARMgr(osm_opensm_t *p_osm)
{
    try {
        if (arMgr)
            delete arMgr;
        arMgr = new AdaptiveRoutingManager(p_osm);
        OSM_LOG(&p_osm->log, OSM_LOG_VERBOSE,
                "AR_MGR - Created Adaptive Routing Manager.\n");
        return 0;
    } catch (exception &e) {
        AdaptiveRoutingManager::printException(e, &p_osm->log);
        return 1;
    }
}


//============================================================================//
int runARMgr(osm_opensm_t *p_osm,
        osm_epi_event_id_t osm_event_id)
{
#ifdef _DEBUG_AR_ONLY_ONE_CYCLE
    static int cycle_id = 0;
    if (cycle_id)
        return 0;
    ++cycle_id;
#endif

    try {
        if (!arMgr)
            return 1;
        arMgr->SetOSMEventId(osm_event_id);
        return arMgr->Run();
    } catch (exception &e) {
        AdaptiveRoutingManager::printException(e, &p_osm->log);
        return 1;
    }
}


//============================================================================//
int updateARMgr(osm_opensm_t *p_osm,
        osm_epi_event_id_t osm_event_id)
{
    try {
        if (!arMgr)
            return 1;
        arMgr->SetOSMEventId(osm_event_id);
        arMgr->UpdateFabricSwitches();
        return 0;
    } catch (exception &e) {
        AdaptiveRoutingManager::printException(e, &p_osm->log);
        return 1;
    }
}


//============================================================================//
void setOSMEventIdARMgr(osm_opensm_t *p_osm,
        osm_epi_event_id_t osm_event_id)
{
    try {
        if (!arMgr)
            return;
        arMgr->SetOSMEventId(osm_event_id);
    } catch (exception &e) {
        AdaptiveRoutingManager::printException(e, &p_osm->log);
    }
}


//============================================================================//
void destroyARMgr(osm_opensm_t *p_osm)
{
    OSM_LOG(&p_osm->log, OSM_LOG_VERBOSE,
            "AR_MGR - Destroy Adaptive Routing Manager.\n");
    try {
        if (arMgr)
            delete arMgr;
    } catch (exception &e) {
        AdaptiveRoutingManager::printException(e, &p_osm->log);
    }
}
