/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2011, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */

/* osm_config.h needs to be in all cpp files before all other includes */
#include <opensm/osm_config.h>
#include <sys/stat.h>
#include "ar_mgr.h"


/****************************************************/
// globals
struct MasterDataBase ar_conf_general_opt_db;
map < u_int64_t, map < string, string > > ar_conf_sw_opt_db;


// The parser itself is part of the ar_conf_parser_bison.yy
extern int
arParseConfFile(const char *fileName);


/****************************************************/
/**************** private methods *******************/
/****************************************************/
//============================================================================//
void AdaptiveRoutingManager::ResetErrorWindow()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    if (!this->m_master_db.m_error_window) {
        // Disable error checking mechanism
        OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
    }
    if (!this->m_master_db.m_max_errors) {
        // No need to allocate array
        OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
    }
    if (this->m_p_error_window_arr) {
        delete[] this->m_p_error_window_arr;
        this->m_p_error_window_arr = NULL;
    }
    if (!this->m_p_error_window_arr) {
        this->m_p_error_window_arr = new struct timeval[this->m_master_db.m_max_errors];
        for (unsigned int idx = 0; idx < this->m_master_db.m_max_errors; idx++) {
            this->m_p_error_window_arr[idx].tv_sec  = 0;
            this->m_p_error_window_arr[idx].tv_usec = 0;
        }
        this->m_oldest_error = this->m_master_db.m_max_errors - 1;
        this->m_num_errors = 0;
    }
    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


/***************/
void AdaptiveRoutingManager::SetDefaultConfParams()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    // Set Master Database
    ar_conf_general_opt_db.m_enable                 = AR_CFG_DEFAULT_ENABLE;
    ar_conf_general_opt_db.m_ar_enable              = AR_CFG_DEFAULT_AR_ENABLE;
    ar_conf_general_opt_db.m_arn_enable             = AR_CFG_DEFAULT_ARN_ENABLE;
    ar_conf_general_opt_db.m_flfr_enable            = AR_CFG_DEFAULT_FLFR_ENABLE;
    ar_conf_general_opt_db.m_flfr_remote_disable    = AR_CFG_DEFAULT_FLFR_REMOTE_DISABLE;
    ar_conf_general_opt_db.m_max_errors             = AR_CFG_DEFAULT_MAX_ERRORS;
    ar_conf_general_opt_db.m_error_window           = AR_CFG_DEFAULT_ERROR_WINDOW;
    ar_conf_general_opt_db.m_ar_log_file_name       = AR_CFG_DEFAULT_LOG_FILE_NAME;
    ar_conf_general_opt_db.m_ar_log_file_size       = AR_CFG_DEFAULT_LOG_FILE_SIZE;
    ar_conf_general_opt_db.m_ar_mode                = AR_CFG_DEFAULT_SW_AR_MODE;
    ar_conf_general_opt_db.m_ageing_time            = AR_CFG_DEFAULT_SW_AGEING_TIME;
    ar_conf_general_opt_db.m_ar_algorithm_str       = AR_CFG_DEFAULT_ALGORITHM_STR;
    ar_conf_general_opt_db.m_en_sl_mask             = AR_CFG_DEFAULT_EN_SL_MASK;
    ar_conf_general_opt_db.m_dfp_en_vl_mask         = AR_CFG_DEFAULT_DFP_EN_VL_MASK;
    ar_conf_general_opt_db.m_disable_tr_mask        = 0x0;
    ar_conf_general_opt_db.m_op_mode                = 0x0;
    ar_conf_general_opt_db.m_max_timeout_num = AR_CFG_DEFAULT_MAX_TIMEOUT_NUM;
    ar_conf_general_opt_db.m_max_cas_on_spine       = AR_CFG_DEFAULT_MAX_CAS_ON_SPINE;

    // Set SW Databse
    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    while (sw_node_it != this->m_sw_db.m_sw_map.end()) {
        sw_node_it->second.m_ageing_time_value = AR_CFG_DEFAULT_SW_AGEING_TIME;
        sw_node_it->second.m_option_on = AR_CFG_DEFAULT_SW_ENABLE;
        ++sw_node_it;
    }
    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void AdaptiveRoutingManager::TakeParsedConfParams()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    // Update Master Database
    this->m_master_db.m_enable              = ar_conf_general_opt_db.m_enable &&
                                              (ar_conf_general_opt_db.m_ar_enable ||
                                               ar_conf_general_opt_db.m_flfr_enable);
    this->m_master_db.m_ar_enable           = ar_conf_general_opt_db.m_ar_enable &&
                                              this->m_master_db.m_enable;
    this->m_master_db.m_arn_enable          = ar_conf_general_opt_db.m_arn_enable &&
                                              this->m_master_db.m_ar_enable;
    this->m_master_db.m_fr_enable           = ar_conf_general_opt_db.m_flfr_enable &&
                                              this->m_master_db.m_enable;
    this->m_master_db.m_frn_enable          = !ar_conf_general_opt_db.m_flfr_remote_disable &&
                                              this->m_master_db.m_fr_enable;

    this->m_master_db.m_max_errors          = ar_conf_general_opt_db.m_max_errors;
    this->m_master_db.m_error_window        = ar_conf_general_opt_db.m_error_window;
    this->m_master_db.m_ar_log_file_name    = ar_conf_general_opt_db.m_ar_log_file_name;
    this->m_master_db.m_ar_log_file_size    = ar_conf_general_opt_db.m_ar_log_file_size;
    this->m_master_db.m_ar_mode             = ar_conf_general_opt_db.m_ar_mode;
    this->m_master_db.m_ageing_time         = ar_conf_general_opt_db.m_ageing_time;
    this->m_master_db.m_en_sl_mask          = ar_conf_general_opt_db.m_en_sl_mask;

    this->m_master_db.m_dfp_en_vl_mask      = ar_conf_general_opt_db.m_dfp_en_vl_mask;
    if (this->m_master_db.m_dfp_en_vl_mask != AR_CFG_DEFAULT_DFP_EN_VL_MASK) {
        OSM_AR_LOG(m_p_osm_log, OSM_LOG_INFO,
                   "Changing DFP_EN_VL_MASK (0x%x) "
                   "parameter is currently not supported using default 0x%x\n",
                   m_master_db.m_dfp_en_vl_mask,
                   AR_CFG_DEFAULT_DFP_EN_VL_MASK);

        this->m_master_db.m_dfp_en_vl_mask = AR_CFG_DEFAULT_DFP_EN_VL_MASK;
    }

    this->m_master_db.m_disable_tr_mask     = ar_conf_general_opt_db.m_disable_tr_mask;
    this->m_master_db.m_op_mode             = ar_conf_general_opt_db.m_op_mode;
    this->m_master_db.m_max_timeout_num = ar_conf_general_opt_db.m_max_timeout_num;
    this->m_master_db.m_max_cas_on_spine    = ar_conf_general_opt_db.m_max_cas_on_spine;

    if(ar_conf_general_opt_db.m_ar_algorithm_str == "TREE"){
        this->m_master_db.m_ar_algorithm = AR_ALGORITHM_TREE;
    } else if(ar_conf_general_opt_db.m_ar_algorithm_str == "DF_PLUS"){
        this->m_master_db.m_ar_algorithm = AR_ALGORITHM_DF_PLUS;
    } else if(ar_conf_general_opt_db.m_ar_algorithm_str == "HC"){
        this->m_master_db.m_ar_algorithm = AR_ALGORITHM_KDOR_HC;
    } else
        this->m_master_db.m_ar_algorithm = AR_ALGORITHM_LAG;

    //Currently only TREE supports FRN/ARN
    //Only TREE and LAG supports FR
    if (this->m_master_db.m_ar_algorithm != AR_ALGORITHM_TREE) {
        this->m_master_db.m_arn_enable = false;
        this->m_master_db.m_frn_enable = false;

        if (this->m_master_db.m_ar_algorithm != AR_ALGORITHM_LAG) {
            this->m_master_db.m_enable = this->m_master_db.m_ar_enable;
            this->m_master_db.m_fr_enable = false;
        }
    }

    // Update SW Databse
    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = this->m_sw_db.m_sw_map.begin();
    while (sw_node_it != this->m_sw_db.m_sw_map.end()) {
        map < u_int64_t, map < string, string > >::iterator it =
                ar_conf_sw_opt_db.find(sw_node_it->second.m_general_sw_info.m_guid);
        map < string, string >::iterator it2;

        // first put general option for this switch
        sw_node_it->second.m_ageing_time_value =
                this->m_master_db.m_ageing_time;

        // next switch if no specific options for this switch
        if (it == ar_conf_sw_opt_db.end()) {
            //printf("Not found configuration for switch = " U64H_FMT "\n", sw_node_it->second.m_general_sw_info.m_guid);*/
            goto next_switch;
        }
        //printf("-->Found configuration for switch = " U64H_FMT "\n", sw_node_it->second.m_general_sw_info.m_guid);*/

        // specific options for this switch
        it2 = (*it).second.begin();
        while (it2 != (*it).second.end()) {
            string opt_str = (*it2).first;
            string opt_val = (*it2).second;
            if (opt_str == AR_CFG_SW_OPT_STR_AGEING) {
                sw_node_it->second.m_ageing_time_value =
                        (u_int16_t)strtoul(opt_val.c_str(), NULL, 0);
            } else if (opt_str == AR_CFG_SW_OPT_STR_ENABLE) {
                if (opt_val == "FALSE") {
                    sw_node_it->second.m_option_on = false;
                } else {    //TRUE
                    sw_node_it->second.m_option_on = true;
                }
                break;
            }
            ++it2;
        }

next_switch:
        ++sw_node_it;
    }
    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


/***************/
bool AdaptiveRoutingManager::IsFileExists(const char *file_name)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    bool rc;

    struct stat file_info;
    memset(&file_info, 0, sizeof(file_info));

    int rc_stat = stat(file_name,
            &file_info);
    if (rc_stat == 0) {
        // We were able to get the file attributes
        // so the file obviously exists
        rc = true;
    } else {    // Not exists or not permission
        rc = false;
    }

    OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);
}


/***************/
void AdaptiveRoutingManager::UpdateUserOptions()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    static bool first_update = true;
    bool parse_ok = false;
    const char *cfg_values_type = (first_update ? "default" : "previous");

    // First default values
    this->SetDefaultConfParams();

    // Check if configuration file exists
    if (this->IsFileExists(this->m_conf_file_name.c_str()) == false) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO,
                   "Configuration file not exists, using %s values\n",
                   cfg_values_type);
        fprintf(stdout, "AR Manager - Configuration file not exists, using %s values\n",
                cfg_values_type);
    } else {
        //Try to get options from configuration file
        if (arParseConfFile(this->m_conf_file_name.c_str())) {
            this->SetDefaultConfParams();
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                       "Parse of configuration file failed, using %s values\n",
                       cfg_values_type);
            fprintf(stdout, "AR Manager - Parse of configuration file failed, using %s values\n",
                    cfg_values_type);
        } else {
            parse_ok = true;
            first_update = false;
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_INFO, "Parse of configuration file succeeded\n");
            //fprintf(stdout, "AR Manager - Parse of configuration file succeeded\n");
        }
    }
    if (parse_ok || first_update) {
        this->TakeParsedConfParams();
    }

    // Each cycle clear error window
    this->ResetErrorWindow();

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


/****************************************************/
/**************** public methods ********************/
/****************************************************/
