/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2013, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */

#include "ar_mgr.h"
#include "packets/internal_packets_layouts.h"

/****************************************************/
/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::ARGroupTableMadGetSetByLid(u_int16_t lid,
        u_int8_t  method,
        u_int16_t group_block,
        u_int8_t  group_table,
        u_int8_t  pLFTID,
        u_int8_t  reset_direction_attr,
        struct SMP_ARGroupTable *p_ar_group_table,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending SMPARGroupTable MAD by lid = %u, method = %u, "   \
            "group block = %u\n",
            lid, method, group_block);
    /*
     * attr_mod[11:0] - group block (block contains 2 groups)
     * attr_mod[13:12] - reset_direction_attr
     * attr_mod[19:16] - group table ==> in AR value is 0
     * attr_mod[31:24] - privateLFTID
     */
    u_int32_t attr_mod = 0x00000fff & group_block;
    attr_mod |= (group_table << 16);
    attr_mod |= (pLFTID << 24);
    attr_mod |= ((reset_direction_attr & 0x3) << 12);

    int rc = this->m_ibis_obj.SMPMadGetSetByLid(lid,
            method,
            AR_IB_ATTR_SMP_AR_GROUP_TABLE,
            attr_mod,
            p_ar_group_table,
            (pack_data_func_t)SMP_ARGroupTable_pack,
            (unpack_data_func_t)SMP_ARGroupTable_unpack,
            (dump_data_func_t)SMP_ARGroupTable_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff); //bits [8-15] are class specific - don't need them here
}


/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::ARGroupTableMadGetSetByDirect(direct_route_t *p_direct_route,
        u_int8_t  method,
        u_int16_t group_block,
        u_int8_t  group_table,
        u_int8_t  pLFTID,
        u_int8_t  reset_direction_attr,
        struct SMP_ARGroupTable *p_ar_group_table,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending SMPARGroupTable MAD by direct = %s, method = %u, "    \
            "group block = %u\n",
            this->m_ibis_obj.ConvertDirPathToStr(p_direct_route).c_str(), method, group_block);
    /*
     * attr_mod[11:0] - group block (block contains 2 groups)
     * attr_mod[13:12] - reset_direction_attr
     * attr_mod[19:16] - group table 
     * attr_mod[31:24] - privateLFTID
     */
    u_int32_t attr_mod = 0x00000fff & group_block;
    attr_mod |= (group_table << 16);
    attr_mod |= (pLFTID << 24);
    attr_mod |= ((reset_direction_attr & 0x3) << 12);

    int rc = this->m_ibis_obj.SMPMadGetSetByDirect(p_direct_route,
            method,
            AR_IB_ATTR_SMP_AR_GROUP_TABLE,
            attr_mod,
            p_ar_group_table,
            (pack_data_func_t)SMP_ARGroupTable_pack,
            (unpack_data_func_t)SMP_ARGroupTable_unpack,
            (dump_data_func_t)SMP_ARGroupTable_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff); //bits [8-15] are class specific - don't need them here
}


/****************************************************/
/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::ARLinearForwardingTableMadGetSetByLid(u_int16_t lid,
        u_int8_t method,
        u_int32_t lid_block,
        u_int8_t pLFTID,
        struct SMP_ARLinearForwardingTable *p_ar_linear_forwarding_table,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending SMPARLinearForwardingTable MAD by lid = %u, " \
            "method = %u, lid block = %u\n",
            lid, method, lid_block);
    /*
     * attr_mod[23:0] - lid block (block contains 32 lids)
     * attr_mod[31:24] - privateLFTID
     */
    u_int32_t attr_mod = (0x00ffffff & lid_block);
    attr_mod |= (pLFTID << 24);

    int rc = this->m_ibis_obj.SMPMadGetSetByLid(lid,
            method,
            AR_IB_ATTR_SMP_AR_LINEAR_FORWARDING_TABLE,
            attr_mod,
            p_ar_linear_forwarding_table,
            (pack_data_func_t)SMP_ARLinearForwardingTable_pack,
            (unpack_data_func_t)SMP_ARLinearForwardingTable_unpack,
            (dump_data_func_t)SMP_ARLinearForwardingTable_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff); //bits [8-15] are class specific - don't need them here
}

/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::ARLinearForwardingTableMadGetSetByDirect(direct_route_t *p_direct_route,
        u_int8_t method,
        u_int32_t lid_block,
        u_int8_t pLFTID,
        struct SMP_ARLinearForwardingTable *p_ar_linear_forwarding_table,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending SMPARLinearForwardingTable MAD by direct = %s, "  \
            "method = %u, lid block = %u\n",
            this->m_ibis_obj.ConvertDirPathToStr(p_direct_route).c_str(), method, lid_block);
    /*
     * attr_mod[23:0] - lid block (block contains 32 lids)
     * attr_mod[31:24] - privateLFTID
     */
    u_int32_t attr_mod = (0x00ffffff & lid_block);
    attr_mod |= (pLFTID << 24);

    int rc = this->m_ibis_obj.SMPMadGetSetByDirect(p_direct_route,
            method,
            AR_IB_ATTR_SMP_AR_LINEAR_FORWARDING_TABLE,
            attr_mod,
            p_ar_linear_forwarding_table,
            (pack_data_func_t)SMP_ARLinearForwardingTable_pack,
            (unpack_data_func_t)SMP_ARLinearForwardingTable_unpack,
            (dump_data_func_t)SMP_ARLinearForwardingTable_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff); //bits [8-15] are class specific - don't need them here
}

/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::ARLinearForwardingTableMadGetSetByDirect(direct_route_t *p_direct_route,
        u_int8_t method,
        u_int32_t lid_block,
        u_int8_t pLFTID,
        struct SMP_ARLinearForwardingTable_SX *p_ar_linear_forwarding_table,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending SMPARLinearForwardingTable (SX) MAD by direct = %s, "  \
            "method = %u, lid block = %u\n",
            this->m_ibis_obj.ConvertDirPathToStr(p_direct_route).c_str(), method, lid_block);
    /*
     * attr_mod[23:0] - lid block (block contains 16 lids)
     * attr_mod[31:24] - privateLFTID
     */
    u_int32_t attr_mod = (0x00ffffff & lid_block);
    attr_mod |= (pLFTID << 24);

    int rc = this->m_ibis_obj.SMPMadGetSetByDirect(p_direct_route,
            method,
            AR_IB_ATTR_SMP_AR_LINEAR_FORWARDING_TABLE_SX,
            attr_mod,
            p_ar_linear_forwarding_table,
            (pack_data_func_t)SMP_ARLinearForwardingTable_SX_pack,
            (unpack_data_func_t)SMP_ARLinearForwardingTable_SX_unpack,
            (dump_data_func_t)SMP_ARLinearForwardingTable_SX_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff); //bits [8-15] are class specific - don't need them here
}


/****************************************************/

/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::PLFTInfoMadGetSetByDirect(direct_route_t *p_direct_route,
        u_int8_t method,
        SMP_PrivateLFTInfo *p_plft_info,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending PLFTInfoMad MAD by direct = %s, "  \
            "method = %u\n",
            this->m_ibis_obj.ConvertDirPathToStr(p_direct_route).c_str(), method);
    /*
     * attr_mod must be 0
     */
    u_int32_t attr_mod = 0;

    int rc = this->m_ibis_obj.SMPMadGetSetByDirect(p_direct_route,
            method,
            AR_IB_ATTR_SMP_PLFT_INFO,
            attr_mod,
            p_plft_info,
            (pack_data_func_t)SMP_PrivateLFTInfo_pack,
            (unpack_data_func_t)SMP_PrivateLFTInfo_unpack,
            (dump_data_func_t)SMP_PrivateLFTInfo_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff); //bits [8-15] are class specific - don't need them here
}

/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::PLFTDefMadGetSetByDirect(direct_route_t *p_direct_route,
        u_int8_t method,
        u_int8_t plft_block,
        struct SMP_PrivateLFTDef *p_plft_def,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending PLFTDef MAD by direct = %s, "  \
            "method = %u, plft block = %u\n",
            this->m_ibis_obj.ConvertDirPathToStr(p_direct_route).c_str(), method, plft_block);
    /*
     * attr_mod[3-0] - block number of of 16 LFTs
     */
    u_int32_t attr_mod = (0x0000000f & plft_block);

    int rc = this->m_ibis_obj.SMPMadGetSetByDirect(p_direct_route,
            method,
            AR_IB_ATTR_SMP_PLFT_DEF,
            attr_mod,
            p_plft_def,
            (pack_data_func_t)SMP_PrivateLFTDef_pack,
            (unpack_data_func_t)SMP_PrivateLFTDef_unpack,
            (dump_data_func_t)SMP_PrivateLFTDef_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff); //bits [8-15] are class specific - don't need them here
}

/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::PLFTMapMadGetSetByDirect(direct_route_t *p_direct_route,
        u_int8_t method,
        u_int8_t pLFTID,
        struct SMP_PrivateLFTMap *p_plft_map,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending PLFTDef MAD by direct = %s, "  \
            "method = %u, pLFTIDk = %u\n",
            this->m_ibis_obj.ConvertDirPathToStr(p_direct_route).c_str(), method, pLFTID);
    /*
     * attr_mod[7-0] - privateLFTID
     */
    u_int32_t attr_mod = (0x000000ff & pLFTID);

    int rc = this->m_ibis_obj.SMPMadGetSetByDirect(p_direct_route,
            method,
            AR_IB_ATTR_SMP_PLFT_MAP,
            attr_mod,
            p_plft_map,
            (pack_data_func_t)SMP_PrivateLFTMap_pack,
            (unpack_data_func_t)SMP_PrivateLFTMap_unpack,
            (dump_data_func_t)SMP_PrivateLFTMap_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff); //bits [8-15] are class specific - don't need them here
}

/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::PortSLToPrivateLFTMapGetSetByDirect(direct_route_t *p_direct_route,
        u_int8_t method,
        u_int8_t port_block,
        struct SMP_PortSLToPrivateLFTMap *p_plft_map,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending ortSLToPrivateLFTMap MAD by direct = %s, "  \
            "method = %u, port block = %u\n",
            this->m_ibis_obj.ConvertDirPathToStr(p_direct_route).c_str(), method, port_block);
    /*
     * attr_mod[5-0] - port_block
     */
    u_int32_t attr_mod = (0x000000ff & port_block);

    int rc = this->m_ibis_obj.SMPMadGetSetByDirect(p_direct_route,
            method,
            AR_IB_ATTR_SMP_PORT_SL_TO_PLFT_MAP,
            attr_mod,
            p_plft_map,
            (pack_data_func_t)SMP_PortSLToPrivateLFTMap_pack,
            (unpack_data_func_t)SMP_PortSLToPrivateLFTMap_unpack,
            (dump_data_func_t)SMP_PortSLToPrivateLFTMap_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff); //bits [8-15] are class specific - don't need them here
}


/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::ExtendedSwitchInfoMadGetSetByDirect(direct_route_t *p_direct_route,
        u_int8_t method,
        struct SMP_ExtSWInfo *p_ext_sw_info,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending ExtendedSwitchInfo MAD by direct = %s, "  \
            "method = %u\n",
            this->m_ibis_obj.ConvertDirPathToStr(p_direct_route).c_str(), method);
    /*
     * attr_mod must be 0
     */
    u_int32_t attr_mod = 0;

    int rc = this->m_ibis_obj.SMPMadGetSetByDirect(p_direct_route,
            method,
            AR_IB_ATTR_SMP_EXTENDED_SWITCH_INFO,
            attr_mod,
            p_ext_sw_info,
            (pack_data_func_t)SMP_ExtSWInfo_pack,
            (unpack_data_func_t)SMP_ExtSWInfo_unpack,
            (dump_data_func_t)SMP_ExtSWInfo_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff); //bits [8-15] are class specific - don't need them here
}

/*
 * Returns: mad status[bits: 0-7] / IBIS_MAD_STATUS_SEND_FAILED / IBIS_MAD_STATUS_RECV_FAILED /
 *  IBIS_MAD_STATUS_TIMEOUT / IBIS_MAD_STATUS_GENERAL_ERR
 */
int AdaptiveRoutingManager::SMPSLToVLMappingTableGetSetByDirect(direct_route_t *p_direct_route,
        u_int8_t method,
        phys_port_t out_port_number,
        phys_port_t in_port_number,
        struct SMP_SLToVLMappingTable *p_slvl_mapping,
        const clbck_data_t *p_clbck_data)
{
    AR_MGR_ENTER;
    AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Sending SMPSLToVLMappingTable MAD by direct = %s"
               " out_port:%u in_port:%u \n",
               this->m_ibis_obj.ConvertDirPathToStr(p_direct_route).c_str(),
               out_port_number, in_port_number);

    u_int32_t attribute_modifier = out_port_number + (in_port_number << 8);

    if (method == IBIS_IB_MAD_METHOD_SET &&
        (tt_is_module_verbosity_active(TT_LOG_MODULE_ARMGR)) &&
            (tt_is_level_verbosity_active(TT_LOG_LEVEL_MAD))) {
        AR_MGR_LOG(TT_LOG_LEVEL_MAD, "Send the following MAD:\n");
        SMP_SLToVLMappingTable_dump(p_slvl_mapping, TT_OUT_PORT);
    }

    int rc = this->m_ibis_obj.SMPMadGetSetByDirect(p_direct_route,
            method,
            IBIS_IB_ATTR_SMP_SLVL_TABLE,
            attribute_modifier,
            p_slvl_mapping,
            (pack_data_func_t)SMP_SLToVLMappingTable_pack,
            (unpack_data_func_t)SMP_SLToVLMappingTable_unpack,
            (dump_data_func_t)SMP_SLToVLMappingTable_dump,
            p_clbck_data);
    AR_MGR_RETURN(rc & 0x00ff);   //bits [8-15] are class specific - don't need them here
}

/****************************************************/
void AdaptiveRoutingManager::ConvertARInfoToStr(
    ARGeneralSWInfo &general_sw_info,
    adaptive_routing_info &ar_info,
    char *buff)
{
    sprintf(buff,
            "\t\t\t\tAdaptive Routing Settings::\n"
            "\t\t\t\tSwitch GUID............" U64H_FMT "\n"
            "\t\t\t\tLID....................%u\n"
            "\t\t\t\tSub Groups Active......%u\n"
            "\t\t\t\tGroup Capability.......%u\n"
            "\t\t\t\tEnable By SL Mask......0x%x\n"
            "\t\t\t\tDisable By TR Mask.....0x%x\n",
            general_sw_info.m_guid,
            general_sw_info.m_lid,
            ar_info.sub_grps_active,
            ar_info.group_cap,
            (ar_info.by_sl_en ? ar_info.enable_by_sl_mask : ALL_SL_VL_AR_ENABLED),
            ar_info.by_transport_disable);
}

string AdaptiveRoutingManager::ConvertARGroupTableBlockToStr(
    struct SMP_ARGroupTable *p_ar_group_table,
    adaptive_routing_info &ar_info,
    unsigned block_num)
{
    string str = "";
    char buff[1024];
    for (unsigned block_entry = 0; block_entry < AR_GROUP_TABLE_BLOCK_SIZE; ++block_entry) {
        u_int16_t group = (block_entry+block_num*AR_GROUP_TABLE_BLOCK_SIZE)/
            (ar_info.sub_grps_active + 1);
        u_int8_t sub_group = (block_entry+block_num*AR_GROUP_TABLE_BLOCK_SIZE)%
            (ar_info.sub_grps_active + 1);

        u_int64_t bitmask = p_ar_group_table->Group[block_entry].SubGroup_0;
        if (bitmask) {
            sprintf(buff,
                    "\t\t\t\tGroup::" U32D_FMT "    Sub Group::" U32D_FMT
                    "    Mask::" U64H_FMT "    [Ports::\"%s\"]\n",
                    group, sub_group, bitmask,
                    ConvertBitMaskToStr(bitmask).c_str());
        } else {
            sprintf(buff,
                    "\t\t\t\tGroup::" U32D_FMT "    Sub Group::" U32D_FMT
                    "    Mask::%s\n",
                    group, sub_group, "Empty");
        }
        str += buff;
    }
    return (str);
}

string AdaptiveRoutingManager::ConvertARLFTTableBlockToStr(struct SMP_ARLinearForwardingTable *p_ar_lft_table,
        unsigned block_num)
{
    string str = "";
    char buff[1024];
    for (unsigned lft_entry = 0; lft_entry < AR_LFT_TABLE_BLOCK_SIZE; ++lft_entry) {
        unsigned fixed_lft_entry = FixLFTEntryNum(lft_entry);
        if (p_ar_lft_table->LidEntry[fixed_lft_entry].GroupNumber != OSM_NO_PATH) {
            sprintf(buff,
                    "\t\t\t\tLID::" U16H_FMT "    Table::%-5u    State::%-10s    Group::%-5u\n",
                    lft_entry + block_num * AR_LFT_TABLE_BLOCK_SIZE,
                    p_ar_lft_table->LidEntry[fixed_lft_entry].TableNumber,
                    ConvertARIBLidStateToStr(p_ar_lft_table->LidEntry[fixed_lft_entry].LidState).c_str(),
                    p_ar_lft_table->LidEntry[fixed_lft_entry].GroupNumber);
        } else {
            sprintf(buff,
                    "\t\t\t\tLID::" U16H_FMT "    Table::%-5u    State::%-10s    Group::%s\n",
                    lft_entry + block_num * AR_LFT_TABLE_BLOCK_SIZE,
                    p_ar_lft_table->LidEntry[fixed_lft_entry].TableNumber,
                    ConvertARIBLidStateToStr(p_ar_lft_table->LidEntry[fixed_lft_entry].LidState).c_str(),
                    "Unreachable");
        }
        str += buff;
    }
    return (str);
}

string AdaptiveRoutingManager::ConvertARLFTTableBlockToStr(struct SMP_ARLinearForwardingTable_SX *p_ar_lft_table,
        unsigned block_num)
{
    string str = "";
    char buff[1024];
    for (unsigned lft_entry = 0; lft_entry < AR_LFT_TABLE_BLOCK_SIZE_SX; ++lft_entry) {
        if (p_ar_lft_table->LidEntry[lft_entry].LidState == AR_IB_LID_STATE_BOUNDED ||
            p_ar_lft_table->LidEntry[lft_entry].LidState == AR_IB_LID_STATE_FREE) {
            sprintf(buff,
                    "\t\t\t\tLID::" U16H_FMT "    Table::%-5u    State::%-10s    "
                    "Default Port::%-5u    Group::%-5u\n",
                    lft_entry + block_num * AR_LFT_TABLE_BLOCK_SIZE_SX,
                    p_ar_lft_table->LidEntry[lft_entry].TableNumber,
                    ConvertARIBLidStateToStr(p_ar_lft_table->LidEntry[lft_entry].LidState).c_str(),
                    p_ar_lft_table->LidEntry[lft_entry].DefaultPort,
                    p_ar_lft_table->LidEntry[lft_entry].GroupNumber);
        } else {
            if (p_ar_lft_table->LidEntry[lft_entry].DefaultPort == 255)
                continue;

            sprintf(buff,
                    "\t\t\t\tLID::" U16H_FMT "    Table::%-5u    State::%-10s    "
                    "Default Port::%-5u    Group::%s\n",
                    lft_entry + block_num * AR_LFT_TABLE_BLOCK_SIZE_SX,
                    p_ar_lft_table->LidEntry[lft_entry].TableNumber,
                    ConvertARIBLidStateToStr(p_ar_lft_table->LidEntry[lft_entry].LidState).c_str(),
                    p_ar_lft_table->LidEntry[lft_entry].DefaultPort,
                    "no group assigned");
        }
        str += buff;
    }
    return (str);
}

string AdaptiveRoutingManager::ConvertSLToVLMappingToStr(
        struct SMP_SLToVLMappingTable *p_slvl_mapping)
{
    string str = "";
    char buff[1024];
    sprintf(buff,"%u, %u, %u, %u, %u, %u, %u, %u, "
            "%u, %u, %u, %u, %u, %u, %u, %u",
            p_slvl_mapping->SL0ToVL,
            p_slvl_mapping->SL1ToVL,
            p_slvl_mapping->SL2ToVL,
            p_slvl_mapping->SL3ToVL,
            p_slvl_mapping->SL4ToVL,
            p_slvl_mapping->SL5ToVL,
            p_slvl_mapping->SL6ToVL,
            p_slvl_mapping->SL7ToVL,
            p_slvl_mapping->SL8ToVL,
            p_slvl_mapping->SL9ToVL,
            p_slvl_mapping->SL10ToVL,
            p_slvl_mapping->SL11ToVL,
            p_slvl_mapping->SL12ToVL,
            p_slvl_mapping->SL13ToVL,
            p_slvl_mapping->SL14ToVL,
            p_slvl_mapping->SL15ToVL);
    str += buff;
    return (str);
}

/****************************************************/
bool AdaptiveRoutingManager::IsEqualSMPARInfo(adaptive_routing_info *p_ar_info1,
        adaptive_routing_info *p_ar_info2, bool ignoreE, bool ignoreTop)
{
    if (!ignoreE) {
        if (p_ar_info1->e != p_ar_info2->e)
            return false;
        if (p_ar_info1->e == 0x0)
            return true;
    }

    if (!ignoreTop &&
        (p_ar_info1->group_top != p_ar_info2->group_top))
        return false;

    if ((p_ar_info1->sub_grps_active != p_ar_info2->sub_grps_active) ||
        (p_ar_info1->no_fallback != p_ar_info2->no_fallback) ||
        (p_ar_info1->ageing_time_value != p_ar_info2->ageing_time_value) ||
        (p_ar_info1->by_transport_disable != p_ar_info2->by_transport_disable) ||
        (p_ar_info1->fr_enabled != p_ar_info2->fr_enabled) ||
        (p_ar_info1->rn_xmit_enabled != p_ar_info2->rn_xmit_enabled) ||
        (p_ar_info1->by_sl_en != p_ar_info2->by_sl_en) ||
        (p_ar_info1->by_sl_en && (p_ar_info1->enable_by_sl_mask !=
                                p_ar_info2->enable_by_sl_mask)))
        return false;
    return true;
}

bool AdaptiveRoutingManager::IsEqualSMPARGroupTableBlock(SMP_ARGroupTable *p_ar_group_table_block1,
        SMP_ARGroupTable *p_ar_group_table_block2)
{
    if (memcmp(p_ar_group_table_block1, p_ar_group_table_block2, sizeof(SMP_ARGroupTable)))
        return false;
    return true;
}

bool AdaptiveRoutingManager::IsEqualSMPARLftTableBlock(SMP_ARLinearForwardingTable *p_ar_lft_table_block1,
        SMP_ARLinearForwardingTable *p_ar_lft_table_block2)
{
    if (memcmp(p_ar_lft_table_block1, p_ar_lft_table_block2, sizeof(SMP_ARLinearForwardingTable)))
        return false;
    return true;
}

bool AdaptiveRoutingManager::IsEqualSMPARLftTableBlock(SMP_ARLinearForwardingTable_SX *p_ar_lft_table_block1,
        SMP_ARLinearForwardingTable_SX *p_ar_lft_table_block2)
{
    if (memcmp(p_ar_lft_table_block1, p_ar_lft_table_block2, sizeof(SMP_ARLinearForwardingTable_SX)))
        return false;
    return true;
}


/****************************************************/
void AdaptiveRoutingManager::ARLftTableBlockSetDefault(SMP_ARLinearForwardingTable *p_block)
{
    /*
     * Set all the entries in the block to default values:
     *    LID State - 2 (static)
     *    Table num - 0
     *    Group num - OSM_NO_PATH (nonexistent port)
     */
    memset(p_block, 0, sizeof(SMP_ARLinearForwardingTable));
    for (unsigned i = 0; i < AR_LFT_TABLE_BLOCK_SIZE; ++i) {
        u_int8_t lft_entry_num = FixLFTEntryNum(i);
        p_block->LidEntry[lft_entry_num].LidState = AR_IB_LID_STATE_STATIC;
        p_block->LidEntry[lft_entry_num].GroupNumber = OSM_NO_PATH;
        p_block->LidEntry[lft_entry_num].TableNumber = 0;
    }
}

void AdaptiveRoutingManager::ARLftTableBlockSetDefault(SMP_ARLinearForwardingTable_SX *p_block)
{
    /*
     * Set all the entries in the block to default values:
     *    LID State - 2 (static)
     *    Table num - 0
     *    Default Port - OSM_NO_PATH (nonexistent port)
     *    Group num - 0
     */
    memset(p_block, 0, sizeof(SMP_ARLinearForwardingTable_SX));
    for (unsigned lft_entry_num = 0; lft_entry_num < AR_LFT_TABLE_BLOCK_SIZE_SX; ++lft_entry_num) {
        p_block->LidEntry[lft_entry_num].LidState = AR_IB_LID_STATE_STATIC;
        p_block->LidEntry[lft_entry_num].DefaultPort = OSM_NO_PATH;
        p_block->LidEntry[lft_entry_num].GroupNumber = 0;
        p_block->LidEntry[lft_entry_num].TableNumber = 0;
    }
}

void AdaptiveRoutingManager::ARLftTableSetDefault(SMP_ARLinearForwardingTable p_lft[])
{
    for (unsigned i = 0; i < AR_LFT_TABLE_NUM_BLOCKS; ++i)
        ARLftTableBlockSetDefault(&p_lft[i]);
}

void AdaptiveRoutingManager::ARLftTableSetDefault(SMP_ARLinearForwardingTable_SX p_lft[])
{
    for (unsigned i = 0; i < AR_LFT_TABLE_NUM_BLOCKS_SX; ++i)
        ARLftTableBlockSetDefault(&p_lft[i]);
}

void AdaptiveRoutingManager::ARGroupTableSetDefault(SMP_ARGroupTable p_group[])
{
    for (unsigned i = 0; i < AR_GROUP_TABLE_NUM_BLOCKS; ++i)
        memset(&p_group[i], 0, sizeof(SMP_ARGroupTable));
}

