/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2019, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */

#ifndef OSM_PLUGINS_COMMON_H
#define OSM_PLUGINS_COMMON_H

#include <infiniband/ibis/ibis.h>
#include <opensm/osm_opensm.h>
#include <opensm/osm_subnet.h>
#include <opensm/osm_node.h>
#include <opensm/osm_port.h>


class OsmMKeyManager : public MKeyManager {
private:
    osm_opensm_t *p_osm;
public:
    OsmMKeyManager(osm_opensm_t *p_osm) : p_osm(p_osm) {};

    uint64_t getMkeyFromDRPath(const direct_route_t *pCurrDirectRoute) const
    {
        osm_subn_t *p_subn = &p_osm->subn;
        if (!p_subn->opt.m_key_per_port)
            return p_osm->subn.opt.m_key;

        osm_port_t *p_sm_port = osm_get_port_by_guid(p_subn, p_subn->sm_port_guid);
        osm_node_t *p_node = osm_port_get_node(p_sm_port);
        uint8_t port_num = 0;
        for (int i = 1; i < pCurrDirectRoute->length && p_node; ++i) {
            p_node = osm_node_get_remote_node(p_node,
                                              pCurrDirectRoute->path.BYTE[i],
                                              &port_num);
        }

        if (!p_node)
            return 0;

        if (osm_node_get_type(p_node) == IB_NODE_TYPE_SWITCH)
            port_num = 0;

        osm_physp_t *p_physp = osm_node_get_physp_ptr(p_node, port_num);
        if (!p_physp)
            return 0;

        return cl_ntoh64(ib_port_info_get_m_key(&p_physp->port_info));
    };

    uint64_t getMkeyByLid(u_int16_t lid) const
    {
        osm_subn_t *p_subn = &p_osm->subn;
        if (!p_subn->opt.m_key_per_port)
            return p_osm->subn.opt.m_key;

        osm_port_t *p_port = osm_get_port_by_lid_ho(p_subn, lid);
        if (!p_port)
            return 0;

        return cl_ntoh64(ib_port_info_get_m_key(&osm_port_get_physp(p_port)->port_info));
    };

};

#endif // OSM_PLUGINS_COMMON_H
