/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2019, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */

/* osm_config.h needs to be in all cpp files before all other includes */
#include <opensm/osm_config.h>

#include "plft_based_ar_algorithm.h"
#include "ar_mgr.h"

//TODO remove doplicated code: AdaptiveRoutingManager::ARUpdateDFLFTTable
void PlftBasedArAlgorithm::UpdateSwitchPlftTable(ARSWDataBaseEntry &sw_db_entry,
        struct SMP_ARLinearForwardingTable_SX p_ar_calculated_lft_table[],
        PLFTMads   &sw_plft_mads,
        u_int16_t calculated_max_lid,
        int plft_id)
{
    OSM_AR_LOG_ENTER(m_p_osm_log);

    // Update blocks
    // In fact we go over only on blocks we need  - until max lid
    unsigned supported_lft_blocks =
            calculated_max_lid / AR_LFT_TABLE_BLOCK_SIZE_SX;
    for (unsigned lft_block = 0; lft_block <= supported_lft_blocks; ++lft_block) {
        if (sw_db_entry.m_osm_update_needed ||
                !m_ar_mgr_.IsEqualSMPARLftTableBlock(
                        &p_ar_calculated_lft_table[lft_block],
                        &sw_plft_mads.m_ar_lft.m_ar_lft_table_sx[lft_block])) {

                sw_plft_mads.m_ar_lft.m_ar_lft_table_sx[lft_block] =
                    p_ar_calculated_lft_table[lft_block];
                sw_plft_mads.m_to_set_lft_table[lft_block] = true;

                OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                           "Need to set LFT block: %u plft_id: %u on "
                           "Switch GUID: " U64H_FMT ", LID: %u\n",
                           lft_block,
                           plft_id,
                           sw_db_entry.m_general_sw_info.m_guid,
                           sw_db_entry.m_general_sw_info.m_lid);
        }
    }

    if (sw_plft_mads.m_max_lid != calculated_max_lid) {
        sw_plft_mads.m_set_lft_top = true;
        sw_plft_mads.m_max_lid = calculated_max_lid;

        OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                   "Need to set LFT top: %u on "
                   "Switch GUID: " U64H_FMT ", LID: %u\n",
                   sw_plft_mads.m_max_lid,
                   sw_db_entry.m_general_sw_info.m_guid,
                   sw_db_entry.m_general_sw_info.m_lid);

    }

    OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
}

//TODO remove doplicated code: AdaptiveRoutingManager::ARMapSL2VLOnHosts
void PlftBasedArAlgorithm::CalculateSl2VlMappingOnHosts(uint16_t en_sl_mask)
{
    OSM_AR_LOG_ENTER(m_p_osm_log);
    OSM_AR_LOG(m_p_osm_log, OSM_LOG_VERBOSE, "CalculateSl2VlMappingOnHosts.\n");

    osm_node_t *p_osm_node;
    u_int8_t op_vls; //nuber of vls opcode
    u_int64_t guid;
    u_int16_t lid;
    ARCADataBaseEntry ca_db_entry;

    BuildSl2VlPerOpVl(en_sl_mask);

    GuidToCADataBaseEntryMap &ca_map = m_ar_mgr_.m_ca_db.m_ca_map;
    //if all need update, invalidate all data
    if (m_ar_mgr_.m_p_osm_subn->need_update) {
        for (GuidToCADataBaseEntryMapIter iter = ca_map.begin();
             iter != ca_map.end();
             ++iter) {
            iter->second.m_to_set_slvl_mapping = true;
        }
    }

    cl_qmap_t *p_node_guid_tbl = &m_ar_mgr_.m_p_osm_subn->node_guid_tbl;

    // Set Leaf SW
    for (p_osm_node = (osm_node_t *) cl_qmap_head(p_node_guid_tbl);
         p_osm_node != (osm_node_t *) cl_qmap_end(p_node_guid_tbl);
         p_osm_node = (osm_node_t *) cl_qmap_next(&p_osm_node->map_item)) {

        if (p_osm_node->node_info.node_type != IB_NODE_TYPE_CA) {
            continue;
        }

        for (u_int8_t port_num = 1;
             port_num <= p_osm_node->node_info.num_ports;
             port_num++) {

            osm_physp_t *p_physp = osm_node_get_physp_ptr(p_osm_node, port_num);
            if (!p_physp || !osm_link_is_healthy(p_physp)) {
                continue;
            }
            guid = ADB2C_BE64_TO_CPU(p_physp->port_guid);
            lid = ADB2C_BE16_TO_CPU(p_physp->port_info.base_lid);

            if (!(p_physp->port_info.capability_mask & IB_PORT_CAP_HAS_SL_MAP)) {

                OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                           "skip slvl_mapping on CA port GUID " U64H_FMT
                           ", LID %u SL_MAP not supported\n",
                           guid, lid);
                continue;
            }

            op_vls = ib_port_info_get_op_vls(&p_physp->port_info);
            SMP_SLToVLMappingTable &slvl_mapping = m_sl2vl_per_op_vls_[op_vls];

            GuidToCADataBaseEntryMapIter iter = ca_map.find(guid);

            if (iter != ca_map.end()) {
                if (!p_physp->need_update &&
                    !iter->second.m_to_set_slvl_mapping &&
                    !memcmp(&(iter->second.m_slvl_mapping),
                            &slvl_mapping,
                            sizeof(slvl_mapping))){
                    continue;
                }
            } else {
                GuidToCADataBaseEntryMapInsertRes res =
                    ca_map.insert(make_pair(guid, ca_db_entry));
                iter = res.first;
            }

            memcpy(&(iter->second.m_slvl_mapping), &slvl_mapping,
                   sizeof(slvl_mapping));
            iter->second.m_to_set_slvl_mapping = true;

            OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                       "Need to set SL2VL on Host GUID " U64H_FMT ", LID %u \n",
                       guid, lid);

        } //CA's ports
    } // CAs

    OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
}

void PlftBasedArAlgorithm::BuildSl2VlPerOpVl(uint16_t en_sl_mask)
{
    OSM_AR_LOG_ENTER(m_p_osm_log);

    //currently support only one vl per plane, there for:
    uint8_t vls_per_stream = m_planes_number_;

    if (en_sl_mask == m_en_sl_mask_) {
        return;
    }
    OSM_AR_LOG(m_p_osm_log, OSM_LOG_VERBOSE, "BuildSl2VlPerOpVl.\n");

    m_en_sl_mask_ = en_sl_mask;
    m_update_sl2vl_ = true;
    u_int8_t slvl_array[MAX_SL_NUMBER];
    u_int8_t num_vls;
    u_int8_t ar_enabled_vl;
    u_int8_t ar_first_enabled_vl;
    u_int8_t max_ar_enabled_vl;

    for (uint8_t op_vls = 1; op_vls <= MAX_OP_VL_CODE; ++op_vls) {

        num_vls = ( 1 << (op_vls-1));

        if (num_vls <= vls_per_stream) { //map all SLs to VL0
            memset(&m_sl2vl_per_op_vls_[op_vls], 0,
                   sizeof(SMP_SLToVLMappingTable));
        } else {
            max_ar_enabled_vl = num_vls - vls_per_stream;
            if (m_en_sl_mask_ == ALL_SL_VL_AR_ENABLED) {
                //SLs will be mapped (round robin) to:
                //VL(0) = 0, VL(n) = VL(n-1) + vls_per_stream, .. max op vl
                ar_enabled_vl = 0;
                ar_first_enabled_vl = 0;
            } else {
                //Disabled SLs will be mapped to VL 0
                //Enabled SLs will be mapped (round robin) to
                //vls_per_stream, .. max_ar_enabled_vl
                ar_enabled_vl = vls_per_stream;
                ar_first_enabled_vl = vls_per_stream;
            }

            for (u_int8_t sl = 0; sl < MAX_SL_NUMBER; ++sl) {
                if ((1 << sl) & m_en_sl_mask_) {

                    slvl_array[sl] = ar_enabled_vl;
                    ar_enabled_vl += vls_per_stream;
                    if (ar_enabled_vl > max_ar_enabled_vl) {
                        ar_enabled_vl = ar_first_enabled_vl;
                    }
                } else {
                    slvl_array[sl] = 0;
                }
                OSM_AR_LOG(m_p_osm_log, OSM_LOG_VERBOSE,
                           "slvl_mapping[%u]=%u\n",
                           sl, slvl_array[sl]);
            }

            SetSLToVLMappingTable(&m_sl2vl_per_op_vls_[op_vls], slvl_array);
        }

    } //op_vls loop

    if (OSM_LOG_DEBUG & m_p_osm_log->level) {
        OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                   "Calculated sl to vl mapping "
                   "vls_per_stream: %u en_sl_mask: 0x%x\n",
                   vls_per_stream, m_en_sl_mask_);

        for (uint8_t op_vls = 1; op_vls <= MAX_OP_VL_CODE; ++op_vls) {
            OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                       "op_vls: %u slvl_mapping %s\n",
                       op_vls,
                       AdaptiveRoutingManager::ConvertSLToVLMappingToStr(
                           &m_sl2vl_per_op_vls_[op_vls]).c_str());
        }
    }
    OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
}

void PlftBasedArAlgorithm::BuildVl2VlPerOpVl()
{
    OSM_AR_LOG_ENTER(m_p_osm_log);

    //currently support only one vl per plane, there for:
    uint8_t vls_per_stream = m_planes_number_;

    OSM_AR_LOG(m_p_osm_log, OSM_LOG_VERBOSE, "BuildVl2VlPerOpVl.\n");
    u_int8_t vl2vl_array[MAX_SL_NUMBER];
    u_int8_t inc_vl2vl_array[MAX_SL_NUMBER];
    u_int8_t num_vls;
    u_int8_t max_vl;
    u_int8_t qos_max_vl; //max vl for the current qos
    //u_int8_t reg_vl;
    //u_int8_t inc_vl;


    memset(m_vl2vl_per_op_vls_, 0, sizeof(m_vl2vl_per_op_vls_));
    memset(m_inc_vl2vl_per_op_vls_, 0, sizeof(m_inc_vl2vl_per_op_vls_));

    for (uint8_t op_vls = 2; op_vls <= MAX_OP_VL_CODE; ++op_vls) {

        num_vls = ( 1 << (op_vls-1));
        max_vl = num_vls - 1;

        /*
        vl = 0;
        inc_vl = 1;
        for (u_int8_t sl = 0; sl < MAX_SL_NUMBER; ++sl) {
            if ((sl % vls_per_stream) == 0) {
                vl = 0;
                inc_vl = 1;
            }
            vl2vl_array[sl] = vl;
            inc_vl2vl_array[sl] = inc_vl;

            if (vl < max_vl) {
                vl++;
                if (inc_vl < max_vl) {
                    inc_vl++;
                }
            }
        }
        */
        //reg_vl = 0;
        //inc_vl = 1;
        for (u_int8_t in_vl = 0, vl = 0; in_vl < MAX_SL_NUMBER; ++in_vl, ++vl) {
            //there are (max_vl / vls_per_stream) qos groups
            //(vl / vls_per_stream) + 1 ) is the qos groups number of given vl [1..last]
            //in each qos group there are vls_per_stream vls so for vl in specific
            //qos group the max vl is:
            qos_max_vl = ((vl / vls_per_stream) + 1 ) * vls_per_stream - 1;

            //when moving to new qos group, if qos_max_vl > max_vl round robin
            //back to out vl = 0
            if ((in_vl % vls_per_stream == 0) && (qos_max_vl > max_vl)) {
                vl = 0;
                //recalculate qos_max_vl with new vl==0
                qos_max_vl = vls_per_stream - 1;
            }
            qos_max_vl = min (qos_max_vl, max_vl);

            vl2vl_array[in_vl] = min(qos_max_vl, vl);
            inc_vl2vl_array[in_vl] = min(qos_max_vl, (uint8_t)(vl + 1));
        }

        SetSLToVLMappingTable(&m_vl2vl_per_op_vls_[op_vls], vl2vl_array);
        SetSLToVLMappingTable(&m_inc_vl2vl_per_op_vls_[op_vls], inc_vl2vl_array);

    } //op_vls loop

    if (OSM_LOG_DEBUG & m_p_osm_log->level) {
        OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                   "Calculated vl to vl mapping vls_per_stream: %u\n",
                   vls_per_stream);

        for (uint8_t op_vls = 1; op_vls <= MAX_OP_VL_CODE; ++op_vls) {
            OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                       "op_vls: %u vl to vl mapping %s\n",
                       op_vls,
                       AdaptiveRoutingManager::ConvertSLToVLMappingToStr(
                           &m_vl2vl_per_op_vls_[op_vls]).c_str());
        }
        for (uint8_t op_vls = 1; op_vls <= MAX_OP_VL_CODE; ++op_vls) {
            OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                       "op_vls: %u increase vl to vl mapping %s\n",
                       op_vls,
                       AdaptiveRoutingManager::ConvertSLToVLMappingToStr(
                           &m_inc_vl2vl_per_op_vls_[op_vls]).c_str());
        }
    }
    OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
}

//TODO remove doplicated code: ar_df_mgr.cpp SetSLToVLMappingTable
void PlftBasedArAlgorithm::SetSLToVLMappingTable(
    SMP_SLToVLMappingTable *p_slvl_mapping,
    u_int8_t slvl_array[MAX_SL_NUMBER])
{
    p_slvl_mapping->SL0ToVL = slvl_array[0];
    p_slvl_mapping->SL1ToVL = slvl_array[1];
    p_slvl_mapping->SL2ToVL = slvl_array[2];
    p_slvl_mapping->SL3ToVL = slvl_array[3];
    p_slvl_mapping->SL4ToVL = slvl_array[4];
    p_slvl_mapping->SL5ToVL = slvl_array[5];
    p_slvl_mapping->SL6ToVL = slvl_array[6];
    p_slvl_mapping->SL7ToVL = slvl_array[7];
    p_slvl_mapping->SL8ToVL = slvl_array[8];
    p_slvl_mapping->SL9ToVL = slvl_array[9];
    p_slvl_mapping->SL10ToVL = slvl_array[10];
    p_slvl_mapping->SL11ToVL = slvl_array[11];
    p_slvl_mapping->SL12ToVL = slvl_array[12];
    p_slvl_mapping->SL13ToVL = slvl_array[13];
    p_slvl_mapping->SL14ToVL = slvl_array[14];
    p_slvl_mapping->SL15ToVL = slvl_array[15];
}

void PlftBasedArAlgorithm::SetSlToPlftMap(
    PortSLToPLFT *p_sl_to_plft_map,
    u_int8_t sl_to_plft_array[MAX_SL_NUMBER])
{
    p_sl_to_plft_map->PLFTToPortSL0 = sl_to_plft_array[0];
    p_sl_to_plft_map->PLFTToPortSL1 = sl_to_plft_array[1];
    p_sl_to_plft_map->PLFTToPortSL2 = sl_to_plft_array[2];
    p_sl_to_plft_map->PLFTToPortSL3 = sl_to_plft_array[3];
    p_sl_to_plft_map->PLFTToPortSL4 = sl_to_plft_array[4];
    p_sl_to_plft_map->PLFTToPortSL5 = sl_to_plft_array[5];
    p_sl_to_plft_map->PLFTToPortSL6 = sl_to_plft_array[6];
    p_sl_to_plft_map->PLFTToPortSL7 = sl_to_plft_array[7];
    p_sl_to_plft_map->PLFTToPortSL8 = sl_to_plft_array[8];
    p_sl_to_plft_map->PLFTToPortSL9 = sl_to_plft_array[9];
    p_sl_to_plft_map->PLFTToPortSL10 = sl_to_plft_array[10];
    p_sl_to_plft_map->PLFTToPortSL11 = sl_to_plft_array[11];
    p_sl_to_plft_map->PLFTToPortSL12 = sl_to_plft_array[12];
    p_sl_to_plft_map->PLFTToPortSL13 = sl_to_plft_array[13];
    p_sl_to_plft_map->PLFTToPortSL14 = sl_to_plft_array[14];
    p_sl_to_plft_map->PLFTToPortSL15 = sl_to_plft_array[15];
}

int PlftBasedArAlgorithm::SetPlftDef(ARSWDataBaseEntry &db_entry,
                                     uint8_t bank_size_kb,
                                     uint8_t banks_number,
                                     uint8_t plft_size_kb,
                                     uint8_t plfts_number,
                                     SMP_PrivateLFTDef &plft_def,
                                     bool &to_set_plft_def)
{
    OSM_AR_LOG_ENTER(m_p_osm_log);

    SMP_PrivateLFTDef calc_plft_def;
    CLEAR_STRUCT(calc_plft_def);

    uint8_t offset = 0;
    uint8_t bank = 0;
    int plft = 0;
    for (; plft < plfts_number; ++plft) {
        if ((offset + plft_size_kb) > bank_size_kb) {
            bank++;
            offset = 0;
        }
        if (bank >= banks_number) {
            break;
        }

        calc_plft_def.LFT[plft].FDB_LID_Offset = offset;
        calc_plft_def.LFT[plft].FDB_LID_Space = plft_size_kb;
        calc_plft_def.LFT[plft].FDB_Table_IX = bank;

        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                   "SetPlftDef GUID " U64H_FMT ", LID: %u "
                   "plft: %u ofset: %u plft_size: %u table_idx: %u.\n",
                   db_entry.m_general_sw_info.m_guid,
                   db_entry.m_general_sw_info.m_lid,
                   plft,
                   calc_plft_def.LFT[plft].FDB_LID_Offset,
                   calc_plft_def.LFT[plft].FDB_LID_Space,
                   calc_plft_def.LFT[plft].FDB_Table_IX);

        offset += plft_size_kb;
    }

    if (plft != plfts_number) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
            "SetPlftDef failed on GUID: " U64H_FMT ", LID: %u\n",
            db_entry.m_general_sw_info.m_guid,
            db_entry.m_general_sw_info.m_lid);
        OSM_AR_LOG_RETURN(m_p_osm_log, 1);
    }

    if (db_entry.m_osm_update_needed ||
        memcmp(&plft_def, &calc_plft_def, sizeof(plft_def)))
    {
        memcpy(&plft_def, &calc_plft_def, sizeof(plft_def));
        to_set_plft_def = true;
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
            "SetPlftDef on GUID: " U64H_FMT ", LID: %u need to set plft def\n",
            db_entry.m_general_sw_info.m_guid,
            db_entry.m_general_sw_info.m_lid);
    }

    OSM_AR_LOG_RETURN(m_p_osm_log, 0);
}

void PlftBasedArAlgorithm::SetExtendedNodeInfoProcess()
{
    OSM_AR_LOG_ENTER(m_p_osm_log);
    OSM_AR_LOG(m_p_osm_log, OSM_LOG_VERBOSE, "SetExtendedNodeInfoProcess.\n");

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &m_ar_mgr_.m_ar_clbck;
    clbck_data.m_handle_data_func = SetExtendedSwitchInfoClbckDlg;

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = m_sw_map_.begin();
    for ( ; sw_node_it != m_sw_map_.end(); ++sw_node_it) {

        if (sw_node_it->second.in_temporary_error)
            continue;

        if(!IsFeatureActive(sw_node_it->second, m_algorithm_feature_)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Switch GUID: " U64H_FMT ", LID: %u - " \
                       "Algorithm (%u) not supported or not enabled, "
                       "Set VL as SL skipped.\n",
                       sw_node_it->second.m_general_sw_info.m_guid,
                       sw_node_it->second.m_general_sw_info.m_lid,
                       m_algorithm_feature_);

            // clear AR configuration from sw - SM should configure this sw
            sw_node_it->second.m_app_data = ARMGR_SW_NOT_UPDATED;
            continue;
        }

        bool *set_ext_sw_info = GetIsToSetExtSwInfoPtr(sw_node_it->second);

        if (*set_ext_sw_info) {

            struct SMP_ExtSWInfo ext_sw_info;
            ext_sw_info.SL2VL_Act = 1;

            clbck_data.m_data1 = &sw_node_it->second;
            clbck_data.m_data2 = set_ext_sw_info;

            m_ar_mgr_.ExtendedSwitchInfoMadGetSetByDirect(
                &sw_node_it->second.m_general_sw_info.m_direct_route,
                IBIS_IB_MAD_METHOD_SET,
                &ext_sw_info, &clbck_data);
        }
    }

    //Receive all Set ExtSwInfo reply
    m_ar_mgr_.m_ibis_obj.MadRecAll();

    if (m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_EXT_SW_INFO]) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                   "Set Ext SW info error count: %u.\n",
                   m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_EXT_SW_INFO]);
        m_ar_mgr_.m_is_temporary_error = true; //Need to recalculate AR groups and LFTS
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void PlftBasedArAlgorithm::MapHostsSl2VlProcess()
{
    OSM_AR_LOG_ENTER(m_p_osm_log);
    OSM_AR_LOG(m_p_osm_log, OSM_LOG_VERBOSE, "MapHostsSl2VlProcess.\n");

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &m_ar_mgr_.m_ar_clbck;
    clbck_data.m_handle_data_func = SetSL2VLMapOnHostsClbckDlg;
    direct_route_t direct_route;

    u_int64_t guid;
    u_int16_t lid;

    cl_qmap_t *p_node_guid_tbl = &m_ar_mgr_.m_p_osm_subn->node_guid_tbl;
    osm_node_t *p_osm_node;

    GuidToCADataBaseEntryMap &ca_map = m_ar_mgr_.m_ca_db.m_ca_map;
    // Go over all CAs instead of iterate on ca_map because CAs may became invalid
    //between cycles
    for (p_osm_node = (osm_node_t *) cl_qmap_head(p_node_guid_tbl);
         p_osm_node != (osm_node_t *) cl_qmap_end(p_node_guid_tbl);
         p_osm_node = (osm_node_t *) cl_qmap_next(&p_osm_node->map_item)) {

        if (p_osm_node->node_info.node_type != IB_NODE_TYPE_CA) {
            continue;
        }

        for (u_int8_t port_num = 1;
             port_num <= p_osm_node->node_info.num_ports;
             port_num++) {

            osm_physp_t *p_physp = osm_node_get_physp_ptr(p_osm_node, port_num);
            if (!p_physp || !osm_link_is_healthy(p_physp)) {
                continue;
            }
            guid = ADB2C_BE64_TO_CPU(p_physp->port_guid);
            lid = ADB2C_BE16_TO_CPU(p_physp->port_info.base_lid);

            if (!(p_physp->port_info.capability_mask & IB_PORT_CAP_HAS_SL_MAP)) {
                continue;
            }

            GuidToCADataBaseEntryMapIter iter = ca_map.find(guid);

            if (iter != ca_map.end() &&
                iter->second.m_to_set_slvl_mapping) {

                OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                           "Send Map SL2VL on Host GUID " U64H_FMT ", LID %u \n",
                           guid, lid);

                ConvertOSMDRToIbisDR(osm_physp_get_dr_path_ptr(p_physp),
                                     &direct_route);

                clbck_data.m_data1 = &(iter->second); //CADataBaseEntry
                clbck_data.m_data2 = (void *)(intptr_t)guid;
                clbck_data.m_data3 = (void *)(intptr_t)lid;

                m_ar_mgr_.SMPSLToVLMappingTableGetSetByDirect(
                    &direct_route,
                    IBIS_IB_MAD_METHOD_SET,
                    0, 0,
                    &iter->second.m_slvl_mapping, &clbck_data);
            } //set_slvl_mapping

        } //CA's ports
    } // CAs

    m_ar_mgr_.m_ibis_obj.MadRecAll();

    OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
}

void PlftBasedArAlgorithm::MapSwitchesVl2VlProcess()
{
    OSM_AR_LOG_ENTER(m_p_osm_log);
    OSM_AR_LOG(m_p_osm_log, OSM_LOG_VERBOSE, "MapSwitchesVl2VlProcess.\n");

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &m_ar_mgr_.m_ar_clbck;
    clbck_data.m_handle_data_func = SetVL2VLMapClbckDlg;

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = m_sw_map_.begin();

    //Set pLFT Info
    for ( ; sw_node_it != m_sw_map_.end(); ++sw_node_it) {

        if (sw_node_it->second.in_temporary_error)
            continue;

        if(!IsFeatureActive(sw_node_it->second, m_algorithm_feature_)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Switch GUID: " U64H_FMT ", LID: %u - " \
                       "Algorithm (%u) not supported or not enabled, "
                       "Set VL2VL skipped.\n",
                       sw_node_it->second.m_general_sw_info.m_guid,
                       sw_node_it->second.m_general_sw_info.m_lid,
                       m_algorithm_feature_);

            // clear AR configuration from sw - SM should configure this sw
            sw_node_it->second.m_app_data = ARMGR_SW_NOT_UPDATED;

            continue;
        }

        SlToVlPtrVecVec &vl2vl = GetVl2VlMapping(sw_node_it->second);
        BoolVecVec &is_to_set_vl2vl = GetIsToSetVl2VlMapping(sw_node_it->second);

        for (u_int8_t in_port = 1;
             in_port <= sw_node_it->second.m_general_sw_info.m_num_ports;
             ++in_port) {

            for (u_int8_t out_port = 1;
                 out_port <= sw_node_it->second.m_general_sw_info.m_num_ports;
                 ++out_port) {

                if (in_port == out_port) {
                    continue;
                }

                if (!is_to_set_vl2vl[in_port][out_port]) {
                    continue;
                }

                if (m_p_osm_log->level & OSM_LOG_DEBUG) {
                    OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG,
                            "Set Vl2Vl Mappingn on "
                            "switch GUID: " U64H_FMT ", LID: %u "
                            "from port %u to port: %u %s\n",
                            sw_node_it->second.m_general_sw_info.m_guid,
                            sw_node_it->second.m_general_sw_info.m_lid,
                            in_port, out_port,
                            AdaptiveRoutingManager::ConvertSLToVLMappingToStr(
                                   vl2vl[in_port][out_port]).c_str());
                }

                clbck_data.m_data1 = &sw_node_it->second;
                clbck_data.m_data2 = (void *)(intptr_t)in_port;
                clbck_data.m_data3 = (void *)(intptr_t)out_port;
                m_ar_mgr_.SMPSLToVLMappingTableGetSetByDirect(
                    &sw_node_it->second.m_general_sw_info.m_direct_route,
                    IBIS_IB_MAD_METHOD_SET,
                    out_port, in_port,
                    vl2vl[in_port][out_port], &clbck_data);
            }//out_port
        }//in_port
    }//sw

    //Receive all Set Private LFTInfo reply
    m_ar_mgr_.m_ibis_obj.MadRecAll();

    //Not Optimal, will be replaced by OSM implementation
    for (sw_node_it = m_sw_map_.begin();
         sw_node_it != m_sw_map_.end(); ++sw_node_it) {
        if(sw_node_it->second.m_support[m_algorithm_feature_] == SUPPORTED) {
            BoolVecVec &is_to_set_vl2vl = GetIsToSetVl2VlMapping(sw_node_it->second);

            is_to_set_vl2vl[0].assign(is_to_set_vl2vl[0].size(), false);
            is_to_set_vl2vl.assign(is_to_set_vl2vl.size(), is_to_set_vl2vl[0]);
        }
    }

    if (m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_VL_TO_VL_MAP]) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                   "Set VL to VL error count: %u.\n",
                   m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_VL_TO_VL_MAP]);
        m_ar_mgr_.m_is_temporary_error = true; //Need to recalculate AR groups and LFTS
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

//TODO remove doplicated code: AdaptiveRoutingManager::ARDefinePLFTs
void PlftBasedArAlgorithm::SetPlftInfoProcess()
{
    OSM_AR_LOG_ENTER(m_p_osm_log);
    OSM_AR_LOG(m_p_osm_log, OSM_LOG_VERBOSE, "SetPlftInfoProcess.\n");

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &m_ar_mgr_.m_ar_clbck;
    clbck_data.m_handle_data_func = SetPrivateLFTInfoClbckDlg;

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = m_sw_map_.begin();

    //Set pLFT Info
    for ( ; sw_node_it != m_sw_map_.end(); ++sw_node_it) {

        if (sw_node_it->second.in_temporary_error)
            continue;

        if(!IsFeatureActive(sw_node_it->second, m_algorithm_feature_)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Switch GUID: " U64H_FMT ", LID: %u - " \
                       "Algorithm (%u) not supported or not enabled, "
                       "Set pLFT Info skipped.\n",
                       sw_node_it->second.m_general_sw_info.m_guid,
                       sw_node_it->second.m_general_sw_info.m_lid,
                       m_algorithm_feature_);

            // clear AR configuration from sw - SM should configure this sw
            sw_node_it->second.m_app_data = ARMGR_SW_NOT_UPDATED;

            continue;
        }

        SMP_PrivateLFTInfo &sw_plft_info = GetPlftInfo(sw_node_it->second);
        uint8_t required_plft_mode = GetRequiredPlftMode(sw_node_it->second);

        if (sw_plft_info.Active_Mode != required_plft_mode) {

            sw_plft_info.Active_Mode = required_plft_mode;
            clbck_data.m_data1 = &sw_node_it->second;
            clbck_data.m_data2 = &sw_plft_info;

            m_ar_mgr_.PLFTInfoMadGetSetByDirect(
                &sw_node_it->second.m_general_sw_info.m_direct_route,
                IBIS_IB_MAD_METHOD_SET,
                &sw_plft_info, &clbck_data);
        }
    }

    //Receive all Set Private LFTInfo reply
    m_ar_mgr_.m_ibis_obj.MadRecAll();

    if (m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_PLFT_INFO]) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                   "Set PLFT info error count: %u.\n",
                   m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_PLFT_INFO]);
        m_ar_mgr_.m_is_temporary_error = true; //Need to recalculate AR groups and LFTS
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void PlftBasedArAlgorithm::SetPlftDefProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_VERBOSE, "SetPlftDefProcess.\n");

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &m_ar_mgr_.m_ar_clbck;
    clbck_data.m_handle_data_func = SetPrivateLFTDefClbckDlg;

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = m_sw_map_.begin();
    for (sw_node_it = m_sw_map_.begin();
         sw_node_it != m_sw_map_.end(); ++sw_node_it) {

        if(!IsFeatureActive(sw_node_it->second, m_algorithm_feature_)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Switch GUID: " U64H_FMT ", LID: %u - " \
                       "Algorithm %u not supported or not enabled, "
                       "Set pLFT Def skipped.\n",
                       sw_node_it->second.m_general_sw_info.m_guid,
                       sw_node_it->second.m_general_sw_info.m_lid,
                       m_algorithm_feature_);

            continue;
        }

        if (sw_node_it->second.in_temporary_error)
            continue;

        if (IsToSetPlftDef(sw_node_it->second)) {

            SMP_PrivateLFTDef * p_plft_def = GetPlftDef(sw_node_it->second);

            clbck_data.m_data1 = &sw_node_it->second;
            clbck_data.m_data2 = (void *)(intptr_t)m_planes_number_;
            clbck_data.m_data3 = GetPlftNumberConfiguredPtr(sw_node_it->second);

            m_ar_mgr_.PLFTDefMadGetSetByDirect(
                &sw_node_it->second.m_general_sw_info.m_direct_route,
                IBIS_IB_MAD_METHOD_SET,
                0, //block 0
                p_plft_def, &clbck_data);
        }
    }

    //Receive all Set Private LFTDef reply
    m_ar_mgr_.m_ibis_obj.MadRecAll();

    if (m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_PLFT_DEF]) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                "Set pLFT Def error count: %u.\n",
                m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_PLFT_DEF]);
        m_ar_mgr_.m_is_temporary_error = true; //Need to recalculate AR groups and LFTS
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void PlftBasedArAlgorithm::MapPlftsProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_VERBOSE, "MapPlftsProcess.\n");

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &m_ar_mgr_.m_ar_clbck;
    clbck_data.m_handle_data_func = SetPortSlToPLFTMapClbckDlg;

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = m_sw_map_.begin();
    for (sw_node_it = m_sw_map_.begin();
         sw_node_it != m_sw_map_.end(); ++sw_node_it) {

        if(!IsFeatureActive(sw_node_it->second, m_algorithm_feature_)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Switch GUID: " U64H_FMT ", LID: %u - " \
                       "Algorithm %u not supported or not enabled, "
                       "Set pLFT Map skipped.\n",
                       sw_node_it->second.m_general_sw_info.m_guid,
                       sw_node_it->second.m_general_sw_info.m_lid,
                       m_algorithm_feature_);

            continue;
        }

        if (sw_node_it->second.in_temporary_error)
            continue;

        if (IsToSetPlftMap(sw_node_it->second)) {

            SMP_PortSLToPrivateLFTMap **p_plft_map = GetPlftMap(sw_node_it->second);

            clbck_data.m_data1 = &sw_node_it->second;
            uint8_t max_block = sw_node_it->second.m_general_sw_info.m_p_osm_sw->
                p_node->node_info.num_ports / PLFT_MAP_PORT_BLOCK_SIZE;

            for (uint8_t port_block = 0; port_block <= max_block; ++port_block) {

                clbck_data.m_data2 = (void *)(intptr_t)port_block;

                m_ar_mgr_.PortSLToPrivateLFTMapGetSetByDirect(
                    &sw_node_it->second.m_general_sw_info.m_direct_route,
                    IBIS_IB_MAD_METHOD_SET,
                    port_block,
                    p_plft_map[port_block],
                    &clbck_data);
            }
        }
    }

    //Receive all Set Private LFTDef reply
    m_ar_mgr_.m_ibis_obj.MadRecAll();

    if (m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_PLFT_MAP]) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                "Set pLFT Map error count: %u.\n",
                m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_PLFT_MAP]);
        m_ar_mgr_.m_is_temporary_error = true; //Need to recalculate AR groups and LFTS
    }

    for (sw_node_it = m_sw_map_.begin();
         sw_node_it != m_sw_map_.end(); ++sw_node_it) {
        if(sw_node_it->second.m_support[m_algorithm_feature_] == SUPPORTED) {
            ToSetPlftMap(sw_node_it->second, false);
        }
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}


//TODO remove doplicated code: AdaptiveRoutingManager::ARLFTTableProcessDF
int PlftBasedArAlgorithm::PlftProcess()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_VERBOSE, "PlftProcess.\n");
    int rc = 0;

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = m_sw_map_.begin();
    for (sw_node_it = m_sw_map_.begin();
         sw_node_it != m_sw_map_.end(); ++sw_node_it) {

        if (sw_node_it->second.in_temporary_error)
            continue;

        if(!IsFeatureActive(sw_node_it->second, m_algorithm_feature_)) {
            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                       "Switch GUID: " U64H_FMT ", LID: %u - " \
                       "Algorithm %u not supported or not enabled, "
                       "PlftProcess skipped.\n",
                       sw_node_it->second.m_general_sw_info.m_guid,
                       sw_node_it->second.m_general_sw_info.m_lid,
                       m_algorithm_feature_);

            continue;
        }

        uint8_t plft_number = *GetPlftNumberConfiguredPtr(sw_node_it->second);
        PLFTMads * plft_mads = GetPlft(sw_node_it->second);

        for (u_int8_t  plft_id = 0; plft_id < plft_number; plft_id++) {

            m_ar_mgr_.ARLFTTableProcess(
                sw_node_it->second,
                plft_mads[plft_id].m_max_lid,
                plft_id,
                plft_mads[plft_id].m_to_set_lft_table,
                plft_mads[plft_id].m_ar_lft);

            if (plft_mads[plft_id].m_set_lft_top)
                m_ar_mgr_.SetLftTop(sw_node_it->second,
                                    &plft_mads[plft_id],
                                    plft_id);
        }
    }

    //Receive all Set ARLFTTable reply
    m_ar_mgr_.m_ibis_obj.MadRecAll();

    if (m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_AR_LFT_TABLE]) {
        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                "Set AR LFT error count: %u.\n",
                m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_AR_LFT_TABLE]);
        m_ar_mgr_.m_is_temporary_error = true; //Need to recalculate AR groups and LFTS
    }

    OSM_AR_LOG_RETURN(this->m_p_osm_log, rc);
}

//TODO remove doplicated code: AdaptiveRoutingManager::ARDragonFlyCycleEnd()
void PlftBasedArAlgorithm::CycleEnd(int rc)
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);
    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_VERBOSE, "CycleEnd.\n");

    m_update_sl2vl_ = false;

    if (m_ar_mgr_.m_is_permanent_error)
        m_ar_mgr_.m_is_temporary_error = false;
    else if(rc && !m_ar_mgr_.m_is_temporary_error)
        m_ar_mgr_.m_is_permanent_error = true;

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = m_sw_map_.begin();
    for (sw_node_it = m_sw_map_.begin();
         sw_node_it != m_sw_map_.end(); ++sw_node_it) {
        RestorePriv(sw_node_it->second);
    }

    if (m_ar_mgr_.m_is_permanent_error) {
        ClearAlgorithmConfiguration();
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
}

void PlftBasedArAlgorithm::ClearAlgorithmConfiguration()
{
    OSM_AR_LOG_ENTER(this->m_p_osm_log);

    m_ar_mgr_.m_ar_clbck.ResetErrorCount();

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &m_ar_mgr_.m_ar_clbck;
    clbck_data.m_handle_data_func = SetExtendedSwitchInfoClbckDlg;

    struct SMP_ExtSWInfo ext_sw_info;
    ext_sw_info.SL2VL_Act = 0; //SL2VL

    OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
               "Clear All PlftBasedArAlgorithm Configuration\n");

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it = m_sw_map_.begin();
    for (sw_node_it = m_sw_map_.begin();
         sw_node_it != m_sw_map_.end(); ++sw_node_it) {


        // clear AR configuration from sw - SM should configure this sw
        sw_node_it->second.m_app_data = ARMGR_SW_NOT_UPDATED;

        bool *set_ext_sw_info = GetIsToSetExtSwInfoPtr(sw_node_it->second);

        //Remove VLToVL configuration
        if (set_ext_sw_info != NULL && *set_ext_sw_info == false) {

            OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_DEBUG,
                    "Disable VL2VL on Switch GUID " U64H_FMT ", LID: %u\n",
                    sw_node_it->second.m_general_sw_info.m_guid,
                    sw_node_it->second.m_general_sw_info.m_lid);


            clbck_data.m_data1 = &sw_node_it->second;
            clbck_data.m_data2 = &set_ext_sw_info;

            m_ar_mgr_.ExtendedSwitchInfoMadGetSetByDirect(
                &sw_node_it->second.m_general_sw_info.m_direct_route,
                IBIS_IB_MAD_METHOD_SET,
                &ext_sw_info, &clbck_data);

        }
    }

    m_ar_mgr_.m_ibis_obj.MadRecAll();

    if (m_ar_mgr_.m_ar_clbck.m_errcnt[AR_CLBCK_SET_EXT_SW_INFO]) {

        OSM_AR_LOG(this->m_p_osm_log, OSM_LOG_ERROR,
                   "Clear PlftBasedArAlgorithm configuration was not completed.\n");
        OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);
    }

    sw_node_it = m_sw_map_.begin();
    for (sw_node_it = m_sw_map_.begin();
         sw_node_it != m_sw_map_.end(); ++sw_node_it) {
        ClearSwitchAlgotithmData(sw_node_it->second);
        //Setting/Unsetting PLFT invalidate groups data
        sw_node_it->second.ClearARGroupsData();
    }

    OSM_AR_LOG_RETURN_VOID(this->m_p_osm_log);

}

//TODO remove doplicated code: AdaptiveRoutingManager::UpdateVlidsLftsDF()
void PlftBasedArAlgorithm::UpdateVlidsLfts()
{
    OSM_AR_LOG_ENTER(m_p_osm_log);
    OSM_AR_LOG(m_p_osm_log, OSM_LOG_DEBUG, "UpdateVlidsLftsDF.\n");

    LidsToLidsVecVec sw_lid_to_vlids;
    m_ar_mgr_.BuildSwToVlidsMap(sw_lid_to_vlids);

    if (sw_lid_to_vlids.empty()) {
        OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
        return;
    }

    map<u_int64_t, ARSWDataBaseEntry>::iterator sw_node_it =
        m_sw_map_.begin();
    for ( ; sw_node_it != m_sw_map_.end(); ++sw_node_it) {

        uint8_t plft_number = *GetPlftNumberConfiguredPtr(sw_node_it->second);
        PLFTMads * plft_mads = GetPlft(sw_node_it->second);

        for (u_int8_t plft_id = 0; plft_id < plft_number; plft_id++) {

            m_ar_mgr_.CalculateVlidsLft(
                sw_lid_to_vlids,
                sw_node_it->second.m_general_sw_info.m_p_osm_sw,
                plft_mads[plft_id].m_ar_lft,
                plft_mads[plft_id].m_to_set_lft_table);

            if (plft_mads[plft_id].m_max_lid < m_ar_mgr_.m_p_osm_subn->max_lid) {
                plft_mads[plft_id].m_max_lid = m_ar_mgr_.m_p_osm_subn->max_lid;
                plft_mads[plft_id].m_set_lft_top = true;
            }
        }
    }

    PlftProcess();

    OSM_AR_LOG_RETURN_VOID(m_p_osm_log);
}

