/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2019, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */

#ifndef _PLFT_BASED_AR_ALGORITHM_H_
#define _PLFT_BASED_AR_ALGORITHM_H_

#include "ar_algorithm.h"

#define MAX_SL_NUMBER 16
#define MAX_OP_VL_CODE 5

typedef vector <vector <bool> > BoolVecVec;
typedef vector <vector <SMP_SLToVLMappingTable *> > SlToVlPtrVecVec;

class PlftBasedArAlgorithm : public AdaptiveRoutingAlgorithm {
protected:

    //max supported planes in algorithm
    //currently support only planes_number == plfts_number
    const uint8_t   m_planes_number_;
    const uint8_t   m_min_vl_number_;
    //const uint8_t   m_plft_number_; //planes_number == plfts_number

    //sl2vl
    SMP_SLToVLMappingTable  m_sl2vl_per_op_vls_[MAX_OP_VL_CODE+1];
    uint16_t                m_en_sl_mask_; //used to calculate m_slvl_per_op_vl_
    bool                    m_update_sl2vl_;
    //vl2vl
    SMP_SLToVLMappingTable  m_vl2vl_per_op_vls_[MAX_OP_VL_CODE+1];
    SMP_SLToVLMappingTable  m_inc_vl2vl_per_op_vls_[MAX_OP_VL_CODE+1];


public:
    PlftBasedArAlgorithm(osm_log_t *p_osm_log,
                         GuidToSWDataBaseEntry &sw_map,
                         AdaptiveRoutingManager &ar_mgr,
                         supported_feature_t algorithm_feature,
                         uint8_t planes_number,
                         uint8_t min_vl_number) :
        AdaptiveRoutingAlgorithm(p_osm_log, sw_map, ar_mgr, algorithm_feature),
        m_planes_number_(planes_number),
        m_min_vl_number_(min_vl_number),
        m_en_sl_mask_(0),
        m_update_sl2vl_(false)
    {
        memset(m_sl2vl_per_op_vls_, 0, sizeof(m_sl2vl_per_op_vls_));
        BuildVl2VlPerOpVl();
    }

    virtual ~PlftBasedArAlgorithm() {}

    virtual int Preprocess() = 0;
    virtual int RunCycle() = 0;

    virtual int Init() = 0;
    virtual void Clear() = 0;

    virtual ThreadPoolTask *GetCalculatePortGroupsTask(
        ARSWDataBaseEntry &db_entry) = 0;

    virtual void ClearAlgorithmConfiguration();
    virtual void CalculateSwitchPortGroups(ARSWDataBaseEntry &sw_db_entry,
                                           LidMapping *p_lid_mapping) = 0;
protected:

    virtual bool *GetIsToSetExtSwInfoPtr(ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual SlToVlPtrVecVec &GetVl2VlMapping(ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual BoolVecVec &GetIsToSetVl2VlMapping(ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual SMP_PrivateLFTInfo &GetPlftInfo(ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual uint8_t GetRequiredPlftMode(ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual bool IsToSetPlftDef(ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual SMP_PrivateLFTDef *GetPlftDef(ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual SMP_PortSLToPrivateLFTMap **GetPlftMap(
        ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual bool IsToSetPlftMap(ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual void ToSetPlftMap(ARSWDataBaseEntry &sw_db_entry, bool is_to_set) = 0;
    virtual uint8_t *GetPlftNumberConfiguredPtr(
        ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual PLFTMads *GetPlft(ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual void UpdateVlidsLfts();

    virtual void RestorePriv(ARSWDataBaseEntry &sw_db_entry) = 0;
    virtual void ClearSwitchAlgotithmData(ARSWDataBaseEntry &sw_db_entry) = 0;

    void UpdateSwitchPlftTable(ARSWDataBaseEntry &sw_db_entry,
        struct SMP_ARLinearForwardingTable_SX p_ar_calculated_lft_table[],
        PLFTMads   &sw_plft_mads,
        u_int16_t calculated_max_lid,
        int plft_id);

    void CalculateSl2VlMappingOnHosts(uint16_t en_sl_mask);

    //update sw PrivateLFTDef and to_set_plft_def according to plft info
    //and required plft number and size
    int SetPlftDef(ARSWDataBaseEntry &db_entry,
                   uint8_t bank_size_kb,
                   uint8_t banks_number,
                   uint8_t plft_size_kb,
                   uint8_t plfts_number,
                   SMP_PrivateLFTDef &plft_def,
                   bool &to_set_plft_def);

    void SetSLToVLMappingTable(SMP_SLToVLMappingTable *p_slvl_mapping,
                               u_int8_t slvl_array[MAX_SL_NUMBER]);

    void SetSlToPlftMap(PortSLToPLFT *p_sl_to_plft_map,
                        u_int8_t sl_to_plft_array[MAX_SL_NUMBER]);

    void CycleEnd(int rc);

    void SetPlftInfoProcess();
    void SetPlftDefProcess();
    void MapPlftsProcess();
    int PlftProcess();

    void SetExtendedNodeInfoProcess();
    void MapHostsSl2VlProcess();
    void MapSwitchesVl2VlProcess();

private:
    int SetHcaLidMapping(osm_physp_t *p_hca_physp,
                         osm_node_t *p_remote_sw_node,
                         LidMapping &lid_mapping);

    void BuildSl2VlPerOpVl(uint16_t en_sl_mask);
    void BuildVl2VlPerOpVl();

};

#endif //_AR_ALGORITHM_H_
