/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2013, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */


#include <errno.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
//#include <infiniband/ibis/packets/packets_types.h> //removed in Ibutils2.1

#include "smparquery.h"
#include "../ar_mgr_core/packets/internal_packets_layouts.h"

/*
static void test()
{
    RNXmitPortMask obj;
    CLEAR_STRUCT(obj);
    u_int8_t buff[1024*1024];
    obj.xmit_port_mask_element[0].generate_arn = 1;
    obj.xmit_port_mask_element[0].generate_frn = 1;
    obj.xmit_port_mask_element[0].pass_on_rn = 1;

    obj.xmit_port_mask_element[1].generate_arn = 0;
    obj.xmit_port_mask_element[1].generate_frn = 0;
    obj.xmit_port_mask_element[1].pass_on_rn = 0;

    obj.xmit_port_mask_element[2].generate_arn = 0;
    obj.xmit_port_mask_element[2].generate_frn = 0;
    obj.xmit_port_mask_element[2].pass_on_rn = 0;

    obj.xmit_port_mask_element[3].generate_arn = 0;
    obj.xmit_port_mask_element[3].generate_frn = 0;
    obj.xmit_port_mask_element[3].pass_on_rn = 0;


    RNXmitPortMask_pack(&obj,buff);

    printf("RNXmitPortMask test 2 1,0  1,0\n");
    printf("0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n",
           buff[0],buff[1],buff[2],buff[3]);

    RNXmitPortMask_unpack(&obj,buff);
    printf("0.arn:%u 0.frn:%u 0.rn:%u\n",
           obj.xmit_port_mask_element[0].generate_arn,
           obj.xmit_port_mask_element[0].generate_frn,
           obj.xmit_port_mask_element[0].pass_on_rn);

    printf("1.arn:%u 1.frn:%u 1.rn:%u\n",
           obj.xmit_port_mask_element[1].generate_arn,
           obj.xmit_port_mask_element[1].generate_frn,
           obj.xmit_port_mask_element[1].pass_on_rn);

}
*/

/****************************************************/
/* globals */
static const match_rec_t match_tbl[] = {
        {"ARInfo", "ARI", &SMPARQuery::ARInfoProcess, false, false, false},
        {"ARGroupTable", "ARGT", &SMPARQuery::ARGroupTableProcess, true, true, true},
        {"ARLFTTable", "ARLT", &SMPARQuery::ARLFTTableProcess, true, false, true},
        {"PLFTInfo", "PLFTI", &SMPARQuery::PLFTInfoProcess, false, false , false},
        {"PLFTDef", "PLFTD", &SMPARQuery::PLFTDefProcess, false, false, true},
        {"PortSLToPLFTMap", "PLFTP", &SMPARQuery::PortSLToPLFTMapProcess, false, false, true},
        {"RNSubGroupDirectionTable", "DIRT", &SMPARQuery::RNSubGroupDirectionTableProcess, false, false, true},
        {"RNGenStringTable", "GSTR", &SMPARQuery::RNGenStringTableProcess, true, false, true},
        {"RNGenBySubGroupPriority", "GSGP", &SMPARQuery::RNGenBySubGroupPriorityProcess, false, false, false},
        {"RNRcvString", "RSTR", &SMPARQuery::RNRcvStringProcess, false, false, true},
        {"RNXmitPortMask", "RNXM", &SMPARQuery::RNXmitPortMaskProcess, false, false, true},
        {"PortRNCounters", "RNPC", &SMPARQuery::PortRNCountersProcess, false, false, false},
        {0, 0, 0, 0, 0, 0}
};


/****************************************************/
#include "smparquery_version.h"

const string get_tool_version()
{
    string version = "unknown";
#ifdef SMPARQUERY_VERSION
    version = SMPARQUERY_VERSION;
#endif
    return version;
}


/****************************************************/
/******************private methods*******************/
/****************************************************/
int SMPARQuery::ConvertStrToDR(direct_route_t *p_dr, string route_path)
{
    char *s, *str, *str2;
    str = str2 = new char[route_path.size() + 1];
    if (!str) {
        ERR_PRINT("Failed to allocate buff for dr\n");
        return 1;
    }
    strcpy(str, route_path.c_str());
    CLEAR_STRUCT(*p_dr);
    while (str && *str) {
        if ((s = strchr(str, ',')))
            *s = 0;
        p_dr->path.BYTE[p_dr->length] = (u_int8_t)atoi(str);
        ++p_dr->length;
        if (!s)
            break;
        str = s + 1;
    }
    delete [] str2;
    return 0;
}


void SMPARQuery::AddGeneralOptions()
{
    this->AddDescription(TOOL_DESC);
    this->AddOptions(OPTION_CA, OPTION_CA_SHORT, OPTION_CA_ARG_DESC, OPTION_CA_DESC);
    this->AddOptions(OPTION_PORT, OPTION_PORT_SHORT, OPTION_PORT_ARG_DESC, OPTION_PORT_DESC);
    this->AddOptions(OPTION_DR, OPTION_DR_SHORT, OPTION_DR_ARG_DESC, OPTION_DR_DESC);
    this->AddOptions(OPTION_LID, OPTION_LID_SHORT, OPTION_LID_ARG_DESC, OPTION_LID_DESC);
    this->AddOptions(OPTION_HELP, OPTION_HELP_SHORT, OPTION_HELP_ARG_DESC, OPTION_HELP_DESC);
    this->AddOptions(OPTION_VERSION, OPTION_VERSION_SHORT, OPTION_VERSION_ARG_DESC, OPTION_VERSION_DESC);
}


void SMPARQuery::Usage()
{
    char buff[4096], buff2[4096];
    string str = "\nUsage: ";
    const match_rec_t *r;
    unsigned n;

    sprintf(buff2,
            "    %s argrouptable 3 1  \t\t# argrouptable by lid, with block modifier\n"  \
            "    %s -D arinfo 0,1,2,3 \t\t# arinfo by direct route\n",
            TOOL_NAME, TOOL_NAME);

    n = sprintf(buff, "%s <op> <dest dr_path|lid> [op params]\n"
            "\nSupported ops (and aliases, case insensitive):\n",
            TOOL_NAME);
    for (r = match_tbl; r->name; ++r) {
        n += snprintf(buff + n,
                sizeof(buff) - n,
                "    %s (%s) <addr>%s%s%s\n",
                r->name,
                r->alias ? r->alias : "",
                r->plft ? " [<plft>]" : "",
                r->group_tbl ? " [<group_table>]" : "",
                r->block_num ? " [<blocknum>]" : "");
        if (n >= sizeof(buff))
            exit(1);
    }
    str += buff;
    str += "\n\nOptions:\n";
    str += this->GetUsageOptions();
    str += "\nExamples:\n";
    str += buff2;
    printf("%s", str.c_str());
}


//Returns: 0 - OK / 1 - parsing error / 2 - OK but need to exit
int SMPARQuery::HandleOption(string name, string value)
{
    //printf("%s %s\n", name.c_str(), value.c_str());
    if (name == OPTION_CA) {
        this->m_ca_name = value;
        return 0;
    } else if (name == OPTION_PORT) {
        this->m_port = (u_int8_t)strtoul(value.c_str(), NULL, 0);
        return 0;
    } else if (name == OPTION_DR) {
        if (this->m_to_use_lid) {
            ERR_PRINT("Can not use both LID and DR as dest addr\n");
            return 1;
        }
        this->m_to_use_dr = true;
        return 0;
    } else if (name == OPTION_LID) {
        if (this->m_to_use_dr) {
            ERR_PRINT("Can not use both LID and DR as dest addr\n");
            return 1;
        }
        this->m_to_use_lid = true;
        return 0;
    } else if (name == OPTION_VERSION) {
        INFO_PRINT("%s\n", get_tool_version().c_str());
        return 2;           //exit with no error
    } else if (name == OPTION_HELP) {
        this->Usage();
        return 2;           //exit with no error
    }

    //if we reached here than unknown option
    return 1;
}


/****************************************************/
/******************public methods********************/
/****************************************************/
SMPARQuery::SMPARQuery() : CommandLineRequester("Main"), m_initialized(false),
    m_ca_name(""), m_port(0),
    m_to_use_dr(false), m_to_use_lid(false),
    m_operation(""), m_dest(""), m_optional_args("")
{
    // Initialize main parser
    this->m_p_main_parser = new CommandLineParser(TOOL_NAME);
    if (!this->m_p_main_parser) {
        ERR_PRINT("Failed to allocate main parser\n");
        return;
    }

    // Add the known option to requester
    this->AddGeneralOptions();

    // Add the requester to the parser
    if (this->m_p_main_parser->AddRequester(this)) {
        ERR_PRINT("Failed add options to main parser, err=%s\n", this->m_p_main_parser->GetErrDesc());
        return;
    }

    this->m_initialized = true;
    return;
}


SMPARQuery::~SMPARQuery()
{
    delete this->m_p_main_parser;
    return;
}


//Returns: 0 - OK / 1 - error / 2 - OK but need to exit
int SMPARQuery::GetOpt(int argc, char **argv)
{
    int i, rc2, rc = 1;

    /*
     * finally parse all options
     */
    rc2 = this->m_p_main_parser->ParseOptions(argc, argv, true);
    if (rc2 == 1) {         //parse error and we need to dump usage
        goto usage_and_exit;
    } else if (rc2 == 2) {  //OK but need to exit
        rc = 2;
        goto exit;
    }

    /*
     * if no lid / direct route were given than mark by lid as default one
     */
    this->m_to_use_lid = true;

    /*
     * also parse operation + dest string + optional args
     */
    this->m_optional_args = this->m_p_main_parser->GetUnknownOptions();
    i = this->m_optional_args.find(' ');
    this->m_operation = this->m_optional_args.substr(0, i);
    this->m_optional_args.erase(0, i + 1);
    i = this->m_optional_args.find(' ');
    this->m_dest = this->m_optional_args.substr(0, i);
    this->m_optional_args.erase(0, i + 1);
    if (this->m_operation == "" || this->m_dest == "")
        goto usage_and_exit;

    printf("op = %s, dest = %s, rest = %s\n", this->m_operation.c_str(), this->m_dest.c_str(), this->m_optional_args.c_str());

    rc = 0;
    goto exit;

usage_and_exit:
    this->Usage();
exit:
    return rc;
}


int SMPARQuery::Run()
{
    int rc;

    /*
     * init ibis
     */
    if (this->m_ibis_obj.Init()) {
        ERR_PRINT("Failed to initialize, err=%s\n", this->m_ibis_obj.GetLastError());
        return 1;
    }

    /*
     * bind to port
     */
    if (this->m_ibis_obj.SetPort(this->m_ca_name.c_str(), this->m_port)) {
        ERR_PRINT("Failed to bind port, err=%s\n", this->m_ibis_obj.GetLastError());
        return 1;
    }

    /*
     * match the operation
     */
    const match_rec_t *r;
    op_fn_t fn = NULL;
    unsigned len = this->m_operation.size();
    for (r = match_tbl; r->name; ++r)
        if (!strncasecmp(r->name, this->m_operation.c_str(), len) ||
                (r->alias && !strncasecmp(r->alias, this->m_operation.c_str(), len)))
            fn = r->fn;

    if (!fn) {
        ERR_PRINT("Operation %s not supported\n", this->m_operation.c_str());
        return 1;
    }

    /*
     * decide if lid or direct route
     */
    if (this->m_to_use_dr) {
        direct_route_t dr;
        if (this->ConvertStrToDR(&dr, this->m_dest)) {
            ERR_PRINT("Can not resolve destination address: %s\n", this->m_dest.c_str());
            return 1;
        }
        rc = fn(&this->m_ibis_obj, 0, &dr, this->m_optional_args);
    } else {
        u_int16_t lid = (u_int16_t)strtoul(this->m_dest.c_str(), NULL, 0);
        rc = fn(&this->m_ibis_obj, lid, NULL, this->m_optional_args);
    }
    return rc;
}


/****************************************************/
/****************************************************/
/****************************************************/


int SMPARQuery::GetNodeDevId(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        u_int16_t &dev_id) {
    struct SMP_NodeInfo node_info;
    int rc = GetNodeInfo(p_ibis, lid, p_direct_route, node_info);
    if (rc)
        return rc;

    dev_id = node_info.DeviceID;
    INFO_PRINT("DevId:%d(0x%x):\n", node_info.DeviceID, node_info.DeviceID);
    return 0;
}

int SMPARQuery::GetNodeInfo(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        struct SMP_NodeInfo &node_info) {
    int rc = 0;

    if (lid) {  // send by lid
        INFO_PRINT("Getting NodeInfo from lid=%u\n",
                lid);
        rc = p_ibis->SMPNodeInfoMadGetByLid(lid,
                &node_info);
    } else {    // send by direct route
        INFO_PRINT("Getting NodeInfo from direct route=%s\n",
                p_ibis->ConvertDirPathToStr(p_direct_route).c_str());
        rc = p_ibis->SMPNodeInfoMadGetByDirect(p_direct_route,
                &node_info);
    }
    rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get NodeInfo, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get NodeInfo, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }
    return 0;
}

int SMPARQuery::PLFTInfoProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string optional_args) {
    struct SMP_PrivateLFTInfo plft_info;
    int rc, i;

    if (optional_args != "") {
        ERR_PRINT("No need for more params for this operation\n");
        return 1;
    };
    if (lid) {  // send by lid
        INFO_PRINT("Getting PLFTInfo from lid=%u\n",
                   lid);
        rc = p_ibis->SMPMadGetSetByLid(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_PLFT_INFO,
                0x0,
                &plft_info,
                (pack_data_func_t)SMP_PrivateLFTInfo_pack,
                (unpack_data_func_t)SMP_PrivateLFTInfo_unpack,
                (dump_data_func_t)SMP_PrivateLFTInfo_dump);
    } else {    // send by direct route
        INFO_PRINT("Getting PLFTInfo from direct route=%s\n",
                   p_ibis->ConvertDirPathToStr(p_direct_route).c_str());
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_PLFT_INFO,
                0x0,
                &plft_info,
                (pack_data_func_t)SMP_PrivateLFTInfo_pack,
                (unpack_data_func_t)SMP_PrivateLFTInfo_unpack,
                (dump_data_func_t)SMP_PrivateLFTInfo_dump);
    }
    rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
            (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get PrivateLFTInfo, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get PrivateLFTInfo, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    INFO_PRINT("PrivateLFTInfo:\n");
    INFO_PRINT("PLFT Active_Mode.........................%u\n", plft_info.Active_Mode);
    INFO_PRINT("PLFT ModeCap.............................%u\n", plft_info.ModeCap);
    INFO_PRINT("PLFT Fallback............................%s\n", !plft_info.NoFallback ? "Enabled" : "Disabled");
    for (i = 0; i < 4; i++) {
        INFO_PRINT("PLFT TableSize%d..........................%u\n", i+1, plft_info.Description_Mode[i].LFTableSize);
        INFO_PRINT("PLFT Number Of PLFTs%d....................%u\n", i+1, plft_info.Description_Mode[i].NumOfLFTables);
    }
    printf("\n");
    return 0;
}

int SMPARQuery::PLFTDefProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string optional_args) {
    struct SMP_PrivateLFTDef plft_def;
    int rc, i;

    i = optional_args.find(' ');
    string block_num_str = optional_args.substr(0, i);
    string group_table_str = optional_args.erase(0, i + 1);

    u_int32_t block_num = (u_int32_t)strtoul(block_num_str.c_str(), NULL, 0);

    if (block_num >= AR_LFT_TABLE_BLOCK_SIZE_SX) {
        ERR_PRINT("block number %u is out of range [0,%u]\n",
                  block_num, AR_LFT_TABLE_BLOCK_SIZE_SX - 1);
        return 1;
    }
    u_int32_t attr_mod = 0x000000f & block_num;

    if (lid) {  // send by lid
        INFO_PRINT("Getting PLFTDef from lid=%u\n",
                   lid);
        rc = p_ibis->SMPMadGetSetByLid(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_PLFT_DEF,
                attr_mod,
                &plft_def,
                (pack_data_func_t)SMP_PrivateLFTDef_pack,
                (unpack_data_func_t)SMP_PrivateLFTDef_unpack,
                (dump_data_func_t)SMP_PrivateLFTDef_dump);
    } else {    // send by direct route
        INFO_PRINT("Getting PLFTDef from direct route=%s\n",
                   p_ibis->ConvertDirPathToStr(p_direct_route).c_str());
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_PLFT_DEF,
                attr_mod,
                &plft_def,
                (pack_data_func_t)SMP_PrivateLFTDef_pack,
                (unpack_data_func_t)SMP_PrivateLFTDef_unpack,
                (dump_data_func_t)SMP_PrivateLFTDef_dump);
    }

    rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here
    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
            (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get PrivateLFTDef, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get PrivateLFTDef, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    INFO_PRINT("PrivateLFTDef:\n");
    for (i = 0; i < 16; i++) {
        INFO_PRINT("PLFT LFT%d Offset.........................%u\n", i+1, plft_def.LFT[i].FDB_LID_Offset);
        INFO_PRINT("PLFT LFT%d Space..........................%u\n", i+1, plft_def.LFT[i].FDB_LID_Space);
        INFO_PRINT("PLFT LFT%d TBLIndex.......................%u\n", i+1, plft_def.LFT[i].FDB_Table_IX);
    }
    printf("\n");
    return 0;
}

int SMPARQuery::PortSLToPLFTMapProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string optional_args) {
    struct SMP_PortSLToPrivateLFTMap plft_map;
    struct SMP_ExtSWInfo ext_sw_info;

    int rc, i;
    int maxSLVLMap = 16;

    i = optional_args.find(' ');
    string port_block_str = optional_args.substr(0, i);

    u_int32_t port_block = (u_int32_t)strtoul(port_block_str.c_str(), NULL, 0);

    u_int32_t attr_mod = 0x000003f & port_block;
    if (lid) {  // send by lid
        rc = p_ibis->SMPMadGetSetByLid(lid,
               IBIS_IB_MAD_METHOD_GET,
               AR_IB_ATTR_SMP_EXTENDED_SWITCH_INFO,
               0x0,
               &ext_sw_info,
               (pack_data_func_t)SMP_ExtSWInfo_pack,
               (unpack_data_func_t)SMP_ExtSWInfo_unpack,
               (dump_data_func_t)SMP_ExtSWInfo_dump);

        rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

        if (rc) {
            if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
                ERR_PRINT("Failed to get ExtSWInfo, err=%s\n", p_ibis->GetLastError());
            else
                ERR_PRINT("Failed to get ExtSWInfo, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
            return 1;
        }
        INFO_PRINT("Extended Switch Info:\n");
        INFO_PRINT("SL2VL_Cap..........................%u\n", ext_sw_info.SL2VL_Cap);
        INFO_PRINT("SL2VL_Act..........................%u\n", ext_sw_info.SL2VL_Act);
        INFO_PRINT("Getting PLFTMap from lid=%u\n", lid);
        //check if VL2VL supported
        if (ext_sw_info.SL2VL_Cap & 0x1)
            maxSLVLMap = 8;
        INFO_PRINT("Getting PLFTMap for Port Block=%d\n", port_block);
        rc = p_ibis->SMPMadGetSetByLid(lid,
                 IBIS_IB_MAD_METHOD_GET,
                 AR_IB_ATTR_SMP_PORT_SL_TO_PLFT_MAP,
                 attr_mod,
                 &plft_map,
                 (pack_data_func_t)SMP_PortSLToPrivateLFTMap_pack,
                 (unpack_data_func_t)SMP_PortSLToPrivateLFTMap_unpack,
                 (dump_data_func_t)SMP_PortSLToPrivateLFTMap_dump);
        if (rc) {
            if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
                ERR_PRINT("Failed to get PortSLToPrivateLFTMap, err=%s\n", p_ibis->GetLastError());
            else
                ERR_PRINT("Failed to get PortSLToPrivateLFTMap, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
            return 1;
        }
        INFO_PRINT("PortSLToPrivateLFTMap:\n");
        for (i = 0; i < 4; i++) {
            INFO_PRINT("PLFT ID FOR Port[%d] SL[0]..........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL0);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[1]..........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL1);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[2]..........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL2);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[3]..........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL3);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[4]..........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL4);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[5]..........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL5);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[6]..........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL6);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[7]..........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL7);
            if(maxSLVLMap > 8) {
                INFO_PRINT("PLFT ID FOR Port[%d] SL[8]..........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL8);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[9]..........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL9);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[10].........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL10);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[11].........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL11);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[12].........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL12);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[13].........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL13);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[14].........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL14);
            }
            INFO_PRINT("PLFT ID FOR Port[%d] SL[15].........................%u\n", (port_block * 4) + i, plft_map.PortSLToPLFT[i].PLFTToPortSL15);
        }
    } else {    // send by direct route
        INFO_PRINT("Getting PLFTMap from direct route=%s\n",
                   p_ibis->ConvertDirPathToStr(p_direct_route).c_str());
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
               IBIS_IB_MAD_METHOD_GET,
               AR_IB_ATTR_SMP_EXTENDED_SWITCH_INFO,
               0x0,
               &ext_sw_info,
               (pack_data_func_t)SMP_ExtSWInfo_pack,
               (unpack_data_func_t)SMP_ExtSWInfo_unpack,
               (dump_data_func_t)SMP_ExtSWInfo_dump);

        rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

        if (rc) {
            if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->GetLastError());
            else
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
            return 1;
        }
        INFO_PRINT("Extended Switch Info:\n");
        INFO_PRINT("SL2VL_Cap..........................%u\n", ext_sw_info.SL2VL_Cap);
        INFO_PRINT("SL2VL_Act..........................%u\n", ext_sw_info.SL2VL_Act);
        if (ext_sw_info.SL2VL_Cap & 0x1)
            maxSLVLMap = 8;
        INFO_PRINT("Getting PLFTMap for Port Block=%d\n", port_block);
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                 IBIS_IB_MAD_METHOD_GET,
                 AR_IB_ATTR_SMP_PORT_SL_TO_PLFT_MAP,
                 attr_mod,
                 &plft_map,
                 (pack_data_func_t)SMP_PortSLToPrivateLFTMap_pack,
                 (unpack_data_func_t)SMP_PortSLToPrivateLFTMap_unpack,
                 (dump_data_func_t)SMP_PortSLToPrivateLFTMap_dump);
        if (rc) {
            if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->GetLastError());
            else
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
            return 1;
        }
        INFO_PRINT("PortSLToPrivateLFTMap:\n");
        for (i = 0; i < 4; i++) {
            INFO_PRINT("PLFT ID FOR Port[%d] SL[0]..........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL0);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[1]..........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL1);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[2]..........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL2);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[3]..........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL3);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[4]..........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL4);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[5]..........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL5);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[6]..........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL6);
            INFO_PRINT("PLFT ID FOR Port[%d] SL[7]..........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL7);
            if(maxSLVLMap > 8) {
                INFO_PRINT("PLFT ID FOR Port[%d] SL[8]..........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL8);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[9]..........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL9);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[10].........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL10);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[11].........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL11);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[12].........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL12);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[13].........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL13);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[14].........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL14);
                INFO_PRINT("PLFT ID FOR Port[%d] SL[15].........................%u\n", i, plft_map.PortSLToPLFT[i].PLFTToPortSL15);
            }
        }
    }

    return 0;
}

int SMPARQuery::ARInfoProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string optional_args) {
    struct adaptive_routing_info ar_info;
    int rc;

    if (optional_args != "") {
        ERR_PRINT("No need for more params for this operation\n");
        return 1;
    };
    if (lid) {  // send by lid
        INFO_PRINT("Getting ARInfo from lid=%u\n",
                lid);
        rc = p_ibis->SMPARInfoGetSetByLid(
            lid, IBIS_IB_MAD_METHOD_GET, false, &ar_info, NULL);
        /*
        rc = p_ibis->SMPMadGetSetByLid(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_AR_INFO,
                0x0,
                &ar_info,
                (pack_data_func_t)adaptive_routing_info_pack,
                (unpack_data_func_t)adaptive_routing_info_unpack,
                (dump_data_func_t)adaptive_routing_info_dump);
                */
    } else {    // send by direct route
        INFO_PRINT("Getting ARInfo from direct route=%s\n",
                p_ibis->ConvertDirPathToStr(p_direct_route).c_str());

        rc = p_ibis->SMPARInfoGetSetByDirect(
            p_direct_route, IBIS_IB_MAD_METHOD_GET, false, &ar_info, NULL);
            /*
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_AR_INFO,
                0x0,
                &ar_info,
                (pack_data_func_t)adaptive_routing_info_pack,
                (unpack_data_func_t)adaptive_routing_info_unpack,
                (dump_data_func_t)adaptive_routing_info_dump);
                */
    }
    rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    INFO_PRINT("ARInfo:\n");
    printf("AR Status.........................%s\n", ar_info.e ? "Enabled" : "Disabled");
    printf("Is ARN Supported..................%s\n", ar_info.is_arn_sup ? "Yes" : "No");
    printf("Is FRN Supported..................%s\n", ar_info.is_frn_sup ? "Yes" : "No");
    printf("Is FR Supported...................%s\n", ar_info.is_fr_sup ? "Yes" : "No");
    printf("FR Enabled........................%s\n", ar_info.fr_enabled ? "Yes" : "No");
    printf("RN Xmit Enabled...................%s\n", ar_info.rn_xmit_enabled ? "Yes" : "No");
    printf("AR Sub Groups Active..............%d\n", ar_info.sub_grps_active);
    printf("AR Groups Copy Supported..........%d\n", ar_info.group_table_copy_sup);
    printf("Direction Num Supported...........%d\n", ar_info.direction_num_sup);
    printf("AR Fallback.......................%s\n", !ar_info.no_fallback ? "Enabled" : "Disabled");
    printf("AR IS4 mode.......................%s\n", ar_info.is4_mode ? "Yes" : "No");
    printf("AR Glb Group......................%s\n", ar_info.glb_groups ? "Yes" : "No");
    printf("AR By SL Cap......................%s\n", ar_info.by_sl_cap ? "Yes" : "No");
    printf("AR By Transport Cap...............%s\n", ar_info.by_transp_cap ? "Yes" : "No");
    printf("AR Dynamic Cap Calc...............%s\n", ar_info.dyn_cap_calc_sup ? "Yes" : "No");
    printf("AR Group Capability...............%u\n", ar_info.group_cap);
    printf("AR Group Top......................%u\n", ar_info.group_top);
    printf("AR Group Table Capability.........%u\n", ar_info.group_table_cap);
    printf("RN String Width Capability........%u\n", ar_info.string_width_cap);
    printf("AR Sub Groups Capability..........0x%01x\n", ar_info.sub_grps_supported);
    printf("AR Version........................%u\n", ar_info.ar_version_cap);
    printf("RN Version........................%u\n", ar_info.rn_version_cap);
    printf("AR By SL..........................%s\n", ar_info.by_sl_en ? "Enabled" : "Disabled");
    printf("AR Enable By SL Mask..............0x%02x\n", ar_info.enable_by_sl_mask);
    printf("AR By TransportDisable............0x%01x\n", ar_info.by_transport_disable);
    printf("AR Ageing Time Value..............%u\n", ar_info.ageing_time_value);
    printf("\n");
    return 0;



}

#define AR_GROUP_TABLE_NUM_BLOCKS_MASK  0xfff

int SMPARQuery::ARGroupTableProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string optional_args) {
    struct SMP_ARGroupTable ar_group_table;
    struct adaptive_routing_info ar_info;
    memset(&ar_group_table, 0, sizeof(SMP_ARGroupTable));
    memset(&ar_info, 0, sizeof(adaptive_routing_info));

    int rc;

    int i = optional_args.find(' ');
    string plft_id_str = optional_args.substr(0, i);
    optional_args.erase(0, i + 1);
    i = optional_args.find(' ');
    string group_table_str = optional_args.substr(0, i);
    string block_num_str = optional_args.erase(0, i + 1);

    u_int8_t  plft_id = (u_int32_t)strtoul(plft_id_str.c_str(), NULL, 0);
    u_int32_t block_num   = (u_int32_t)strtoul(block_num_str.c_str(), NULL, 0);
    u_int8_t  group_table = (u_int32_t)strtoul(group_table_str.c_str(), NULL, 0);

    if (plft_id >= MAX_SUPPORTED_PLFT_NUMBER) {
        ERR_PRINT("plft id number %u is out of range [0,%u]\n",
                  plft_id, MAX_SUPPORTED_PLFT_NUMBER - 1);
        return 1;
    }

    if (block_num >= AR_GROUP_TABLE_NUM_BLOCKS) {
        ERR_PRINT("block number %u is out of range [0,%u]\n",
                block_num, AR_GROUP_TABLE_NUM_BLOCKS - 1);
        return 1;
    }
    if (group_table >= ARMGR_MAX_GROUP_TABLES) {
        ERR_PRINT("group table number %u is out of range [0,%u]\n",
                block_num, ARMGR_MAX_GROUP_TABLES - 1);
        return 1;
    }

    /*
     * attr_mod[11:0] - group block (block contains 2 groups)
     * attr_mod[19:16] - group table
     * attr_mod[31:24] - privateLFTID ==> in ARvalue is 0
     */
    u_int32_t attr_mod = (0xff & plft_id) << 24;
    attr_mod |= (0xf & group_table) << 16;
    attr_mod |= AR_GROUP_TABLE_NUM_BLOCKS_MASK & block_num;

    if (lid) {  // send by lid
        // aquiring ARInfo for getting PSE bit
        rc = p_ibis->SMPMadGetSetByLid(lid,
               IBIS_IB_MAD_METHOD_GET,
               AR_IB_ATTR_SMP_AR_INFO,
               0x0,
               &ar_info,
               (pack_data_func_t)adaptive_routing_info_pack,
               (unpack_data_func_t)adaptive_routing_info_unpack,
               (dump_data_func_t)adaptive_routing_info_dump);

        rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

        if (rc) {
            if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                    (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->GetLastError());
            else
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
            return 1;
        }

        INFO_PRINT("Getting ARGroupTable from lid=%u, plft=%u, table=%u, block=%u \n",
                   lid, plft_id, group_table,
                   (AR_GROUP_TABLE_NUM_BLOCKS_MASK & block_num));
        rc = p_ibis->SMPMadGetSetByLid(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_AR_GROUP_TABLE,
                attr_mod,
                &ar_group_table,
                (pack_data_func_t)SMP_ARGroupTable_pack,
                (unpack_data_func_t)SMP_ARGroupTable_unpack,
                (dump_data_func_t)SMP_ARGroupTable_dump);
    } else {    // send by direct route
        // aquiring ARInfo for getting PSE bit
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
               IBIS_IB_MAD_METHOD_GET,
               AR_IB_ATTR_SMP_AR_INFO,
               0x0,
               &ar_info,
               (pack_data_func_t)adaptive_routing_info_pack,
               (unpack_data_func_t)adaptive_routing_info_unpack,
               (dump_data_func_t)adaptive_routing_info_dump);

        rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

        if (rc) {
            if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                    (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->GetLastError());
            else
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
            return 1;
        }

        INFO_PRINT("Getting ARGroupTable from direct route=%s, plft=%u, table=%u, block=%u\n",
                   p_ibis->ConvertDirPathToStr(p_direct_route).c_str(), plft_id, group_table, block_num);
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_AR_GROUP_TABLE,
                attr_mod,
                &ar_group_table,
                (pack_data_func_t)SMP_ARGroupTable_pack,
                (unpack_data_func_t)SMP_ARGroupTable_unpack,
                (dump_data_func_t)SMP_ARGroupTable_dump);
    }
    rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get ARGroupTable, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get ARGroupTable, err=%s\n\n"    \
                    "NOTE: make sure AR is Enabled\n",
                    p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    unsigned group_num;
    unsigned sub_group_num;
    INFO_PRINT("ARGroupTable (Sub groups in group: %d):\n", ar_info.sub_grps_active+1 );
    for (unsigned group_entry = 0; group_entry < AR_GROUP_TABLE_BLOCK_SIZE; ++group_entry) {
        u_int64_t bitmask = ar_group_table.Group[group_entry].SubGroup_0;

        group_num =  (group_entry + block_num * AR_GROUP_TABLE_BLOCK_SIZE) /
            (ar_info.sub_grps_active+1);
        sub_group_num = (group_entry + block_num * AR_GROUP_TABLE_BLOCK_SIZE) %
            (ar_info.sub_grps_active+1);

        if (bitmask) {
            printf("Group...%u\tSub Group...%u\tMask....."
                   U64H_FMT "\t\t[Ports...\"%s\"]\n",
                    group_num, sub_group_num, bitmask,
                    ConvertBitMaskToStr(bitmask).c_str());
        } else {
            printf("Group...%u\tSub Group...%u\tMask....."
                   U64H_FMT "\t\t[Ports...\"%s\"]\n",
                   group_num, sub_group_num, bitmask,
                    "Empty");
        }
    }
    return 0;
}

int SMPARQuery::ARLFTTableProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string optional_args) {

    int i = optional_args.find(' ');
    u_int32_t max_tbl_blocks;
    string plft_id_str = optional_args.substr(0, i);
    string block_num_str = optional_args.erase(0, i + 1);

    max_tbl_blocks = AR_LFT_TABLE_NUM_BLOCKS_SX;

    u_int32_t plft_id = (u_int32_t)strtoul(plft_id_str.c_str(), NULL, 0);
    u_int32_t block_num = (u_int32_t)strtoul(block_num_str.c_str(), NULL, 0);

    if (block_num >= max_tbl_blocks) {
        ERR_PRINT("block number %u is out of range [0,%u]\n",
                block_num, max_tbl_blocks - 1);
        return 1;
    }

    if (plft_id >= MAX_SUPPORTED_PLFT_NUMBER) {
        ERR_PRINT("plft id number %u is out of range [0,%u]\n",
                  plft_id, MAX_SUPPORTED_PLFT_NUMBER - 1);
        return 1;
    }

    return GetLFTTable_SX(p_ibis, lid, p_direct_route, plft_id, block_num);
}

int SMPARQuery::GetLFTTable(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        u_int32_t plft_id, u_int32_t block_num) {
    struct SMP_ARLinearForwardingTable ar_lft_table;
    memset(&ar_lft_table, 0, sizeof(SMP_ARLinearForwardingTable));
    int rc;

    /*
     * attr_mod[23:0] - lid block (block contains 32 lids)
     * attr_mod[31:24] - privateLFTID ==> in ARvalue is 0
     */

    u_int32_t attr_mod = (0xff & plft_id) << 24;
    attr_mod |= (0x00ffffff & block_num);

    if (lid) {  // send by lid
        INFO_PRINT("Getting ARLFTTable from lid=%u, plft=%u, block=%u\n",
                   lid, plft_id, block_num);
        rc = p_ibis->SMPMadGetSetByLid(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_AR_LINEAR_FORWARDING_TABLE,
                attr_mod,
                &ar_lft_table,
                (pack_data_func_t)SMP_ARLinearForwardingTable_pack,
                (unpack_data_func_t)SMP_ARLinearForwardingTable_unpack,
                (dump_data_func_t)SMP_ARLinearForwardingTable_dump);
    } else {    // send by direct route
        INFO_PRINT("Getting ARLFTTable from direct route=%s, block=%u\n",
                p_ibis->ConvertDirPathToStr(p_direct_route).c_str(), block_num);
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_AR_LINEAR_FORWARDING_TABLE,
                attr_mod,
                &ar_lft_table,
                (pack_data_func_t)SMP_ARLinearForwardingTable_pack,
                (unpack_data_func_t)SMP_ARLinearForwardingTable_unpack,
                (dump_data_func_t)SMP_ARLinearForwardingTable_dump);
    }
    rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get ARLFTTable, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get ARLFTTable, err=%s\n\n"    \
                    "NOTE: make sure AR is Enabled\n",
                    p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    INFO_PRINT("ARLFTTable:\n");
    for (unsigned lft_entry = 0; lft_entry < AR_LFT_TABLE_BLOCK_SIZE; ++lft_entry) {
        unsigned fixed_lft_entry = FixLFTEntryNum(lft_entry);
        if (ar_lft_table.LidEntry[fixed_lft_entry].GroupNumber != OSM_NO_PATH) {
            printf("LID....." U16H_FMT "\t\tTable.....%-5u\t\tState.....%-10s\t\tGroup.....%-5u\n",
                    lft_entry + block_num * AR_LFT_TABLE_BLOCK_SIZE,
                    ar_lft_table.LidEntry[fixed_lft_entry].TableNumber,
                    ConvertARIBLidStateToStr(ar_lft_table.LidEntry[fixed_lft_entry].LidState).c_str(),
                    ar_lft_table.LidEntry[fixed_lft_entry].GroupNumber);
        } else {
            printf("LID....." U16H_FMT "\t\tTable.....%-5u\t\tState.....%-10s\t\tGroup.....%s\n",
                    lft_entry + block_num * AR_LFT_TABLE_BLOCK_SIZE,
                    ar_lft_table.LidEntry[fixed_lft_entry].TableNumber,
                    ConvertARIBLidStateToStr(ar_lft_table.LidEntry[fixed_lft_entry].LidState).c_str(),
                    "Unreachable");
        }
    }
    return 0;
}

int SMPARQuery::GetLFTTable_SX(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        u_int32_t plft_id, u_int32_t block_num) {
    struct SMP_ARLinearForwardingTable_SX ar_lft_table;
    memset(&ar_lft_table, 0, sizeof(SMP_ARLinearForwardingTable_SX));
    int rc;

    struct adaptive_routing_info ar_info;
    rc = GetArInfo(p_ibis, lid, p_direct_route, ar_info);

    if (rc)
        return rc;

    if (!ar_info.e) {
        ERR_PRINT("AR/FR is not enabled.\n");
        return 1;
    }


    /*
     * attr_mod[23:0] - lid block (block contains 16 lids)
     * attr_mod[31:24] - privateLFTID ==> in ARvalue is 0
     */
    u_int32_t attr_mod = (0xff & plft_id) << 24;
    attr_mod |= (0x00ffffff & block_num);

    if (lid) {  // send by lid
        INFO_PRINT("Getting ARLFTTable from lid=%u, plft=%u, block=%u\n",
                   lid, plft_id, block_num);
        rc = p_ibis->SMPMadGetSetByLid(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_AR_LINEAR_FORWARDING_TABLE_SX,
                attr_mod,
                &ar_lft_table,
                (pack_data_func_t)SMP_ARLinearForwardingTable_SX_pack,
                (unpack_data_func_t)SMP_ARLinearForwardingTable_SX_unpack,
                (dump_data_func_t)SMP_ARLinearForwardingTable_SX_dump);
    } else {    // send by direct route
        INFO_PRINT("Getting ARLFTTable from direct route=%s, block=%u\n",
                p_ibis->ConvertDirPathToStr(p_direct_route).c_str(), block_num);
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_AR_LINEAR_FORWARDING_TABLE_SX,
                attr_mod,
                &ar_lft_table,
                (pack_data_func_t)SMP_ARLinearForwardingTable_SX_pack,
                (unpack_data_func_t)SMP_ARLinearForwardingTable_SX_unpack,
                (dump_data_func_t)SMP_ARLinearForwardingTable_SX_dump);
    }
    rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get ARLFTTable, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get ARLFTTable, err=%s\n\n"    \
                    "NOTE: make sure AR is Enabled\n",
                    p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    INFO_PRINT("ARLFTTable:\n");
    for (unsigned lft_entry = 0; lft_entry < AR_LFT_TABLE_BLOCK_SIZE_SX; ++lft_entry) {
        if (ar_lft_table.LidEntry[lft_entry].LidState == AR_IB_LID_STATE_BOUNDED ||
            ar_lft_table.LidEntry[lft_entry].LidState == AR_IB_LID_STATE_FREE ||
            ar_info.fr_enabled) { //if fr enabled group number
                                  //is is valid also for static
            printf("LID....." U16H_FMT "\t\tTable.....%-5u\t\tState.....%-10s\t\t"
                   "Default Port.....%-5u\t\tGroup.....%-5u\n",
                    lft_entry + block_num * AR_LFT_TABLE_BLOCK_SIZE_SX,
                    ar_lft_table.LidEntry[lft_entry].TableNumber,
                    ConvertARIBLidStateToStr(ar_lft_table.LidEntry[lft_entry].LidState).c_str(),
                    ar_lft_table.LidEntry[lft_entry].DefaultPort,
                    ar_lft_table.LidEntry[lft_entry].GroupNumber);
        } else {
            printf("LID....." U16H_FMT "\t\tTable.....%-5u\t\tState.....%-10s\t\t"
                   "Default Port.....%-5u\t\tGroup.....%s\n",
                    lft_entry + block_num * AR_LFT_TABLE_BLOCK_SIZE_SX,
                    ar_lft_table.LidEntry[lft_entry].TableNumber,
                    ConvertARIBLidStateToStr(ar_lft_table.LidEntry[lft_entry].LidState).c_str(),
                    ar_lft_table.LidEntry[lft_entry].DefaultPort,
                    "no group assigned");
        }
    }
    return 0;
}

int SMPARQuery::GetArInfo(Ibis *p_ibis, u_int16_t lid,
                          direct_route_t *p_direct_route,
                          struct adaptive_routing_info &ar_info) {

    memset(&ar_info, 0, sizeof(ar_info));

    int rc;

    if (lid) {  // send by lid
        // aquiring ARInfo for getting PSE bit
        rc = p_ibis->SMPMadGetSetByLid(lid,
               IBIS_IB_MAD_METHOD_GET,
               AR_IB_ATTR_SMP_AR_INFO,
               0x0,
               &ar_info,
               (pack_data_func_t)adaptive_routing_info_pack,
               (unpack_data_func_t)adaptive_routing_info_unpack,
               (dump_data_func_t)adaptive_routing_info_dump);

        rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

        if (rc) {
            if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                    (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->GetLastError());
            else
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
            return 1;
        }
    } else {    // send by direct route
        // aquiring ARInfo for getting PSE bit
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
               IBIS_IB_MAD_METHOD_GET,
               AR_IB_ATTR_SMP_AR_INFO,
               0x0,
               &ar_info,
               (pack_data_func_t)adaptive_routing_info_pack,
               (unpack_data_func_t)adaptive_routing_info_unpack,
               (dump_data_func_t)adaptive_routing_info_dump);

        rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

        if (rc) {
            if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                    (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->GetLastError());
            else
                ERR_PRINT("Failed to get ARInfo, err=%s\n", p_ibis->ConvertMadStatusToStr(rc).c_str());
            return 1;
        }
    }
    return 0;
}

int SMPARQuery::RNSubGroupDirectionTableProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string optional_args) {

    //struct SMP_ARGroupTable ar_group_table;
    struct rn_sub_group_direction_tbl sub_group_direction_table;
    struct adaptive_routing_info ar_info;
    memset(&sub_group_direction_table, 0, sizeof(sub_group_direction_table));

    int rc = GetArInfo(p_ibis, lid, p_direct_route, ar_info);

    if (rc)
        return rc;

    if (!ar_info.is_arn_sup && !ar_info.is_frn_sup) {
        ERR_PRINT("RNSubGroupDirectionTable is not supported.\n");
        return 1;
    }

    int i = optional_args.find(' ');
    string block_num_str = optional_args.substr(0, i);
    u_int32_t block_num   = (u_int32_t)strtoul(block_num_str.c_str(), NULL, 0);

    u_int32_t top_block_num =
        ((ar_info.group_top + 1) * (ar_info.sub_grps_active + 1) - 1) /
        RN_SUB_GROUP_DIRECTION_BLOCK_SIZE;

    if (block_num > top_block_num) {
        ERR_PRINT("block number %u is greater than top block number %u\n",
                block_num, top_block_num);
        return 1;
    }

    //attr_mod[15:0] - block number
    //u_int32_t attr_mod = (0xff & block_num);

    if (lid) {  // send by lid
        INFO_PRINT("Getting RNSubGroupDirectionTable from lid=%u, block=%u \n",
                   lid, block_num);
        rc = p_ibis->SMPRNSubGroupDirectionTableGetSetByLid(
            lid, IBIS_IB_MAD_METHOD_GET, block_num,
            &sub_group_direction_table, NULL);

            /*
        rc = p_ibis->SMPMadGetSetByLid(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_RN_SUB_GROUP_DIRECTION_TABLE,
                attr_mod,
                &sub_group_direction_table,
                (pack_data_func_t)RNSubGroupDirectionTable_pack,
                (unpack_data_func_t)RNSubGroupDirectionTable_unpack,
                (dump_data_func_t)RNSubGroupDirectionTable_dump);
                */
    } else {    // send by direct route
        INFO_PRINT("Getting RNSubGroupDirectionTable from direct route=%s, block=%u\n",
                   p_ibis->ConvertDirPathToStr(p_direct_route).c_str(), block_num);

        rc = p_ibis->SMPRNSubGroupDirectionTableGetSetByDirect(
            p_direct_route, IBIS_IB_MAD_METHOD_GET, block_num,
            &sub_group_direction_table, NULL);

        /*
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_RN_SUB_GROUP_DIRECTION_TABLE,
                attr_mod,
                &sub_group_direction_table,
                (pack_data_func_t)RNSubGroupDirectionTable_pack,
                (unpack_data_func_t)RNSubGroupDirectionTable_unpack,
                (dump_data_func_t)RNSubGroupDirectionTable_dump);
                */
    }
    //rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get RNSubGroupDirectionTable, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get RNSubGroupDirectionTable, err=%s\n",
                    p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    unsigned group_num;
    unsigned sub_group_num;
    INFO_PRINT("RNSubGroupDirectionTable (Sub groups in group: %d):\n", ar_info.sub_grps_active+1 );
    for (unsigned group_entry = 0;
         group_entry < RN_SUB_GROUP_DIRECTION_BLOCK_SIZE; ++group_entry) {

        group_num =  (group_entry + block_num * RN_SUB_GROUP_DIRECTION_BLOCK_SIZE) /
            (ar_info.sub_grps_active+1);
        sub_group_num = (group_entry + block_num * RN_SUB_GROUP_DIRECTION_BLOCK_SIZE) %
            (ar_info.sub_grps_active+1);

        if (group_num > ar_info.group_top)
            break;

        printf("Group...%u\tSub Group...%u\tDirection...%u\n",
               group_num, sub_group_num,
               sub_group_direction_table.direction_of_sub_group[group_entry]);

    }
    return 0;
}

int SMPARQuery::RNGenStringTableProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string optional_args) {

    //struct SMP_ARGroupTable ar_group_table;
    struct rn_gen_string_tbl rn_gen_string_table;
    struct adaptive_routing_info ar_info;
    memset(&rn_gen_string_table, 0, sizeof(rn_gen_string_table));

    int rc = GetArInfo(p_ibis, lid, p_direct_route, ar_info);

    if (rc)
        return rc;

    if (!ar_info.is_arn_sup && !ar_info.is_frn_sup) {
        ERR_PRINT("RNGenStringTable is not supported.\n");
        return 1;
    }

    int i = optional_args.find(' ');
    string plft_id_str = optional_args.substr(0, i);
    string block_num_str = optional_args.erase(0, i + 1);
    u_int32_t block_num   = (u_int32_t)strtoul(block_num_str.c_str(), NULL, 0);
    u_int32_t plft_id = (u_int32_t)strtoul(plft_id_str.c_str(), NULL, 0);

    u_int32_t top_block_num = ar_info.direction_num_sup /
        RN_GEN_STRING_TABLE_BLOCK_SIZE;

    if (block_num > top_block_num) {
        ERR_PRINT("block number %u is greater than top block number %u\n",
                block_num, top_block_num);
        return 1;
    }

    //attr_mod[?:8] - block number
    //attr_mod[7:0] - privateLFTID
    //u_int32_t attr_mod = (0xf & plft_id);
    //attr_mod |= (0xf & block_num) << 8; //!!!

    if (lid) {  // send by lid
        INFO_PRINT("Getting RNGenStringTable from lid=%u, plft=%u,"
                   " block=%u \n",
                   lid, plft_id, block_num);
        rc = p_ibis->SMPRNGenStringTableGetSetByLid(lid,
                         IBIS_IB_MAD_METHOD_GET,
                         block_num,
                         plft_id,
                         &rn_gen_string_table,
                         NULL);

    } else {    // send by direct route
        INFO_PRINT("Getting RNGenStringTable from direct route=%s, plft=%u,"
                   " block=%u\n",
                   p_ibis->ConvertDirPathToStr(p_direct_route).c_str(),
                   plft_id, block_num);
        rc = p_ibis->SMPRNGenStringTableGetSetByDirect(p_direct_route,
                         IBIS_IB_MAD_METHOD_GET,
                         block_num,
                         plft_id,
                         &rn_gen_string_table,
                         NULL);

        /*
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_RN_GEN_STRING_TABLE,
                attr_mod,
                &rn_gen_string_table,
                (pack_data_func_t)RNGenStringTable_pack,
                (unpack_data_func_t)RNGenStringTable_unpack,
                (dump_data_func_t)RNGenStringTable_dump);
                */
    }
    //rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get RNGenStringTable, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get RNGenStringTable, err=%s\n",
                    p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    unsigned direction;
    for (unsigned block_entry = 0;
         block_entry < RN_GEN_STRING_TABLE_BLOCK_SIZE; ++block_entry) {

        direction =  (block_entry + block_num * RN_GEN_STRING_TABLE_BLOCK_SIZE);

        if (direction > ar_info.direction_num_sup)
            break;

        printf("Direction...%u\tGenerated String...%u\n",direction,
               rn_gen_string_table.element[block_entry].rn_gen_string);
    }
    return 0;
}

int SMPARQuery::RNGenBySubGroupPriorityProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string /*optional_args*/) {

    struct rn_gen_by_sub_group_prio rn_gen_by_sg_priority;
    memset(&rn_gen_by_sg_priority, 0, sizeof(rn_gen_by_sg_priority));
    struct adaptive_routing_info ar_info;

    int rc = GetArInfo(p_ibis, lid, p_direct_route, ar_info);

    if (rc)
        return rc;

    if (!ar_info.is_arn_sup && !ar_info.is_frn_sup) {
        ERR_PRINT("RNGenBySubGroupPriorityProcess is not supported.\n");
        return 1;
    }

    if (lid) {  // send by lid
        INFO_PRINT("Getting RNGenBySubGroupPriority from lid=%u\n",lid);

        rc = p_ibis->SMPRNGenBySubGroupPriorityMadGetSetByLid(
            lid,
            IBIS_IB_MAD_METHOD_GET,
            &rn_gen_by_sg_priority,
            NULL);

        /*
        rc = p_ibis->SMPMadGetSetByLid(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_RN_GEN_BY_SUB_GROUP_PRIORITY,
                0,
                &rn_gen_by_sg_priority,
                (pack_data_func_t)RNGenBySubGroupPriority_pack,
                (unpack_data_func_t)RNGenBySubGroupPriority_unpack,
                (dump_data_func_t)RNGenBySubGroupPriority_dump);
                */
    } else {    // send by direct route
        INFO_PRINT("Getting RNGenBySubGroupPriority from direct route=%s\n",
                   p_ibis->ConvertDirPathToStr(p_direct_route).c_str());

        rc = p_ibis->SMPRNGenBySubGroupPriorityMadGetSetByDirect(
            p_direct_route,
            IBIS_IB_MAD_METHOD_GET,
            &rn_gen_by_sg_priority,
            NULL);

        /*
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_RN_GEN_BY_SUB_GROUP_PRIORITY,
                0,
                &rn_gen_by_sg_priority,
                (pack_data_func_t)RNGenBySubGroupPriority_pack,
                (unpack_data_func_t)RNGenBySubGroupPriority_unpack,
                (dump_data_func_t)RNGenBySubGroupPriority_dump);
                */
    }
    //rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get RNGenBySubGroupPriority, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get RNGenBySubGroupPriority, err=%s\n",
                    p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    for (u_int8_t sg = 0;
         sg < ar_info.sub_grps_active + 1; ++sg) {

        printf("SubGroup Priority...%u\tgen_arn...%u\tgen_frn...%u\n",
               (int)sg,
               (int)rn_gen_by_sg_priority.element[sg].gen_arn,
               (int)rn_gen_by_sg_priority.element[sg].gen_frn);
    }
    return 0;
}

static const char *DecisionToStr(u_int8_t decision){
    switch (decision) {
        case 0:
            return "Discard";
        case 1:
            return "Consume ARN";
        case 2:
            return "Consume ARN/FRN";
        case 3:
            return "Pass-on";
    }

    return "Invalid";
}

int SMPARQuery::RNRcvStringProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string optional_args) {

    //struct SMP_ARGroupTable ar_group_table;
    struct rn_rcv_string rcv_string;
    struct adaptive_routing_info ar_info;
    memset(&rcv_string, 0, sizeof(rn_rcv_string));

    int rc = GetArInfo(p_ibis, lid, p_direct_route, ar_info);

    if (rc)
        return rc;

    if (!ar_info.is_arn_sup && !ar_info.is_frn_sup) {
        ERR_PRINT("RNRcvString is not supported.\n");
        return 1;
    }

    int i = optional_args.find(' ');
    string block_num_str = optional_args.substr(0, i);
    u_int32_t block_num   = (u_int32_t)strtoul(block_num_str.c_str(), NULL, 0);

    u_int32_t max_string = (0x1 << (ar_info.string_width_cap)) - 1;
    u_int32_t top_block_num = max_string /
        RN_RCV_STRING_BLOCK_SIZE;

    if (block_num > top_block_num) {
        ERR_PRINT("block number %u is greater than top block number %u\n",
                block_num, top_block_num);
        return 1;
    }

    //attr_mod[12:0] - block number
    //u_int32_t attr_mod = (0x1fff & block_num);

    if (lid) {  // send by lid
        INFO_PRINT("Getting RNRcvString from lid=%u, block=%u \n",
                   lid, block_num);
        rc = p_ibis->SMPRNRcvStringGetSetByLid(
            lid,
            IBIS_IB_MAD_METHOD_GET,
            block_num,
            &rcv_string,
            NULL);
        /*
        rc = p_ibis->SMPMadGetSetByLid(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_RN_RCV_STRING,
                attr_mod,
                &rcv_string,
                (pack_data_func_t)RNRcvString_pack,
                (unpack_data_func_t)RNRcvString_unpack,
                (dump_data_func_t)RNRcvString_dump);
                */
    } else {    // send by direct route
        INFO_PRINT("Getting RNRcvString from direct route=%s, block=%u\n",
                   p_ibis->ConvertDirPathToStr(p_direct_route).c_str(), block_num);
        rc = p_ibis->SMPRNRcvStringGetSetByDirect(
            p_direct_route,
            IBIS_IB_MAD_METHOD_GET,
            block_num,
            &rcv_string,
            NULL);

        /*
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_RN_RCV_STRING,
                attr_mod,
                &rcv_string,
                (pack_data_func_t)RNRcvString_pack,
                (unpack_data_func_t)RNRcvString_unpack,
                (dump_data_func_t)RNRcvString_dump);
                */
    }
    //rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get RNRcvString, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get RNRcvString, err=%s\n",
                    p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    unsigned string_num;
    for (unsigned block_entry = 0;
         block_entry < RN_SUB_GROUP_DIRECTION_BLOCK_SIZE; ++block_entry) {

        string_num =  (block_entry + block_num * RN_RCV_STRING_BLOCK_SIZE);

        if (string_num > max_string)
            break;

        printf("String...%u\tDecision...%s\tpLFT...%u\tString to string...%u\n",
               string_num,
               DecisionToStr(rcv_string.element[block_entry].decision),
               rcv_string.element[block_entry].plft_id,
               rcv_string.element[block_entry].string2string);
    }
    return 0;
}

int SMPARQuery::RNXmitPortMaskProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string optional_args) {

    //struct SMP_ARGroupTable ar_group_table;
    struct rn_xmit_port_mask xmit_port_mask;
    struct adaptive_routing_info ar_info;
    memset(&xmit_port_mask, 0, sizeof(rn_xmit_port_mask));

    int rc = GetArInfo(p_ibis, lid, p_direct_route, ar_info);

    if (rc)
        return rc;

    if (!ar_info.is_arn_sup && !ar_info.is_frn_sup) {
        ERR_PRINT("RNXmitPortMask is not supported.\n");
        return 1;
    }

    struct SMP_NodeInfo node_info;
    rc = GetNodeInfo(p_ibis, lid, p_direct_route, node_info);
    if (rc)
        return rc;

    int i = optional_args.find(' ');
    string block_num_str = optional_args.substr(0, i);
    u_int32_t block_num   = (u_int32_t)strtoul(block_num_str.c_str(), NULL, 0);

    u_int32_t top_block_num = node_info.NumPorts /
        RN_XMIT_PORT_MASK_BLOCK_SIZE;

    if (block_num > top_block_num) {
        ERR_PRINT("block number %u is greater than top block number %u\n",
                block_num, top_block_num);
        return 1;
    }

    //attr_mod[0] - block number
    //u_int32_t attr_mod = (0x1 & block_num);

    if (lid) {  // send by lid
        INFO_PRINT("Getting RNXmitPortMask from lid=%u, block=%u \n",
                   lid, block_num);
        rc = p_ibis->SMPRNXmitPortMaskGetSetByLid(
            lid,
            IBIS_IB_MAD_METHOD_GET,
            block_num,
            &xmit_port_mask,
            NULL);

        /*
        rc = p_ibis->SMPMadGetSetByLid(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_RN_XMIT_PORT_MASK,
                attr_mod,
                &xmit_port_mask,
                (pack_data_func_t)RNXmitPortMask_pack,
                (unpack_data_func_t)RNXmitPortMask_unpack,
                (dump_data_func_t)RNXmitPortMask_dump);
                */
    } else {    // send by direct route
        INFO_PRINT("Getting RNXmitPortMask from direct route=%s, block=%u\n",
                   p_ibis->ConvertDirPathToStr(p_direct_route).c_str(), block_num);

        rc = p_ibis->SMPRNXmitPortMaskGetSetByDirect(
            p_direct_route,
            IBIS_IB_MAD_METHOD_GET,
            block_num,
            &xmit_port_mask,
            NULL);

        /*
        rc = p_ibis->SMPMadGetSetByDirect(p_direct_route,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_SMP_RN_XMIT_PORT_MASK,
                attr_mod,
                &xmit_port_mask,
                (pack_data_func_t)RNXmitPortMask_pack,
                (unpack_data_func_t)RNXmitPortMask_unpack,
                (dump_data_func_t)RNXmitPortMask_dump);
                */
    }
    //rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here

    if (rc) {
        if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
            ERR_PRINT("Failed to get RNXmitPortMask, err=%s\n", p_ibis->GetLastError());
        else
            ERR_PRINT("Failed to get RNXmitPortMask, err=%s\n",
                    p_ibis->ConvertMadStatusToStr(rc).c_str());
        return 1;
    }

    unsigned port_num;
    for (unsigned block_entry = 0;
         block_entry < RN_XMIT_PORT_MASK_BLOCK_SIZE; ++block_entry) {

        port_num =  (block_entry + block_num * RN_XMIT_PORT_MASK_BLOCK_SIZE);

        if (port_num > node_info.NumPorts)
            break;

        u_int8_t element =
            xmit_port_mask.element[block_entry];

        printf("Port...%u\tGenerate ARN...%u\tGenerate FRN...%u\tPass on RN...%u\n",
               port_num,
               (bool)(element & RN_XMIT_PORT_MASK_GENERATE_ARN),
               (bool)(element & RN_XMIT_PORT_MASK_GENERATE_FRN),
               (bool)(element & RN_XMIT_PORT_MASK_PASS_ON_RN));
    }
    return 0;
}

int SMPARQuery::PortRNCountersProcess(Ibis *p_ibis,
        u_int16_t lid, direct_route_t *p_direct_route,
        string /*optional_args*/) {

    //struct SMP_ARGroupTable ar_group_table;
    struct port_rn_counters counters;
    struct adaptive_routing_info ar_info;
    memset(&counters, 0, sizeof(port_rn_counters));

    if (!lid) {
        ERR_PRINT("PortRNCounters supports only lid route.\n");
        return 1;
    }

    int rc = GetArInfo(p_ibis, lid, p_direct_route, ar_info);

    if (rc)
        return rc;

    if (!ar_info.is_arn_sup && !ar_info.is_frn_sup) {
        ERR_PRINT("PortRNCounters is not supported.\n");
        return 1;
    }

    struct SMP_NodeInfo node_info;
    rc = GetNodeInfo(p_ibis, lid, p_direct_route, node_info);
    if (rc)
        return rc;

    if (lid)
        INFO_PRINT("Getting PortRNCounters from lid=%u\n",lid);
    else
        INFO_PRINT("Getting PortRNCounters from direct route=%s\n",
                   p_ibis->ConvertDirPathToStr(p_direct_route).c_str());

    for (u_int8_t port_num = 1;
         port_num <=  node_info.NumPorts;
         ++port_num) {

        //counters.port_select = port_num;

        rc = p_ibis->VSPortRNCountersGet(lid, port_num, &counters, NULL);

        /*
        rc = p_ibis->VSMadGetSet(lid,
                IBIS_IB_MAD_METHOD_GET,
                AR_IB_ATTR_VS_PORT_RN_COUNTERS,
                0x0,
                &counters,
                (pack_data_func_t)PortRNCounters_pack,
                (unpack_data_func_t)PortRNCounters_unpack,
                (dump_data_func_t)PortRNCounters_dump);


        rc = rc & 0x00ff;   //bits [8-15] are class specific - don't need them here
    */

        if (rc) {
            if ((rc == IBIS_MAD_STATUS_GENERAL_ERR) || (rc == IBIS_MAD_STATUS_TIMEOUT) ||
                    (rc == IBIS_MAD_STATUS_RECV_FAILED) || (rc == IBIS_MAD_STATUS_SEND_FAILED))
                ERR_PRINT("Failed to get PortRNCounters, port:%u err=%s\n",
                          port_num, p_ibis->GetLastError());
            else
                ERR_PRINT("Failed to get PortRNCounters, port:%u err=%s\n",
                        port_num, p_ibis->ConvertMadStatusToStr(rc).c_str());
            return 1;
        }


        printf("Port...%u\trcv rn pkt..." U64H_FMT
               "\tXmit rn pkt..." U64H_FMT
               "\tRcv rn error..." U64H_FMT
               "\tRcv switch relay rn error..." U64H_FMT
               "\n",
               port_num,
               counters.port_rcv_rn_pkt,
               counters.port_xmit_rn_pkt,
               counters.port_rcv_rn_error,
               counters.port_rcv_switch_relay_rn_error);
    }
    return 0;
}

/****************************************************/
int main(int argc, char **argv)
{
    SMPARQuery smparquery_obj;

    // check if init was done
    if (smparquery_obj.IsInitialized() == false)
        return 1;

    // get args
    int rc = smparquery_obj.GetOpt(argc, argv);
    if (rc) {
        if (rc == 2)    //2 - OK but need to exit
            return 0;
        return 1;
    }

    // run
    rc = smparquery_obj.Run();

    return rc;
}

