/*                  - Mellanox Confidential and Proprietary -
 *
 *  Copyright (C) 2010-2011, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 *  Except as specifically permitted herein, no portion of the information,
 *  including but not limited to object code and source code, may be reproduced,
 *  modified, distributed, republished or otherwise exploited in any form or by
 *  any means for any purpose without the prior written permission of Mellanox
 *  Technologies Ltd. Use of software subject to the terms and conditions
 *  detailed in the file "COPYING".
 *
 */


#ifndef SMP_AR_QUERY_H_
#define SMP_AR_QUERY_H_


#include <errno.h>
#include <stdlib.h>

#include <infiniband/misc/cmdparser/cmdparser.h>
#include <infiniband/ibis/ibis.h>

#include "smparquery_types.h"
#include "../ar_mgr_core/ar_mgr_types.h"


/****************************************************/
class SMPARQuery : public CommandLineRequester {
private:
    ////////////////////
    //members
    ////////////////////
    bool m_initialized;
    CommandLineParser *m_p_main_parser;
    Ibis m_ibis_obj;

    string m_ca_name;
    u_int8_t m_port;
    bool m_to_use_dr;
    bool m_to_use_lid;

    string m_operation;
    string m_dest;
    string m_optional_args;

    ////////////////////
    //methods
    ////////////////////
    int ConvertStrToDR(direct_route_t *p_dr, string routep_path);
    void AddGeneralOptions();
    void Usage();

    //Returns: 0 - OK / 1 - parsing error / 2 - OK but need to exit
    int HandleOption(string name, string value);

public:
    ////////////////////
    //methods
    ////////////////////
    SMPARQuery();
    ~SMPARQuery();

    inline bool IsInitialized() { return m_initialized; }

    //Returns: 0 - OK / 1 - error / 2 - OK but need to exit
    int GetOpt(int argc, char **argv);

    static int GetNodeDevId(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            u_int16_t &dev_id);

    static int  GetNodeInfo(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            struct SMP_NodeInfo &node_info);

    static int GetArInfo(Ibis *p_ibis, u_int16_t lid,
            direct_route_t *p_direct_route,
            struct adaptive_routing_info &ar_info);

    static int PLFTInfoProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);

    static int PLFTDefProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);

    static int PortSLToPLFTMapProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);

    static int ARInfoProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);
    static int ARGroupTableProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);
    static int ARLFTTableProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);

    //Unused all devices considered SX
    static int GetLFTTable(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            u_int32_t plft_id, u_int32_t block_num);
    static int GetLFTTable_SX(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            u_int32_t plft_id, u_int32_t block_num);

    static int RNSubGroupDirectionTableProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);

    static int RNGenStringTableProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);

    static int RNGenBySubGroupPriorityProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);

    static int RNRcvStringProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);

    static int RNXmitPortMaskProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);

    static int PortRNCountersProcess(Ibis *p_ibis,
            u_int16_t lid, direct_route_t *p_direct_route,
            string optional_args);

    int Run();
};


#endif  /* if define SMP_AR_QUERY_H_ */

